@@ -87,18 +87,18 @@ struct _RBDAAPSrc
 	guint32 bytes_per_read;
 	gboolean chunked;
 	gboolean first_chunk;
-	
+
 	gint64 size;
-	
+
 	/* Seek stuff */
 	gint64 curoffset;
-	glong seek_time;
 	gint64 seek_bytes;
-	glong seek_time_to_return;
 	gboolean do_seek;
 #ifdef HAVE_GSTREAMER_0_8
 	gboolean need_flush;
 	gboolean send_discont;
+	glong seek_time_to_return;
+	glong seek_time;
 #endif
 };
 
@@ -137,8 +137,8 @@ _do_init (GType daap_src_type)
 		NULL,
 		NULL
 	};
-	GST_DEBUG_CATEGORY_INIT (rb_daap_src_debug, 
-				 "daapsrc", GST_DEBUG_FG_WHITE, 
+	GST_DEBUG_CATEGORY_INIT (rb_daap_src_debug,
+				 "daapsrc", GST_DEBUG_FG_WHITE,
 				 "Rhythmbox built in DAAP source element");
 
 	g_type_add_interface_static (daap_src_type, GST_TYPE_URI_HANDLER,
@@ -146,7 +146,6 @@ _do_init (GType daap_src_type)
 }
 
 #ifdef HAVE_GSTREAMER_0_8
-GType rb_daap_src_get_type (void);
 GST_BOILERPLATE_FULL (RBDAAPSrc, rb_daap_src, GstElement, GST_TYPE_ELEMENT, _do_init);
 #else
 GST_BOILERPLATE_FULL (RBDAAPSrc, rb_daap_src, GstElement, GST_TYPE_PUSH_SRC, _do_init);
@@ -182,10 +181,10 @@ static gboolean rb_daap_src_start (GstBaseSrc *bsrc);
 static gboolean rb_daap_src_stop (GstBaseSrc *bsrc);
 static gboolean rb_daap_src_is_seekable (GstBaseSrc *bsrc);
 static gboolean rb_daap_src_get_size (GstBaseSrc *src, guint64 *size);
+static gboolean rb_daap_src_do_seek (GstBaseSrc *src, GstSegment *segment);
 static GstFlowReturn rb_daap_src_create (GstPushSrc *psrc, GstBuffer **outbuf);
 #endif
 
-
 #ifdef HAVE_GSTREAMER_0_8
 
 static const GstFormat *
@@ -234,7 +233,7 @@ enum
 	PROP_BYTESPERREAD
 };
 
-static void 
+static void
 rb_daap_src_base_init (gpointer g_class)
 {
 	GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
@@ -245,7 +244,7 @@ rb_daap_src_base_init (gpointer g_class)
 	gst_element_class_set_details (element_class, &rb_daap_src_details);
 }
 
-static void 
+static void
 rb_daap_src_class_init (RBDAAPSrcClass *klass)
 {
 #ifdef HAVE_GSTREAMER_0_8
@@ -293,7 +292,7 @@ rb_daap_src_class_init (RBDAAPSrcClass *klass)
 	gobject_class->finalize = rb_daap_src_finalize;
 
 	g_object_class_install_property (gobject_class, PROP_LOCATION,
-			g_param_spec_string ("location", 
+			g_param_spec_string ("location",
 					     "file location",
 					     "location of the file to read",
 					     NULL,
@@ -303,12 +302,12 @@ rb_daap_src_class_init (RBDAAPSrcClass *klass)
 	gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (rb_daap_src_stop);
 	gstbasesrc_class->is_seekable = GST_DEBUG_FUNCPTR (rb_daap_src_is_seekable);
 	gstbasesrc_class->get_size = GST_DEBUG_FUNCPTR (rb_daap_src_get_size);
+	gstbasesrc_class->do_seek = GST_DEBUG_FUNCPTR (rb_daap_src_do_seek);
 
 	gstpushsrc_class->create = GST_DEBUG_FUNCPTR (rb_daap_src_create);
 #endif
 }
 
-
 #ifdef HAVE_GSTREAMER_0_8
 static void
 rb_daap_src_init (RBDAAPSrc *src)
@@ -352,13 +351,13 @@ rb_daap_src_finalize (GObject *object)
 {
 	RBDAAPSrc *src;
 	src = RB_DAAP_SRC (object);
-	
+
 #ifdef HAVE_GSTREAMER_0_8
 	if (GST_FLAG_IS_SET (src, RB_DAAP_SRC_OPEN)) {
 		rb_daap_src_close_file (src);
 	}
 #endif
-	
+
 	g_free (src->daap_uri);
 	src->daap_uri = NULL;
 
@@ -368,7 +367,6 @@ rb_daap_src_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-
 static void
 rb_daap_src_set_property (GObject *object,
 			  guint prop_id,
@@ -504,7 +502,7 @@ _expect_char (RBDAAPSrc *src, guchar expected)
 	return TRUE;
 }
 
-static gboolean 
+static gboolean
 rb_daap_src_read_chunk_size (RBDAAPSrc *src, gboolean first_chunk, gint64 *chunk_size)
 {
 	gchar chunk_buf[30];
@@ -570,7 +568,7 @@ _split_uri (const gchar *daap_uri, gchar **host, guint *port, gchar **path)
 		*path = g_strdup ("/");
 		pathstart = daap_uri + locationlen;
 	}
-	
+
 	portstart = strrchr (hostport, ':');
 	if (portstart) {
 		*host = g_strndup (hostport, portstart - hostport);
@@ -581,61 +579,6 @@ _split_uri (const gchar *daap_uri, gchar **host, guint *port, gchar **path)
 	}
 }
 
-
-#ifdef HAVE_GSTREAMER_0_8
-/* I tell you, it'd be nice if Gstreamer's typefind element actually listened
- * to this stuff, then it wouldnt have to do all that seeking and testing
- * the streams against the typefind functions.  Here I am, /telling/ it what
- * sort of stream it is, and it can't even listen.  Bah.
- */
-static GstCaps *
-rb_daap_src_getcaps (GstPad *pad)
-{
-	RBDAAPSrc *src = NULL;
-	const gchar *extension = NULL;
-	static GstStaticCaps mp3_caps = GST_STATIC_CAPS ("audio/mpeg, mpegversion = (int) 1, layer = (int) [ 1, 3 ]");
-	static GstStaticCaps ogg_caps = GST_STATIC_CAPS ("application/ogg");
-	static GstStaticCaps wav_caps = GST_STATIC_CAPS ("audio/x-wav");
-	static GstStaticCaps m4a_caps = GST_STATIC_CAPS ("audio/x-m4a");
-	static GstStaticCaps aac_caps = GST_STATIC_CAPS ("audio/mpeg, mpegversion = (int) { 2, 4 }, framed = (bool) false");
-
-	src = RB_DAAP_SRC (GST_OBJECT_PARENT (pad));
-
-	if (src->daap_uri == NULL) {
-		return gst_caps_new_any ();
-	}
-
-	extension = strrchr (src->daap_uri, '.');
-	extension++;
-
-	if ((g_strncasecmp (extension, "mp3", 3) == 0) ||
-	    (g_strncasecmp (extension, "mp2", 3) == 0) ||
-	    (g_strncasecmp (extension, "mp1", 3) == 0) ||
-	    (g_strncasecmp (extension, "mpga", 4) == 0)) {
-		return gst_caps_copy (gst_static_caps_get(&mp3_caps));
-	}
-
-	if ((g_strncasecmp (extension, "ogg", 3) == 0) ||
-	    (g_strncasecmp (extension, "oggm", 4) == 0)) {
-		return gst_caps_copy (gst_static_caps_get(&ogg_caps));
-	}
-
-	if ((g_strncasecmp (extension, "wav", 3) == 0)) {
-		return gst_caps_copy (gst_static_caps_get(&wav_caps));
-	}
-
-	if ((g_strncasecmp (extension, "m4a", 3) == 0)) {
-		return gst_caps_copy (gst_static_caps_get(&m4a_caps));
-	}
-
-	if ((g_strncasecmp (extension, "aac", 3) == 0)) {
-		return gst_caps_copy (gst_static_caps_get(&aac_caps));
-	}
-	
-	return gst_caps_new_any ();
-}
-#endif
-
 static gboolean
 rb_daap_src_open (RBDAAPSrc *src)
 {
@@ -672,7 +615,7 @@ rb_daap_src_open (RBDAAPSrc *src)
 	}
 
 	_split_uri (src->daap_uri, &host, &port, &path);
-	
+
 	server.sin_family = AF_INET;
 	server.sin_port = htons (port);
 	server.sin_addr.s_addr = inet_addr (host);
@@ -703,7 +646,11 @@ rb_daap_src_open (RBDAAPSrc *src)
 	}
 
 	/* The following can fail if the source is no longer connected */
+#ifdef HAVE_GSTREAMER_0_8
 	headers = rb_daap_source_get_headers (source, src->daap_uri, src->seek_time, &src->seek_bytes);
+#else
+	headers = rb_daap_source_get_headers (source, src->daap_uri, src->seek_bytes);
+#endif
 	if (headers == NULL) {
 		g_free (host);
 		g_free (path);
@@ -740,7 +687,7 @@ rb_daap_src_open (RBDAAPSrc *src)
 
 	end_headers = strstr (response, "\r\n\r\n");
 	if (!end_headers) {
-		/* this means the DAAP server returned more than 4k of headers.  
+		/* this means the DAAP server returned more than 4k of headers.
 		 * not terribly likely.
 		 */
 		g_free (response);
@@ -785,10 +732,10 @@ rb_daap_src_open (RBDAAPSrc *src)
 					}
 				}
 			}
-			
+
 		} else {
-			GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, 
-					   ("HTTP error: %s", http_status_phrase), 
+			GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+					   ("HTTP error: %s", http_status_phrase),
 					   (NULL));
 			ok = FALSE;
 		}
@@ -824,13 +771,13 @@ rb_daap_src_start (GstBaseSrc *bsrc)
 	if (src->sock_fd != -1) {
 		close (src->sock_fd);
 	}
-	
+
 	src->curoffset = 0;
 
 	if (rb_daap_src_open (src)) {
 		src->buffer = src->buffer_base;
-		src->seek_time_to_return = src->seek_time;
 #ifdef HAVE_GSTREAMER_0_8
+		src->seek_time_to_return = src->seek_time;
 		if (src->seek_bytes != 0) {
 			src->need_flush = TRUE;
 			src->send_discont = TRUE;
@@ -848,7 +795,7 @@ rb_daap_src_start (GstBaseSrc *bsrc)
 		return FALSE;
 	}
 }
-	
+
 #ifdef HAVE_GSTREAMER_0_8
 static void
 rb_daap_src_close_file (RBDAAPSrc *src)
@@ -868,7 +815,7 @@ rb_daap_src_close_file (RBDAAPSrc *src)
 static gboolean
 rb_daap_src_stop (GstBaseSrc *bsrc)
 {
-	/* don't do anything - this seems to get called during setup, but 
+	/* don't do anything - this seems to get called during setup, but
 	 * we don't get started again afterwards.
 	 */
 	return TRUE;
@@ -927,7 +874,7 @@ rb_daap_src_create (GstPushSrc *psrc, GstBuffer **outbuf)
 		src->need_flush = FALSE;
 		return GST_DATA (event);
 	}
-	
+
 	if (src->send_discont) {
 		GstEvent *event;
 
@@ -956,11 +903,11 @@ rb_daap_src_create (GstPushSrc *psrc, GstBuffer **outbuf)
 		}
 		src->first_chunk = FALSE;
 	}
-	
+
 	readsize = src->bytes_per_read;
 	if (src->chunked && readsize > src->size)
 		readsize = src->size;
-	
+
 #ifdef HAVE_GSTREAMER_0_8
 	buf = gst_buffer_new ();
 	g_return_val_if_fail (buf, NULL);
@@ -992,7 +939,7 @@ rb_daap_src_create (GstPushSrc *psrc, GstBuffer **outbuf)
 		return GST_FLOW_UNEXPECTED;
 #endif
 	}
-	
+
 	if (src->chunked)
 		src->size -= readsize;
 
@@ -1048,7 +995,6 @@ rb_daap_src_change_state (GstElement *element)
 	return GST_STATE_SUCCESS;
 }
 
-			  
 static gboolean
 rb_daap_src_srcpad_event (GstPad *pad,
 			  GstEvent *event)
@@ -1107,10 +1053,10 @@ rb_daap_src_srcpad_event (GstPad *pad,
 	return TRUE;
 }
 
-static gboolean 
-rb_daap_src_srcpad_query (GstPad *pad, 
-			  GstQueryType type, 
-			  GstFormat *format, 
+static gboolean
+rb_daap_src_srcpad_query (GstPad *pad,
+			  GstQueryType type,
+			  GstFormat *format,
 			  gint64 *value)
 {
 	RBDAAPSrc *src = RB_DAAP_SRC (gst_pad_get_parent (pad));
@@ -1142,7 +1088,7 @@ rb_daap_src_srcpad_query (GstPad *pad,
 			return FALSE;
 			break;
 	}
-	
+
 	return TRUE;
 }
 #else
@@ -1150,7 +1096,20 @@ rb_daap_src_srcpad_query (GstPad *pad,
 gboolean
 rb_daap_src_is_seekable (GstBaseSrc *bsrc)
 {
-	return FALSE;
+	return TRUE;
+}
+
+gboolean
+rb_daap_src_do_seek (GstBaseSrc *bsrc, GstSegment *segment)
+{
+	RBDAAPSrc *src = RB_DAAP_SRC (bsrc);
+	if (segment->format == GST_FORMAT_BYTES) {
+		src->do_seek = TRUE;
+		src->seek_bytes = segment->start;
+		return TRUE;
+	} else {
+		return FALSE;
+	}
 }
 
 gboolean
@@ -1170,7 +1129,6 @@ static gboolean
 plugin_init (GstPlugin *plugin)
 {
 	gboolean ret = gst_element_register (plugin, "rbdaapsrc", GST_RANK_PRIMARY, RB_TYPE_DAAP_SRC);
-
 	return ret;
 }
 
@@ -1184,6 +1142,7 @@ GST_PLUGIN_DEFINE_STATIC (GST_VERSION_MAJOR,
 			  PACKAGE,
 			  "");
 
+#ifdef HAVE_GSTREAMER_0_8
 /*** RB DAAP SEEK INTERFACE **************************************************/
 
 void
@@ -1201,6 +1160,7 @@ rb_daap_src_get_time (GstElement *element)
 	return src->seek_time_to_return;
 }
 
+#endif
 /*** GSTURIHANDLER INTERFACE *************************************************/
 
 static guint
@@ -1226,7 +1186,7 @@ rb_daap_src_uri_get_uri (GstURIHandler *handler)
 }
 
 static gboolean
-rb_daap_src_uri_set_uri (GstURIHandler *handler, 
+rb_daap_src_uri_set_uri (GstURIHandler *handler,
 			 const gchar *uri)
 {
 	RBDAAPSrc *src = RB_DAAP_SRC (handler);
@@ -1240,9 +1200,8 @@ rb_daap_src_uri_set_uri (GstURIHandler *handler,
 	return TRUE;
 }
 
-
-static void 
-rb_daap_src_uri_handler_init (gpointer g_iface, 
+static void
+rb_daap_src_uri_handler_init (gpointer g_iface,
 			      gpointer iface_data)
 {
 	GstURIHandlerInterface *iface = (GstURIHandlerInterface *) g_iface;
