@@ -1,7 +1,8 @@
-/*
- *  arch-tag: Header files for new internet radio station dialog
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- *  Copyright (C) 2002, 2003 Colin Walters <walters@debian.org>
+ *  arch-tag: Header files for new station dialog
+ *
+ *  Copyright (C) 2005 Renato Araujo Oliveira Filho <renato.filho@indt.org.br>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,12 +16,12 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
 #include <gtk/gtkdialog.h>
-#include "rhythmdb.h"
+#include "rb-entry-view.h"
 
 #ifndef __RB_NEW_STATION_DIALOG_H
 #define __RB_NEW_STATION_DIALOG_H
@@ -46,11 +47,13 @@ typedef struct
 typedef struct
 {
 	GtkDialogClass parent_class;
+
+        void (*location_added) (RBNewStationDialog *dialog,
+				const char         *uri);
 } RBNewStationDialogClass;
 
 GType      rb_new_station_dialog_get_type (void);
-
-GtkWidget *rb_new_station_dialog_new      (RhythmDB *db);
+GtkWidget* rb_new_station_dialog_new      (RBEntryView *view);
 
 G_END_DECLS
 
