@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -25,7 +25,7 @@
 #include <string.h>
 
 #include <glib/gi18n.h>
-#include <gst/gst.h>
+#include <gst/tag/tag.h>
 #include <gst/gsturi.h>
 
 #ifdef HAVE_GSTREAMER_0_8
@@ -33,6 +33,7 @@
 #define gst_caps_unref gst_caps_free
 #define gst_tag_setter_add_tag_values gst_tag_setter_add_values
 #define gst_tag_setter_set_tag_merge_mode gst_tag_setter_set_merge_mode
+#define gst_tag_setter_merge_tags gst_tag_setter_merge
 #endif
 #ifdef HAVE_GSTREAMER_0_10
 #include <gst/gsttagsetter.h>
@@ -47,8 +48,7 @@ G_DEFINE_TYPE(RBMetaData, rb_metadata, G_TYPE_OBJECT)
 
 static void rb_metadata_finalize (GObject *object);
 
-typedef GstElement *(*RBAddTaggerElem) (GstBin *, GstElement *);
-
+typedef GstElement *(*RBAddTaggerElem) (RBMetaData *, GstElement *);
 
 /*
  * The list of mine type prefixes for files that shouldn't display errors about being non-audio.
@@ -57,7 +57,8 @@ typedef GstElement *(*RBAddTaggerElem) (GstBin *, GstElement *);
 const char * ignore_mime_types[] = {
 	"image/",
 	"text/",
-	"application/xml"
+	"application/xml",
+	"application/zip"
 };
 
 /*
@@ -77,12 +78,13 @@ struct RBMetadataGstType
 struct RBMetaDataPrivate
 {
 	char *uri;
-  
+
 	GHashTable *metadata;
 
 	GstElement *pipeline;
 	GstElement *sink;
 	gulong typefind_cb_id;
+	GstTagList *tags;
 
 	/* Array of RBMetadataGstType */
 	GPtrArray *supported_types;
@@ -132,22 +134,22 @@ rb_metadata_class_init (RBMetaDataClass *klass)
 #endif
 }
 
-
 static GstElement *
-rb_add_flac_tagger (GstBin *pipeline, GstElement *element)
+rb_add_flac_tagger (RBMetaData *md, GstElement *element)
 {
 	GstElement *tagger = NULL;
 
 	if (!(tagger = gst_element_factory_make ("flactag", "flactag")))
 		return NULL;
 
-	gst_bin_add (GST_BIN (pipeline), tagger);
+	gst_bin_add (GST_BIN (md->priv->pipeline), tagger);
 	gst_element_link_many (element, tagger, NULL);
 
+	gst_tag_setter_merge_tags (GST_TAG_SETTER (tagger), md->priv->tags, GST_TAG_MERGE_REPLACE_ALL);
+
 	return tagger;
 }
 
-
 #ifdef HAVE_GSTREAMER_0_10
 static void
 id3_pad_added_cb (GstElement *demux, GstPad *pad, GstElement *mux)
@@ -161,24 +163,71 @@ id3_pad_added_cb (GstElement *demux, GstPad *pad, GstElement *mux)
 		rb_debug ("linked pad from id3de to id3mux");
 }
 
+static gboolean
+rb_gst_plugin_greater (const char *plugin, const char *element, gint major, gint minor, gint micro)
+{
+	const char *version;
+	GstPlugin *p;
+	guint i;
+	guint count;
+
+	if (gst_default_registry_check_feature_version (element, major, minor, micro + 1))
+		return TRUE;
+
+	if (!gst_default_registry_check_feature_version (element, major, minor, micro))
+		return FALSE;
+
+	p = gst_default_registry_find_plugin (plugin);
+	if (p == NULL)
+		return FALSE;
+
+	version = gst_plugin_get_version (p);
+
+	/* check if it's not a release */
+	count = sscanf (version, "%u.%u.%u.%u", &i, &i, &i, &i);
+	return (count > 3);
+}
+
 static GstElement *
-rb_add_id3_tagger (GstBin *pipeline, GstElement *element)
+rb_add_id3_tagger (RBMetaData *md, GstElement *element)
 {
 	GstElement *demux = NULL;
 	GstElement *mux = NULL;
 
 	demux = gst_element_factory_make ("id3demux", NULL);
-	mux =  gst_element_factory_make ("id3mux", NULL);
+
+	mux = gst_element_factory_make ("id3v2mux", NULL);
+	if (mux != NULL) {
+		/* check for backwards id3v2mux merge-mode */
+		if (!rb_gst_plugin_greater ("taglib", "id3v2mux", 0, 10, 3)) {
+			rb_debug ("using id3v2mux with backwards merge mode");
+			gst_tag_setter_set_tag_merge_mode (GST_TAG_SETTER (mux), GST_TAG_MERGE_REPLACE);
+		} else {
+			rb_debug ("using id3v2mux");
+		}
+	} else {
+		mux =  gst_element_factory_make ("id3mux", NULL);
+
+		/* check for backwards id3mux merge-mode */
+		if (mux && !rb_gst_plugin_greater ("mad", "id3mux", 0, 10, 3)) {
+			rb_debug ("using id3mux with backwards merge mode");
+			gst_tag_setter_set_tag_merge_mode (GST_TAG_SETTER (mux), GST_TAG_MERGE_REPLACE);
+		} else {
+			rb_debug ("using id3mux");
+		}
+	}
 
 	if (demux == NULL || mux == NULL)
 		goto error;
 
-	gst_bin_add_many (pipeline, demux, mux, NULL);
+	gst_bin_add_many (GST_BIN (md->priv->pipeline), demux, mux, NULL);
 	if (!gst_element_link (element, demux))
 		goto error;
 
 	g_signal_connect (demux, "pad-added", (GCallback)id3_pad_added_cb, mux);
 
+	gst_tag_setter_merge_tags (GST_TAG_SETTER (mux), md->priv->tags, GST_TAG_MERGE_REPLACE_ALL);
+
 	return mux;
 
 error:
@@ -186,14 +235,101 @@ error:
 	g_object_unref (mux);
 	return NULL;
 }
+
+static void
+ogg_pad_added_cb (GstElement *demux, GstPad *pad, RBMetaData *md)
+{
+	GstCaps *caps;
+	GstStructure *structure;
+	const gchar *mimetype;
+	GstPad *conn_pad = NULL;
+	GstElement *mux;
+
+	caps = gst_pad_get_caps (pad);
+	structure = gst_caps_get_structure (caps, 0);
+	mimetype = gst_structure_get_name (structure);
+
+	mux = g_object_get_data (G_OBJECT (demux), "mux");
+
+	if (strcmp (mimetype, "audio/x-vorbis") == 0) {
+		GstElement *tagger, *parser;
+		GstBin *bin;
+		GstState state;
+
+		rb_debug ("found vorbis stream in ogg container, using vorbistag");
+
+		parser = gst_element_factory_make ("vorbisparse", NULL);
+		if (parser == NULL) {
+			rb_debug ("could not create vorbisparse element");
+			goto end;
+		}
+
+		tagger = gst_element_factory_make ("vorbistag", NULL);
+		if (tagger == NULL) {
+			rb_debug ("could not create vorbistag element");
+			gst_object_unref (parser);
+			goto end;
+		}
+
+		bin = GST_BIN (gst_element_get_parent (mux));
+		gst_bin_add_many (bin, tagger, parser, NULL);
+		gst_object_unref (GST_OBJECT (bin));
+
+		/* connect and bring them up to the same state */
+		gst_element_link_many (tagger, parser, mux, NULL);
+		gst_element_get_state (mux, &state, NULL, 0);
+		gst_element_set_state (parser, state);
+		gst_element_set_state (tagger, state);
+
+		conn_pad = gst_element_get_compatible_pad (tagger, pad, NULL);
+		gst_pad_link (pad, conn_pad);
+
+		gst_tag_setter_merge_tags (GST_TAG_SETTER (tagger), md->priv->tags, GST_TAG_MERGE_REPLACE_ALL);
+	} else {
+		conn_pad = gst_element_get_compatible_pad (mux, pad, NULL);
+		gst_pad_link (pad, conn_pad);
+		rb_debug ("found stream in ogg container with no known tagging element");
+	}
+
+end:
+	gst_caps_unref (caps);
+}
+
+static GstElement *
+rb_add_ogg_tagger (RBMetaData *md, GstElement *element)
+{
+	GstElement *demux = NULL;
+	GstElement *mux = NULL;
+
+	demux = gst_element_factory_make ("oggdemux", NULL);
+	mux =  gst_element_factory_make ("oggmux", NULL);
+
+	if (demux == NULL || mux == NULL)
+		goto error;
+
+	gst_bin_add_many (GST_BIN (md->priv->pipeline), demux, mux, NULL);
+	if (!gst_element_link (element, demux))
+		goto error;
+
+	g_object_set_data (G_OBJECT (demux), "mux", mux);
+	g_signal_connect (demux, "pad-added", (GCallback)ogg_pad_added_cb, md);
+
+	return mux;
+
+error:
+	g_object_unref (demux);
+	g_object_unref (mux);
+	return NULL;
+}
+
 #elif HAVE_GSTREAMER_0_8
 static GstElement *
-rb_add_id3_tagger (GstBin *pipeline, GstElement *element)
+rb_add_id3_tagger (RBMetaData *md, GstElement *element)
 {
 	GstElement *tagger, *identity;
 	GstCaps *filtercaps = NULL;
 
-	if (!(tagger = gst_element_factory_make ("id3tag", "tagger"))) 
+	if (!(tagger = gst_element_factory_make ("id3tag", "tagger")))
 		return NULL;
 
 	if (!(identity = gst_element_factory_make ("identity", "identity"))) {
@@ -201,7 +337,7 @@ rb_add_id3_tagger (GstBin *pipeline, GstElement *element)
 		return NULL;
 	}
 
-	gst_bin_add_many (GST_BIN (pipeline), tagger, identity, NULL);
+	gst_bin_add_many (GST_BIN (md->priv->pipeline), tagger, identity, NULL);
 
 	filtercaps = gst_caps_new_simple ("application/x-id3", NULL);
 	if (!gst_element_link_filtered (tagger, identity, filtercaps)) {
@@ -215,6 +351,8 @@ rb_add_id3_tagger (GstBin *pipeline, GstElement *element)
 
 	gst_element_link_many(element, tagger, NULL);
 
+	gst_tag_setter_merge_tags (GST_TAG_SETTER (tagger), md->priv->tags, GST_TAG_MERGE_REPLACE_ALL);
+
 	return identity;
 }
 #endif
@@ -236,27 +374,28 @@ static void
 rb_metadata_init (RBMetaData *md)
 {
 	RBAddTaggerElem tagger;
-	gboolean has_gnomevfssink;
-	gboolean has_id3;
-        
+	gboolean has_gnomevfssink = FALSE;
+	gboolean has_id3 = FALSE;
+
 	md->priv = RB_METADATA_GET_PRIVATE (md);
 
 	md->priv->supported_types = g_ptr_array_new ();
-	
+
  	/* the list of supported types serves two purposes:
  	 * - it knows how to construct elements for tag writing
  	 * - it knows human-readable names for MIME types so we can say
  	 *     "There is no plugin available to play WMV files"
  	 *     rather than " .. play video/x-ms-asf files".
  	 *
- 	 * only registering types we have plugins for defeats the second 
+ 	 * only registering types we have plugins for defeats the second
 	 * purpose.
  	 */
 	has_gnomevfssink = (gst_element_factory_find ("gnomevfssrc") != NULL &&
 			    gst_element_factory_find ("gnomevfssink") != NULL);
 #ifdef HAVE_GSTREAMER_0_10
-	has_id3 = (gst_default_registry_check_feature_version ("id3mux", 0, 10, 2) &&
-		   gst_element_factory_find ("id3demux") != NULL);
+	has_id3 = (gst_element_factory_find ("id3demux") != NULL) &&
+			(gst_default_registry_check_feature_version ("id3mux", 0, 10, 2) ||
+			 (gst_element_factory_find ("id3v2mux") != NULL));
 #elif HAVE_GSTREAMER_0_8
 	has_id3 = (gst_element_factory_find ("id3tag") != NULL);
 #endif
@@ -265,8 +404,21 @@ rb_metadata_init (RBMetaData *md)
 	add_supported_type (md, "application/x-id3", tagger, "MP3");
 	add_supported_type (md, "audio/mpeg", tagger, "MP3");
 
-	add_supported_type (md, "application/ogg", NULL, "Ogg Vorbis");
-	add_supported_type (md, "audio/x-vorbis", NULL, "Ogg Vorbis");
+#ifdef HAVE_GSTREAMER_0_10
+	{
+		gboolean has_vorbis;
+
+		has_vorbis = ((gst_element_factory_find ("vorbistag") != NULL) &&
+			      gst_default_registry_check_feature_version ("vorbisparse", 0, 10, 6) &&
+			      gst_default_registry_check_feature_version ("oggmux", 0, 10, 6) &&
+			      gst_default_registry_check_feature_version ("oggdemux", 0, 10, 6));
+		tagger = (has_gnomevfssink && has_vorbis) ?  rb_add_ogg_tagger : NULL;
+	}
+#else
+	tagger = NULL;
+#endif
+	add_supported_type (md, "application/ogg", tagger, "Ogg Vorbis");
+	add_supported_type (md, "audio/x-vorbis", tagger, "Ogg Vorbis");
 
 	add_supported_type (md, "audio/x-mod", NULL, "MOD");
 	add_supported_type (md, "audio/x-wav", NULL, "WAV");
@@ -301,6 +453,7 @@ rb_metadata_finalize (GObject *object)
 
 	g_free (md->priv->type);
 	g_free (md->priv->uri);
+	g_clear_error (&md->priv->error);
 
 	G_OBJECT_CLASS (rb_metadata_parent_class)->finalize (object);
 }
@@ -335,7 +488,7 @@ rb_metadata_gst_tag_to_field (const char *tag)
 		return RB_METADATA_FIELD_COMMENT;
 	else if (!strcmp (tag, GST_TAG_TRACK_NUMBER))
 		return RB_METADATA_FIELD_TRACK_NUMBER;
-	else if (!strcmp (tag, GST_TAG_TRACK_COUNT)) 
+	else if (!strcmp (tag, GST_TAG_TRACK_COUNT))
 		return RB_METADATA_FIELD_MAX_TRACK_NUMBER;
 	else if (!strcmp (tag, GST_TAG_ALBUM_VOLUME_NUMBER))
 		return RB_METADATA_FIELD_DISC_NUMBER;
@@ -371,6 +524,10 @@ rb_metadata_gst_tag_to_field (const char *tag)
 		return RB_METADATA_FIELD_ALBUM_GAIN;
 	else if (!strcmp (tag, GST_TAG_ALBUM_PEAK))
 		return RB_METADATA_FIELD_ALBUM_PEAK;
+#ifdef GST_TAG_MUSICBRAINZ_TRACKID
+	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_TRACKID))
+		return RB_METADATA_FIELD_MUSICBRAINZ_TRACKID;
+#endif
 	else
 		return -1;
 }
@@ -430,6 +587,10 @@ rb_metadata_gst_field_to_gst_tag (RBMetaDataField field)
 		return GST_TAG_ALBUM_GAIN;
 	case RB_METADATA_FIELD_ALBUM_PEAK:
 		return GST_TAG_ALBUM_PEAK;
+#ifdef GST_TAG_MUSICBRAINZ_TRACKID
+	case RB_METADATA_FIELD_MUSICBRAINZ_TRACKID:
+		return GST_TAG_MUSICBRAINZ_TRACKID;
+#endif
 	default:
 		return NULL;
 	}
@@ -467,7 +628,7 @@ make_undecodable_error (RBMetaData *md)
 	human_name= rb_metadata_gst_type_to_name (md, md->priv->type);
 	if (human_name == NULL)
 		human_name = rb_mime_get_friendly_name (md->priv->type);
-	
+
 	if (human_name) {
 		return g_strdup_printf (_("The GStreamer plugins to decode \"%s\" files cannot be found"),
 					human_name);
@@ -501,9 +662,9 @@ rb_metadata_gst_load_tag (const GstTagList *list, const gchar *tag, RBMetaData *
 	newval = g_new0 (GValue, 1);
 	g_value_init (newval, type);
 	if (!g_value_transform (val, newval)) {
-		
+
 		rb_debug ("Could not transform tag value type %s into %s",
-			  g_type_name (G_VALUE_TYPE (val)), 
+			  g_type_name (G_VALUE_TYPE (val)),
 			  g_type_name (G_VALUE_TYPE (newval)));
 		g_value_unset (newval);
 		g_free (newval);
@@ -514,7 +675,7 @@ rb_metadata_gst_load_tag (const GstTagList *list, const gchar *tag, RBMetaData *
 	case G_TYPE_STRING: {
 		/* Reject invalid utf-8 strings, and then
 		 * remove leading and trailing whitespace.
-		 */ 
+		 */
 		char *str;
 		str = g_value_dup_string (newval);
 
@@ -543,7 +704,7 @@ rb_metadata_gst_load_tag (const GstTagList *list, const gchar *tag, RBMetaData *
 	}
 
 	case RB_METADATA_FIELD_DURATION: {
-		/* GStreamer sends us duration in ns, 
+		/* GStreamer sends us duration in ns,
 		 * but we need it in seconds
 		 */
 		guint64 duration;
@@ -556,7 +717,7 @@ rb_metadata_gst_load_tag (const GstTagList *list, const gchar *tag, RBMetaData *
 		break;
 	}
 
-	g_hash_table_insert (md->priv->metadata, 
+	g_hash_table_insert (md->priv->metadata,
 			     GINT_TO_POINTER (field),
 			     newval);
 }
@@ -579,7 +740,7 @@ rb_metadata_gst_fakesink_handoff_cb (GstElement *fakesink, GstBuffer *buf, GstPa
 		rb_debug ("caught handoff after eos!");
 		return;
 	}
-		
+
 	rb_debug ("in fakesink handoff");
 	md->priv->handoff = TRUE;
 }
@@ -629,7 +790,6 @@ rb_metadata_gst_typefind_cb (GstElement *typefind, guint probability, GstCaps *c
 	g_signal_handler_disconnect (typefind, md->priv->typefind_cb_id);
 }
 
-
 static void
 rb_metadata_gst_new_decoded_pad_cb (GstElement *decodebin, GstPad *pad, gboolean last, RBMetaData *md)
 {
@@ -677,7 +837,7 @@ rb_metadata_gst_new_decoded_pad_cb (GstElement *decodebin, GstPad *pad, gboolean
 	 * under 0.8 this causes assertion faliures when a pad with no caps in found
 	 * it isn't /needed/ under 0.8, so we don't do it.
 	 *
-	 * This seems to cause some deadlocks with video files, so only do it 
+	 * This seems to cause some deadlocks with video files, so only do it
 	 * when we get no/any caps.
 	 */
 	if (cancel)
@@ -700,7 +860,7 @@ rb_metadata_gst_unknown_type_cb (GstElement *decodebin, GstPad *pad, GstCaps *ca
 
 		g_free (md->priv->type);
 		md->priv->type = g_strdup (mimetype);
-		
+
 		rb_debug ("decodebin emitted unknown type signal for %s", mimetype);
 	} else {
 		rb_debug ("decodebin emitted unknown type signal");
@@ -755,7 +915,7 @@ rb_metadata_bus_handler (GstBus *bus, GstMessage *message, RBMetaData *md)
 			rb_debug ("caught error: %s, but we've already got one", gerror->message);
 		} else {
 			rb_debug ("caught error: %s ", gerror->message);
-			
+
 			g_clear_error (&md->priv->error);
 			md->priv->error = g_error_new_literal (RB_METADATA_ERROR,
 							       RB_METADATA_ERROR_GENERAL,
@@ -792,7 +952,7 @@ rb_metadata_bus_handler (GstBus *bus, GstMessage *message, RBMetaData *md)
 	return FALSE;
 }
 
-static void 
+static void
 rb_metadata_event_loop (RBMetaData *md, GstElement *element, gboolean block)
 {
 	GstBus *bus;
@@ -894,7 +1054,7 @@ rb_metadata_load (RBMetaData *md,
 
 #ifdef HAVE_GSTREAMER_0_8
 	g_object_set (G_OBJECT (md->priv->sink), "signal-handoffs", TRUE, NULL);
- 
+
  	g_signal_connect_object (pipeline, "error", G_CALLBACK (rb_metadata_gst_error_cb), md, 0);
  	g_signal_connect_object (pipeline, "found-tag", G_CALLBACK (rb_metadata_gst_found_tag), md, 0);
 
@@ -903,20 +1063,21 @@ rb_metadata_load (RBMetaData *md,
 #endif
  	g_signal_connect_object (decodebin, "new-decoded-pad", G_CALLBACK (rb_metadata_gst_new_decoded_pad_cb), md, 0);
  	g_signal_connect_object (decodebin, "unknown-type", G_CALLBACK (rb_metadata_gst_unknown_type_cb), md, 0);
-  
+
  	/* locate the decodebin's typefind, so we can get the have_type signal too.
- 	 * this is kind of nasty, since it relies on an essentially arbitrary string 
+ 	 * this is kind of nasty, since it relies on an essentially arbitrary string
  	 * in the decodebin code not changing.  the alternative is to have our own
  	 * typefind instance before the decodebin.  it might not like that.
  	 */
  	typefind = gst_bin_get_by_name (GST_BIN (decodebin), "typefind");
  	g_assert (typefind != NULL);
-	md->priv->typefind_cb_id = g_signal_connect_object (typefind, 
-							    "have_type", 
-							    G_CALLBACK (rb_metadata_gst_typefind_cb), 
-							    md, 
+	md->priv->typefind_cb_id = g_signal_connect_object (typefind,
+							    "have_type",
+							    G_CALLBACK (rb_metadata_gst_typefind_cb),
+							    md,
 							    0);
- 
+	gst_object_unref (GST_OBJECT (typefind));
+
  	gst_element_link (urisrc, decodebin);
 
 	md->priv->pipeline = pipeline;
@@ -961,7 +1122,7 @@ rb_metadata_load (RBMetaData *md,
 		GstBus *bus;
 		bus = gst_element_get_bus (GST_ELEMENT (pipeline));
 		if (bus) {
-			gst_bus_post (bus, 
+			gst_bus_post (bus,
 			gst_message_new_application (GST_OBJECT (pipeline), NULL));
 			gst_object_unref (bus);
 		}
@@ -977,7 +1138,7 @@ rb_metadata_load (RBMetaData *md,
 			GstFormat format = GST_FORMAT_TIME;
 			gint64 length;
 			GValue *newval;
-			
+
 #ifdef HAVE_GSTREAMER_0_8
 			if (gst_element_query (md->priv->sink, GST_QUERY_TOTAL, &format, &length)) {
 #endif
@@ -986,9 +1147,9 @@ rb_metadata_load (RBMetaData *md,
 				g_assert (format == GST_FORMAT_TIME);
 #endif
 				newval = g_new0 (GValue, 1);
-				
+
 				rb_debug ("duration query succeeded");
-				
+
 				g_value_init (newval, G_TYPE_ULONG);
 				/* FIXME - use guint64 for duration? */
 				g_value_set_ulong (newval, (long) (length / GST_SECOND));
@@ -1015,7 +1176,7 @@ rb_metadata_load (RBMetaData *md,
 #ifdef HAVE_GSTREAMER_0_10
 	if (gst_element_query_duration (urisrc, &file_size_format, &file_size))
 		g_assert (file_size_format == GST_FORMAT_BYTES);
-	
+
 	state_ret = gst_element_set_state (pipeline, GST_STATE_NULL);
 	if (state_ret == GST_STATE_CHANGE_ASYNC) {
 		g_warning ("Failed to return metadata reader to NULL state");
@@ -1027,22 +1188,7 @@ rb_metadata_load (RBMetaData *md,
 	 * these don't include the URI as the import errors source
 	 * already displays it.
 	 */
-	if (md->priv->error) {
-		g_propagate_error (error, md->priv->error);
-	} else if (!md->priv->type) {
-		/* ignore really small files that can't be identified */
-		gint error_code = RB_METADATA_ERROR_UNRECOGNIZED;
-		if (file_size > 0 && file_size < REALLY_SMALL_FILE_SIZE) {
-			rb_debug ("ignoring %s because it's too small to care about", md->priv->uri);
-			error_code = RB_METADATA_ERROR_NOT_AUDIO_IGNORE;
-		}
-		
-		g_clear_error (error);
-		g_set_error (error,
-			     RB_METADATA_ERROR,
-			     error_code,
-			     _("The MIME type of the file could not be identified"));
-	} else if (md->priv->non_audio || !md->priv->handoff) {
+	if (md->priv->non_audio || !md->priv->handoff) {
 		gboolean ignore = FALSE;
 		int i;
 
@@ -1065,15 +1211,35 @@ rb_metadata_load (RBMetaData *md,
 				     "%s", msg);
 			g_free (msg);
 		} else {
+			/* we don't need an error message here (it'll never be
+			 * displayed).  using NULL causes crashes with some C
+			 * libraries, and gcc doesn't like zero-length format
+			 * strings, so we use a single space instead.
+			 */
 			g_set_error (error,
 				     RB_METADATA_ERROR,
 				     RB_METADATA_ERROR_NOT_AUDIO_IGNORE,
-				     NULL);
+				     " ");
 		}
+	} else if (md->priv->error) {
+		g_propagate_error (error, md->priv->error);
+	} else if (!md->priv->type) {
+		/* ignore really small files that can't be identified */
+		gint error_code = RB_METADATA_ERROR_UNRECOGNIZED;
+		if (file_size > 0 && file_size < REALLY_SMALL_FILE_SIZE) {
+			rb_debug ("ignoring %s because it's too small to care about", md->priv->uri);
+			error_code = RB_METADATA_ERROR_NOT_AUDIO_IGNORE;
+		}
+
+		g_clear_error (error);
+		g_set_error (error,
+			     RB_METADATA_ERROR,
+			     error_code,
+			     _("The MIME type of the file could not be identified"));
 	} else {
 		/* yay, it worked */
 		rb_debug ("successfully read metadata for %s", uri);
-	
+
 		/* it doesn't matter if we don't recognise the format,
 		 * as long as gstreamer can play it, we can put it in
 		 * the library.
@@ -1095,13 +1261,13 @@ rb_metadata_can_save (RBMetaData *md, const char *mimetype)
 {
 #ifdef ENABLE_TAG_WRITING
 	return rb_metadata_gst_type_to_tag_function (md, mimetype) != NULL;
-#else 
+#else
 	return FALSE;
 #endif
 }
 
 static void
-rb_metadata_gst_add_tag_data (gpointer key, const GValue *val, GstTagSetter *tagsetter)
+rb_metadata_gst_add_tag_data (gpointer key, const GValue *val, RBMetaData *md)
 {
 	RBMetaDataField field = GPOINTER_TO_INT (key);
 	const char *tag = rb_metadata_gst_field_to_gst_tag (field);
@@ -1122,9 +1288,10 @@ rb_metadata_gst_add_tag_data (gpointer key, const GValue *val, GstTagSetter *tag
 			if (g_value_transform (val, &newval)) {
 				rb_debug("Setting %s",tag);
 
-				gst_tag_setter_add_tag_values (GST_TAG_SETTER (tagsetter),
-							   GST_TAG_MERGE_REPLACE_ALL,
-							   tag, &newval, NULL);
+				gst_tag_list_add_values (md->priv->tags,
+							 GST_TAG_MERGE_APPEND,
+							 tag, &newval,
+							 NULL);
 			}
 			g_value_unset (&newval);
 		}
@@ -1143,7 +1310,7 @@ rb_metadata_file_valid (char *original, char *newfile)
 
 	/* TODO: check that the tags are correct? */
 
-	if (error != NULL)	
+	if (error != NULL)
 		g_error_free (error);
 	g_object_unref (G_OBJECT (md));
 	return ret;
@@ -1155,7 +1322,6 @@ rb_metadata_save (RBMetaData *md, GError **error)
 	GstElement *pipeline = NULL;
 	GstElement *gnomevfssrc = NULL;
         GstElement *retag_end = NULL; /* the last elemet after retagging subpipeline */
-	GstElement *tagger = NULL;
 	const char *plugin_name = NULL;
 	char *tmpname = NULL;
 	GnomeVFSHandle *handle = NULL;
@@ -1171,6 +1337,7 @@ rb_metadata_save (RBMetaData *md, GError **error)
 		goto vfs_error;
 
 	pipeline = gst_pipeline_new ("pipeline");
+	md->priv->pipeline = pipeline;
 
 #ifdef HAVE_GSTREAMER_0_8
 	g_signal_connect_object (pipeline, "error", G_CALLBACK (rb_metadata_gst_error_cb), md, 0);
@@ -1180,7 +1347,7 @@ rb_metadata_save (RBMetaData *md, GError **error)
 	plugin_name = "gnomevfssrc";
 	if (!(gnomevfssrc = gst_element_factory_make (plugin_name, plugin_name)))
 		goto missing_plugin;
-	gst_bin_add (GST_BIN (pipeline), gnomevfssrc);		  
+	gst_bin_add (GST_BIN (pipeline), gnomevfssrc);
 	g_object_set (G_OBJECT (gnomevfssrc), "location", md->priv->uri, NULL);
 
 	/* Sink */
@@ -1193,45 +1360,36 @@ rb_metadata_save (RBMetaData *md, GError **error)
 	g_signal_connect_object (md->priv->sink, "eos", G_CALLBACK (rb_metadata_gst_eos_cb), md, 0);
 #endif
 
+	md->priv->tags = gst_tag_list_new ();
+	g_hash_table_foreach (md->priv->metadata,
+			      (GHFunc) rb_metadata_gst_add_tag_data,
+			      md);
+
 	/* Tagger element(s) */
 	add_tagger_func = rb_metadata_gst_type_to_tag_function (md, md->priv->type);
-	
+
 	if (!add_tagger_func) {
 		g_set_error (error,
 			     RB_METADATA_ERROR,
 			     RB_METADATA_ERROR_UNSUPPORTED,
-			     "Unsupported file type: %s", md->priv->type);
+			     _("Unsupported file type: %s"), md->priv->type);
 		goto out_error;
 	}
 
-	retag_end = add_tagger_func (GST_BIN (pipeline), gnomevfssrc);
+	retag_end = add_tagger_func (md, gnomevfssrc);
 	if (!retag_end) {
 		g_set_error (error,
 			     RB_METADATA_ERROR,
 			     RB_METADATA_ERROR_UNSUPPORTED,
-			     "Unable to create tag-writing elements");
+			     _("Unable to create tag-writing elements"));
 		goto out_error;
 	}
 
-        tagger = gst_bin_get_by_interface (GST_BIN (pipeline), GST_TYPE_TAG_SETTER);
-	if (!tagger) {
-		g_set_error (error,
-			     RB_METADATA_ERROR,
-			     RB_METADATA_ERROR_UNSUPPORTED,
-			     "Unable to find tag-writing element");
-		goto out_error;
-	}
-
-	gst_tag_setter_set_tag_merge_mode (GST_TAG_SETTER (tagger), GST_TAG_MERGE_REPLACE_ALL);
-	g_hash_table_foreach (md->priv->metadata, 
-			      (GHFunc) rb_metadata_gst_add_tag_data,
-			      GST_TAG_SETTER (tagger));
-	
-	gst_bin_add (GST_BIN (pipeline), md->priv->sink); 
+	gst_bin_add (GST_BIN (pipeline), md->priv->sink);
 	gst_element_link_many (retag_end, md->priv->sink, NULL);
 
-	md->priv->pipeline = pipeline;
 	gst_element_set_state (pipeline, GST_STATE_PLAYING);
+
 #ifdef HAVE_GSTREAMER_0_8
 	while (gst_bin_iterate (GST_BIN (pipeline))
 	       && md->priv->error == NULL
@@ -1246,7 +1404,7 @@ rb_metadata_save (RBMetaData *md, GError **error)
 			g_set_error (error,
 				     RB_METADATA_ERROR,
 				     RB_METADATA_ERROR_INTERNAL,
-				     "Timeout while setting pipeline to NULL");
+				     _("Timeout while setting pipeline to NULL"));
 			goto out_error;
 		}
 	}
@@ -1266,10 +1424,10 @@ rb_metadata_save (RBMetaData *md, GError **error)
 			g_set_error (error,
 				     RB_METADATA_ERROR,
 				     RB_METADATA_ERROR_INTERNAL,
-				     "File corrupted during write");
+				     _("File corrupted during write"));
 			goto out_error;
 		}
-		
+
 		if ((result = gnome_vfs_move (tmpname, md->priv->uri, TRUE)) != GNOME_VFS_OK)
 				goto vfs_error;
 	}
@@ -1280,20 +1438,25 @@ vfs_error:
 		     RB_METADATA_ERROR,
 		     RB_METADATA_ERROR_GNOMEVFS,
 		     "%s",
-		     gnome_vfs_result_to_string (result)); 
+		     gnome_vfs_result_to_string (result));
 	goto out_error;
 missing_plugin:
 	g_set_error (error,
 		     RB_METADATA_ERROR,
 		     RB_METADATA_ERROR_MISSING_PLUGIN,
 		     _("Failed to create %s element; check your installation"),
-		     plugin_name); 
+		     plugin_name);
 out_error:
 	if (handle != NULL)
 		gnome_vfs_close (handle);
 	if (tmpname != NULL)
 		gnome_vfs_unlink (tmpname);
+
 out:
+	if (md->priv->tags)
+		gst_tag_list_free (md->priv->tags);
+	md->priv->tags = NULL;
+
 	if (pipeline != NULL)
 		gst_object_unref (GST_OBJECT (pipeline));
 	md->priv->pipeline = NULL;
@@ -1325,7 +1488,7 @@ rb_metadata_set (RBMetaData *md, RBMetaDataField field,
 {
 	GValue *newval;
 	GType type;
-	
+
 	if (rb_metadata_gst_field_to_gst_tag (field) == NULL) {
 		return FALSE;
 	}
@@ -1341,4 +1504,3 @@ rb_metadata_set (RBMetaData *md, RBMetaDataField field,
 			     newval);
 	return TRUE;
 }
-
