@@ -28,6 +28,7 @@
 #include <libxml/tree.h>
 
 #include "config.h"
+#include "rb-refstring.h"
 #include "rhythmdb-query-results.h"
 
 G_BEGIN_DECLS
@@ -48,8 +49,6 @@ GType rhythmdb_entry_get_type (void);
 
 #define RHYTHMDB_TYPE_ENTRY      (rhythmdb_entry_get_type ())
 #define RHYTHMDB_ENTRY(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), RHYTHMDB_TYPE_ENTRY, RhythmDBEntry))
-#define RHYTHMDB_IS_ENTRY(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), RHYTHMDB_TYPE_ENTRY))
-
 
 typedef void (*RhythmDBEntryActionFunc) (RhythmDBEntry *entry, gpointer data);
 typedef char* (*RhythmDBEntryStringFunc) (RhythmDBEntry *entry, gpointer data);
@@ -60,6 +59,7 @@ typedef struct {
 	char 				*name;
 
 	guint				entry_type_data_size;
+	gboolean			save_to_disk;
 
 	/* virtual functions here */
 	RhythmDBEntryActionFunc		post_entry_create;
@@ -89,15 +89,12 @@ GType rhythmdb_entry_type_get_type (void);
 #define RHYTHMDB_ENTRY_TYPE(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), RHYTHMDB_TYPE_ENTRY_TYPE, RhythmDBEntryType_))
 #define RHYTHMDB_IS_ENTRY_TYPE(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), RHYTHMDB_TYPE_ENTRY_TYPE))
 
-
-
 typedef GPtrArray RhythmDBQuery;
 GType rhythmdb_query_get_type (void);
 #define RHYTHMDB_TYPE_QUERY	(rhythmdb_query_get_type ())
 #define RHYTHMDB_QUERY(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), RHYTHMDB_TYPE_QUERY, RhythmDBQuery))
 #define RHYTHMDB_IS_QUERY(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), RHYTHMDB_TYPE_QUERY))
 
-
 #define RHYTHMDB_ENTRY_TYPE_SONG (rhythmdb_entry_song_get_type ())
 #define RHYTHMDB_ENTRY_TYPE_IRADIO_STATION (rhythmdb_entry_iradio_get_type ())
 #define RHYTHMDB_ENTRY_TYPE_PODCAST_POST (rhythmdb_entry_podcast_post_get_type ())
@@ -129,14 +126,15 @@ typedef enum
 	RHYTHMDB_QUERY_PROP_CURRENT_TIME_WITHIN,
 	RHYTHMDB_QUERY_PROP_CURRENT_TIME_NOT_WITHIN,
 	RHYTHMDB_QUERY_PROP_YEAR_EQUALS,
-	RHYTHMDB_QUERY_PROP_YEAR_GREATER,   
-	RHYTHMDB_QUERY_PROP_YEAR_LESS,   
+	RHYTHMDB_QUERY_PROP_YEAR_GREATER,
+	RHYTHMDB_QUERY_PROP_YEAR_LESS,
 } RhythmDBQueryType;
 
 /* If you modify this enum, don't forget to modify rhythmdb_prop_get_type */
 typedef enum
 {
 	RHYTHMDB_PROP_TYPE = 0,
+	RHYTHMDB_PROP_ENTRY_ID,
 	RHYTHMDB_PROP_TITLE,
 	RHYTHMDB_PROP_GENRE,
 	RHYTHMDB_PROP_ARTIST,
@@ -187,7 +185,9 @@ typedef enum
 	RHYTHMDB_PROP_COPYRIGHT,
 	RHYTHMDB_PROP_IMAGE,
 	RHYTHMDB_PROP_POST_TIME,
-	
+
+	RHYTHMDB_PROP_MUSICBRAINZ_TRACKID,
+
 	RHYTHMDB_NUM_PROPERTIES
 } RhythmDBPropType;
 
@@ -218,6 +218,7 @@ typedef struct {
 } RhythmDBEntryChange;
 
 const char *rhythmdb_entry_get_string	(RhythmDBEntry *entry, RhythmDBPropType propid);
+RBRefString *rhythmdb_entry_get_refstring (RhythmDBEntry *entry, RhythmDBPropType propid);
 char *rhythmdb_entry_dup_string	(RhythmDBEntry *entry, RhythmDBPropType propid);
 gboolean rhythmdb_entry_get_boolean	(RhythmDBEntry *entry, RhythmDBPropType propid);
 guint64 rhythmdb_entry_get_uint64	(RhythmDBEntry *entry, RhythmDBPropType propid);
@@ -227,8 +228,6 @@ gpointer rhythmdb_entry_get_pointer     (RhythmDBEntry *entry, RhythmDBPropType
 
 RhythmDBEntryType rhythmdb_entry_get_entry_type (RhythmDBEntry *entry);
 
-
-
 typedef enum
 {
 	RHYTHMDB_ERROR_ACCESS_FAILED,
@@ -263,9 +262,9 @@ typedef struct
 
 	/* virtual methods */
 
-	void		(*impl_load)		(RhythmDB *db, gboolean *dead);
+	gboolean	(*impl_load)		(RhythmDB *db, gboolean *dead, GError **error);
 	void		(*impl_save)		(RhythmDB *db);
-	
+
 	void		(*impl_entry_new)	(RhythmDB *db, RhythmDBEntry *entry);
 
 	gboolean	(*impl_entry_set)	(RhythmDB *db, RhythmDBEntry *entry,
@@ -278,8 +277,8 @@ typedef struct
 
 	void            (*impl_entry_delete_by_type) (RhythmDB *db, RhythmDBEntryType type);
 
-	RhythmDBEntry *	(*impl_lookup_by_location)(RhythmDB *db, const char *uri);
-	
+	RhythmDBEntry *	(*impl_lookup_by_location)(RhythmDB *db, RBRefString *uri);
+
 	gboolean 	(*impl_evaluate_query)	(RhythmDB *db, GPtrArray *query, RhythmDBEntry *entry);
 
 	void		(*impl_entry_foreach)	(RhythmDB *db, GFunc func, gpointer data);
@@ -287,8 +286,11 @@ typedef struct
 	void		(*impl_do_full_query)	(RhythmDB *db, GPtrArray *query,
 						 RhythmDBQueryResults *results,
 						 gboolean *cancel);
-} RhythmDBClass;
 
+	void		(*impl_entry_type_registered) (RhythmDB *db,
+						       const char *name,
+						       RhythmDBEntryType type);
+} RhythmDBClass;
 
 GType		rhythmdb_get_type	(void);
 
@@ -316,10 +318,6 @@ void		rhythmdb_add_uri_with_type	(RhythmDB *db, const char *uri, RhythmDBEntryTy
 void		rhythmdb_entry_get	(RhythmDB *db, RhythmDBEntry *entry, RhythmDBPropType propid, GValue *val);
 void		rhythmdb_entry_set	(RhythmDB *db, RhythmDBEntry *entry,
 					 guint propid, const GValue *value);
-void		rhythmdb_entry_set_nonotify	(RhythmDB *db, RhythmDBEntry *entry,
-						 guint propid, const GValue *value);
-void		rhythmdb_entry_set_uninserted   (RhythmDB *db, RhythmDBEntry *entry,
-						 guint propid, const GValue *value);
 
 char *		rhythmdb_entry_get_playback_uri	(RhythmDBEntry *entry);
 
@@ -327,12 +325,11 @@ gpointer	rhythmdb_entry_get_type_data (RhythmDBEntry *entry, guint expected_size
 #define		RHYTHMDB_ENTRY_GET_TYPE_DATA(e,t)	((t*)rhythmdb_entry_get_type_data((e),sizeof(t)))
 
 void		rhythmdb_entry_delete	(RhythmDB *db, RhythmDBEntry *entry);
-void            rhythmdb_entry_delete_by_type (RhythmDB *db, 
+void            rhythmdb_entry_delete_by_type (RhythmDB *db,
 					       RhythmDBEntryType type);
 void		rhythmdb_entry_move_to_trash (RhythmDB *db,
 					      RhythmDBEntry *entry);
 
-
 RhythmDBEntry *	rhythmdb_entry_lookup_by_location (RhythmDB *db, const char *uri);
 
 gboolean	rhythmdb_evaluate_query		(RhythmDB *db, GPtrArray *query,
@@ -374,8 +371,6 @@ void		rhythmdb_do_full_query_async_parsed	(RhythmDB *db,
 							 RhythmDBQueryResults *results,
 							 GPtrArray *query);
 
-void		rhythmdb_entry_sync_mirrored		(RhythmDB *db, RhythmDBEntry *entry, guint propid);
-
 GPtrArray *	rhythmdb_query_parse			(RhythmDB *db, ...);
 void		rhythmdb_query_append			(RhythmDB *db, GPtrArray *query, ...);
 void		rhythmdb_query_append_prop_multiple	(RhythmDB *db, GPtrArray *query, RhythmDBPropType propid, GList *items);
@@ -389,6 +384,8 @@ void		rhythmdb_query_serialize		(RhythmDB *db, GPtrArray *query,
 
 GPtrArray *	rhythmdb_query_deserialize		(RhythmDB *db, xmlNodePtr node);
 
+char *		rhythmdb_query_to_string		(RhythmDB *db, GPtrArray *query);
+
 gboolean	rhythmdb_query_is_time_relative		(RhythmDB *db, GPtrArray *query);
 
 const xmlChar *	rhythmdb_nice_elt_name_from_propid	(RhythmDB *db, RhythmDBPropType propid);
@@ -403,9 +400,8 @@ char *		rhythmdb_compute_status_normal		(gint n_songs, glong duration,
 							 const char *singular,
 							 const char *plural);
 
-
-RhythmDBEntryType rhythmdb_entry_register_type          (const char *name);
-RhythmDBEntryType rhythmdb_entry_type_get_by_name       (const char *name);
+RhythmDBEntryType rhythmdb_entry_register_type          (RhythmDB *db, const char *name);
+RhythmDBEntryType rhythmdb_entry_type_get_by_name       (RhythmDB *db, const char *name);
 
 RhythmDBEntryType rhythmdb_entry_song_get_type          (void);
 RhythmDBEntryType rhythmdb_entry_iradio_get_type        (void);
@@ -416,8 +412,8 @@ RhythmDBEntryType rhythmdb_entry_ignore_get_type        (void);
 
 GType rhythmdb_get_property_type (RhythmDB *db, guint property_id);
 
-void rhythmdb_entry_ref (RhythmDB *db, RhythmDBEntry *entry);
-void rhythmdb_entry_unref (RhythmDB *db, RhythmDBEntry *entry);
+RhythmDBEntry* rhythmdb_entry_ref (RhythmDBEntry *entry);
+void rhythmdb_entry_unref (RhythmDBEntry *entry);
 
 G_END_DECLS
 
