@@ -60,7 +60,6 @@
 #include "eel-gconf-extensions.h"
 #include "rb-util.h"
 
-
 #if WITH_DBUS
 #include <dbus/dbus-glib.h>
 #include "rb-shell-glue.h"
@@ -73,14 +72,7 @@
 #include <dbus/dbus-glib-lowlevel.h>
 #endif
 
-#include <nautilus-burn-drive.h>
-#ifndef NAUTILUS_BURN_CHECK_VERSION 	 
-#define NAUTILUS_BURN_CHECK_VERSION(a,b,c) FALSE 	 
-#endif
-
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
-#include <nautilus-burn.h>
-#endif
+#define HAVE_LIBGNOME_GOPTION defined(GNOME_PARAM_GOPTION_CONTEXT)
 
 static gboolean debug           = FALSE;
 static char *debug_match        = NULL;
@@ -88,7 +80,10 @@ static gboolean quit            = FALSE;
 static gboolean no_registration = FALSE;
 static gboolean no_update	= FALSE;
 static gboolean dry_run		= FALSE;
-static char *rhythmdb_file = NULL;
+static char *rhythmdb_file	= NULL;
+static char *playlists_file	= NULL;
+static char **remaining_args    = NULL;
+
 #if WITH_DBUS
 static gboolean load_uri_args (const char **args, GFunc handler, gpointer user_data);
 static void dbus_load_uri (const char *filename, DBusGProxy *proxy);
@@ -105,7 +100,7 @@ static gboolean send_present_message (DBusConnection *connection, guint32 curren
 int
 main (int argc, char **argv)
 {
-	GnomeProgram *program;
+	GnomeProgram *program = NULL;
 #if WITH_DBUS || WITH_OLD_DBUS
 	DBusGConnection *session_bus;
 	GError *error = NULL;
@@ -113,25 +108,26 @@ main (int argc, char **argv)
 	RBShell *rb_shell;
 	char **new_argv;
 	gboolean activated;
-	poptContext poptContext;
-        GValue context_as_value = { 0 };
+
+
+	GOptionContext *context;
+	static const GOptionEntry options []  = {
+		{ "debug",           'd', 0, G_OPTION_ARG_NONE,         &debug,           N_("Enable debug output"), NULL },
+		{ "debug-match",     'D', 0, G_OPTION_ARG_STRING,       &debug_match,     N_("Enable debug output matching a specified string"), NULL },
+		{ "no-update",	       0, 0, G_OPTION_ARG_NONE,         &no_update,       N_("Do not update the library with file changes"), NULL },
+		{ "no-registration", 'n', 0, G_OPTION_ARG_NONE,         &no_registration, N_("Do not register the shell"), NULL },
+		{ "dry-run",	       0, 0, G_OPTION_ARG_NONE,         &dry_run,         N_("Don't save any data permanently (implies --no-registration)"), NULL },
+		{ "rhythmdb-file",     0, 0, G_OPTION_ARG_STRING,       &rhythmdb_file,   N_("Path for database file to use"), NULL },
+		{ "playlists-file",    0, 0, G_OPTION_ARG_STRING,       &playlists_file,   N_("Path for playlists file to use"), NULL },
+		{ "quit",	     'q', 0, G_OPTION_ARG_NONE,         &quit,            N_("Quit Rhythmbox"), NULL },
+		{ G_OPTION_REMAINING,  0, 0, G_OPTION_ARG_STRING_ARRAY, &remaining_args,  NULL, N_("[URI...]") },
+		{ NULL }
+	};
 
 	rb_profile_start ("starting rhythmbox");
 
-	struct poptOption popt_options[] =
-	{
-		{ "debug",			'd',POPT_ARG_NONE,	&debug,				0, N_("Enable debug output"), NULL },
-		{ "debug-match",		'D',POPT_ARG_STRING,	&debug_match,			0, N_("Enable debug output matching a specified string"), NULL },
-		{ "no-update",			0,  POPT_ARG_NONE,	&no_update,			0, N_("Do not update the library with file changes"), NULL },
-		{ "no-registration",		'n',POPT_ARG_NONE,	&no_registration,		0, N_("Do not register the shell"), NULL },
-		{ "dry-run",			0,  POPT_ARG_NONE,	&dry_run,			0, N_("Don't save any data permanently (implies --no-registration)"), NULL },
-		{ "rhythmdb-file",		0,  POPT_ARG_STRING,	&rhythmdb_file,			0, N_("Path for database file to use"), NULL },
-		{ "quit",			'q',POPT_ARG_NONE,	&quit,				0, N_("Quit Rhythmbox"), NULL },
-#ifdef HAVE_GSTREAMER_0_8
-		{NULL, '\0', POPT_ARG_INCLUDE_TABLE, NULL, 0, "GStreamer", NULL},
-#endif
-		POPT_TABLEEND
-	};
+	context = g_option_context_new (NULL);
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
 
 	/* Disable event sounds for now by passing "--disable-sound" to libgnomeui.
 	 * See: http://bugzilla.gnome.org/show_bug.cgi?id=119222 */
@@ -141,37 +137,45 @@ main (int argc, char **argv)
 	new_argv[argc+1] = NULL;
 
 #ifdef HAVE_GSTREAMER_0_8
-	popt_options[(sizeof(popt_options)/sizeof(popt_options[0]))-2].arg
-		= (void *) gst_init_get_popt_table ();
-#elif HAVE_GSTREAMER_0_10
-        /* To pass options to GStreamer in 0.10, RB needs to use GOption, for
+        /* To pass options to GStreamer in 0.8, RB needs to use popt, for
          * now, GStreamer can live without them (people can still use env
          * vars, after all) */
+	gst_init (NULL, NULL);
+#elif HAVE_GSTREAMER_0_10
 	rb_profile_start ("initializing gstreamer");
-        gst_init (NULL, NULL);
+	g_option_context_add_group (context, gst_init_get_option_group ());
 	rb_profile_end ("initializing gstreamer");
 #endif
 
 	gtk_set_locale ();
 	rb_profile_start ("initializing gnome program");
+
+#if HAVE_LIBGNOME_GOPTION
 	program = gnome_program_init (PACKAGE, VERSION,
-				      LIBGNOMEUI_MODULE, argc+1, new_argv,
-				      GNOME_PARAM_POPT_TABLE, popt_options,
+				      LIBGNOMEUI_MODULE,
+				      argc+1, new_argv,
+				      GNOME_PARAM_GOPTION_CONTEXT, context,
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Rhythmbox"),
 				      GNOME_PARAM_APP_DATADIR, DATADIR,
 				      NULL);
-	rb_profile_end ("initializing gnome program");
+#else
+	g_option_context_parse (context, &argc, &argv, NULL);
+	g_option_context_free (context);
+	program = gnome_program_init (PACKAGE, VERSION,
+				      LIBGNOMEUI_MODULE,
+				      argc+1, new_argv,
+				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Rhythmbox"),
+				      GNOME_PARAM_APP_DATADIR, DATADIR,
+				      NULL);
+#endif
 
-	g_object_get_property (G_OBJECT (program),
-                               GNOME_PARAM_POPT_CONTEXT,
-                               g_value_init (&context_as_value, G_TYPE_POINTER));
-        poptContext = g_value_get_pointer (&context_as_value);
+	rb_profile_end ("initializing gnome program");
 
 	rb_profile_start ("initializing gnome auth manager");
 	gnome_authentication_manager_init ();
 	rb_profile_end ("initializing gnome auth manager");
 
-	g_random_set_seed (time(0));
+	g_random_set_seed (time (0));
 
 #ifdef ENABLE_NLS
 	/* initialize i18n */
@@ -189,7 +193,7 @@ main (int argc, char **argv)
 	else
 		rb_debug_init (debug);
 	rb_debug ("initializing Rhythmbox %s", VERSION);
-	
+
 	/* TODO: kill this function */
 	rb_threads_init ();
 
@@ -211,7 +215,7 @@ main (int argc, char **argv)
 		flags = DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT;
 #else
 		flags = DBUS_NAME_FLAG_DO_NOT_QUEUE;
-#endif                
+#endif
 
 #ifndef WITH_OLD_DBUS
 		DBusGProxy *bus_proxy;
@@ -263,10 +267,7 @@ main (int argc, char **argv)
 				   request_name_reply);
 			activated = FALSE;
 		}
-		
-						
 	}
-		
 #endif
 
 	if (!activated) {
@@ -278,24 +279,20 @@ main (int argc, char **argv)
 #ifdef WITH_RHYTHMDB_GDA
 		gda_init (PACKAGE, VERSION, argc, argv);
 #endif
- 	
+
 		rb_refstring_system_init ();
 
 		rb_file_helpers_init ();
 
 		rb_debug ("Going to create a new shell");
-	
+
 		glade_gnome_init ();
-	
+
 		rb_stock_icons_init ();
 
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
-		nautilus_burn_init ();
-#endif
-
 		gtk_window_set_default_icon_name ("rhythmbox");
-	
-		rb_shell = rb_shell_new (argc, argv, no_registration, no_update, dry_run, rhythmdb_file);
+
+		rb_shell = rb_shell_new (argc, argv, no_registration, no_update, dry_run, rhythmdb_file, playlists_file);
 		g_object_weak_ref (G_OBJECT (rb_shell), main_shell_weak_ref_cb, NULL);
 		if (!no_registration && session_bus != NULL) {
 #if WITH_DBUS
@@ -310,14 +307,14 @@ main (int argc, char **argv)
 			obj = rb_shell_get_player (rb_shell);
 			path = rb_shell_get_player_path (rb_shell);
 			dbus_g_connection_register_g_object (session_bus, path, obj);
-			
+
 			/* register playlist manager object */
 			dbus_g_object_type_install_info (RB_TYPE_PLAYLIST_MANAGER, &dbus_glib_rb_playlist_manager_object_info);
 			obj = rb_shell_get_playlist_manager (rb_shell);
 			path = rb_shell_get_playlist_manager_path (rb_shell);
 			dbus_g_connection_register_g_object (session_bus, path, obj);
 #elif WITH_OLD_DBUS
-			register_dbus_handler (dbus_g_connection_get_connection (session_bus), 
+			register_dbus_handler (dbus_g_connection_get_connection (session_bus),
 					       rb_shell);
 #endif /* WITH_DBUS */
 		}
@@ -353,7 +350,7 @@ main (int argc, char **argv)
 				dbus_g_proxy_call_no_reply (shell_proxy, "quit",
 							    G_TYPE_INVALID);
 			} else {
-				load_uri_args (poptGetArgs (poptContext), (GFunc) dbus_load_uri, shell_proxy);
+				load_uri_args ((const char **) remaining_args, (GFunc) dbus_load_uri, shell_proxy);
 				dbus_g_proxy_call_no_reply (shell_proxy, "present",
 							    G_TYPE_UINT, current_time,
 							    G_TYPE_INVALID);
@@ -361,7 +358,7 @@ main (int argc, char **argv)
 			g_object_unref (G_OBJECT (shell_proxy));
 		}
 #elif WITH_OLD_DBUS
-		if (!send_present_message (dbus_g_connection_get_connection (session_bus), 
+		if (!send_present_message (dbus_g_connection_get_connection (session_bus),
 					   current_time))
 			g_warning ("Unable to send dbus message to existing rhythmbox instance");
 #endif /* WITH_DBUS */
@@ -382,16 +379,18 @@ main (int argc, char **argv)
 		rb_refstring_system_shutdown ();
 
 		gnome_vfs_shutdown ();
-
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
-		nautilus_burn_shutdown ();
-#endif
 	}
 
+#ifdef HAVE_GSTREAMER_0_10
+	gst_deinit ();
+#endif
+
 	g_strfreev (new_argv);
 
 	rb_debug ("THE END");
 	rb_profile_end ("starting rhythmbox");
+	g_object_unref (program);
+
 	exit (0);
 }
 
@@ -409,12 +408,12 @@ load_uri_args (const char **args, GFunc handler, gpointer user_data)
 		rb_debug ("examining argument %s", args[i]);
 
 		uri = gnome_vfs_make_uri_from_shell_arg (args[i]);
-			
+
 		if (rb_uri_is_local (uri) == FALSE || rb_uri_exists (uri)) {
 			handler (uri, user_data);
 		}
 		g_free (uri);
-		
+
 		handled = TRUE;
 	}
 	return handled;
@@ -442,7 +441,6 @@ main_shell_weak_ref_cb (gpointer data, GObject *objptr)
 	gtk_main_quit ();
 }
 
-
 #ifdef WITH_OLD_DBUS
 /* old dbus support (0.31-0.34) */
 
@@ -487,7 +485,7 @@ handle_dbus_message (DBusConnection *connection, DBusMessage *message, void *dat
 	if (dbus_message_is_method_call (message, "org.gnome.Rhythmbox.Shell", "present")) {
 		DBusMessageIter iter;
 		guint32 current_time;
-		
+
 		if (!dbus_message_iter_init (message, &iter))
 			return DBUS_HANDLER_RESULT_NEED_MEMORY;
 
@@ -521,4 +519,3 @@ register_dbus_handler (DBusConnection *connection, RBShell *shell)
 }
 
 #endif
-
