@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- * 
+ *
  *  arch-tag: Implementation of song cut/paste handler object
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
@@ -47,7 +47,7 @@ static void rb_shell_clipboard_get_property (GObject *object,
 				   	     guint prop_id,
 					     GValue *value,
 					     GParamSpec *pspec);
-static gboolean rb_shell_clipboard_sync (RBShellClipboard *clipboard);
+static void rb_shell_clipboard_sync (RBShellClipboard *clipboard);
 static void rb_shell_clipboard_cmd_select_all (GtkAction *action,
 					       RBShellClipboard *clipboard);
 static void rb_shell_clipboard_cmd_select_none (GtkAction *action,
@@ -86,7 +86,8 @@ static void rb_shell_clipboard_cmd_song_info (GtkAction *action,
 					      RBShellClipboard *clipboard);
 static void rb_shell_clipboard_cmd_queue_song_info (GtkAction *action,
 						    RBShellClipboard *clipboard);
-static gboolean rebuild_playlist_menu (RBShellClipboard *clipboard);
+static void rebuild_playlist_menu (RBShellClipboard *clipboard);
+static gboolean rebuild_playlist_menu_idle (RBShellClipboard *clipboard);
 
 struct RBShellClipboardPrivate
 {
@@ -172,7 +173,6 @@ static const char *playlist_menu_paths[] = {
 };
 static guint num_playlist_menu_paths = G_N_ELEMENTS (playlist_menu_paths);
 
-
 G_DEFINE_TYPE (RBShellClipboard, rb_shell_clipboard, G_TYPE_OBJECT)
 
 static void
@@ -242,7 +242,25 @@ rb_shell_clipboard_init (RBShellClipboard *shell_clipboard)
 
 	shell_clipboard->priv->deleted_queue = g_async_queue_new ();
 
-	shell_clipboard->priv->idle_deletion_id = g_idle_add ((GSourceFunc) rb_shell_clipboard_idle_poll_deletions, shell_clipboard); 
+	shell_clipboard->priv->idle_deletion_id = g_idle_add ((GSourceFunc) rb_shell_clipboard_idle_poll_deletions, shell_clipboard);
+}
+
+static void
+unset_source_internal (RBShellClipboard *clipboard)
+{
+	if (clipboard->priv->source != NULL) {
+		RBEntryView *songs = rb_source_get_entry_view (clipboard->priv->source);
+
+		if (songs) {
+			g_signal_handlers_disconnect_by_func (G_OBJECT (songs),
+							      G_CALLBACK (rb_shell_clipboard_entryview_changed_cb),
+							      clipboard);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (songs),
+							      G_CALLBACK (rb_shell_clipboard_entries_changed_cb),
+							      clipboard);
+		}
+	}
+	clipboard->priv->source = NULL;
 }
 
 static void
@@ -257,8 +275,12 @@ rb_shell_clipboard_finalize (GObject *object)
 
 	g_return_if_fail (shell_clipboard->priv != NULL);
 
+	/* release references to the source */
+	unset_source_internal (shell_clipboard);
+
 	g_hash_table_destroy (shell_clipboard->priv->signal_hash);
 
+	g_list_foreach (shell_clipboard->priv->entries, (GFunc)rhythmdb_entry_unref, NULL);
 	g_list_free (shell_clipboard->priv->entries);
 
 	g_async_queue_unref (shell_clipboard->priv->deleted_queue);
@@ -269,7 +291,7 @@ rb_shell_clipboard_finalize (GObject *object)
 		g_source_remove (shell_clipboard->priv->idle_deletion_id);
 	if (shell_clipboard->priv->idle_playlist_id)
 		g_source_remove (shell_clipboard->priv->idle_playlist_id);
-	
+
 	G_OBJECT_CLASS (rb_shell_clipboard_parent_class)->finalize (object);
 }
 
@@ -279,7 +301,7 @@ rb_shell_clipboard_constructor (GType type, guint n_construct_properties,
 {
 	RBShellClipboard *clip;
 	RBShellClipboardClass *klass;
-	GObjectClass *parent_class;  
+	GObjectClass *parent_class;
 
 	klass = RB_SHELL_CLIPBOARD_CLASS (g_type_class_peek (RB_TYPE_SHELL_CLIPBOARD));
 
@@ -287,7 +309,7 @@ rb_shell_clipboard_constructor (GType type, guint n_construct_properties,
 	clip = RB_SHELL_CLIPBOARD (parent_class->constructor (type,
 							      n_construct_properties,
 							      construct_properties));
-	
+
 	g_signal_connect_object (G_OBJECT (clip->priv->db),
 				 "entry_deleted",
 				 G_CALLBACK (rb_shell_clipboard_entry_deleted_cb),
@@ -300,18 +322,8 @@ static void
 rb_shell_clipboard_set_source_internal (RBShellClipboard *clipboard,
 					RBSource *source)
 {
-	if (clipboard->priv->source != NULL) {
-		RBEntryView *songs = rb_source_get_entry_view (clipboard->priv->source);
+	unset_source_internal (clipboard);
 
-		if (songs) {
-			g_signal_handlers_disconnect_by_func (G_OBJECT (songs),
-							      G_CALLBACK (rb_shell_clipboard_entryview_changed_cb),
-							      clipboard);
-			g_signal_handlers_disconnect_by_func (G_OBJECT (songs),
-							      G_CALLBACK (rb_shell_clipboard_entries_changed_cb),
-							      clipboard);
-		}
-	}
 	clipboard->priv->source = source;
 	rb_debug ("selected source %p", source);
 
@@ -339,6 +351,8 @@ rb_shell_clipboard_set_source_internal (RBShellClipboard *clipboard,
 						 clipboard, 0);
 		}
 	}
+
+	rebuild_playlist_menu (clipboard);
 }
 
 static void
@@ -368,8 +382,14 @@ rb_shell_clipboard_set_property (GObject *object,
 		clipboard->priv->ui_mgr = g_value_get_object (value);
 		break;
 	case PROP_PLAYLIST_MANAGER:
+		if (clipboard->priv->playlist_manager != NULL) {
+			g_signal_handlers_disconnect_by_func (clipboard->priv->playlist_manager,
+							      G_CALLBACK (rb_shell_clipboard_playlist_added_cb),
+							      clipboard);
+		}
+
 		clipboard->priv->playlist_manager = g_value_get_object (value);
-		if (clipboard->priv->playlist_manager) {
+		if (clipboard->priv->playlist_manager != NULL) {
 			g_signal_connect_object (G_OBJECT (clipboard->priv->playlist_manager),
 						 "playlist-added", G_CALLBACK (rb_shell_clipboard_playlist_added_cb),
 						 clipboard, 0);
@@ -379,14 +399,23 @@ rb_shell_clipboard_set_property (GObject *object,
 
 		break;
 	case PROP_QUEUE_SOURCE:
-		clipboard->priv->queue_source = g_value_get_object (value);
-		if (clipboard->priv->queue_source) {
+		if (clipboard->priv->queue_source != NULL) {
 			RBEntryView *sidebar;
-			g_object_get (G_OBJECT (clipboard->priv->queue_source), "sidebar", &sidebar, NULL);
+			g_object_get (clipboard->priv->queue_source, "sidebar", &sidebar, NULL);
+			g_signal_handlers_disconnect_by_func (sidebar,
+							      G_CALLBACK (rb_shell_clipboard_entryview_changed_cb),
+							      clipboard);
+			g_object_unref (sidebar);
+		}
+
+		clipboard->priv->queue_source = g_value_get_object (value);
+		if (clipboard->priv->queue_source != NULL) {
+			RBEntryView *sidebar;
+			g_object_get (clipboard->priv->queue_source, "sidebar", &sidebar, NULL);
 			g_signal_connect_object (G_OBJECT (sidebar), "selection-changed",
 						 G_CALLBACK (rb_shell_clipboard_entryview_changed_cb),
 						 clipboard, 0);
-			g_object_unref (G_OBJECT (sidebar));
+			g_object_unref (sidebar);
 		}
 		break;
 	default:
@@ -395,7 +424,7 @@ rb_shell_clipboard_set_property (GObject *object,
 	}
 }
 
-static void 
+static void
 rb_shell_clipboard_get_property (GObject *object,
 			         guint prop_id,
 			         GValue *value,
@@ -441,7 +470,9 @@ rb_shell_clipboard_set_source (RBShellClipboard *clipboard,
 }
 
 RBShellClipboard *
-rb_shell_clipboard_new (GtkActionGroup *actiongroup,GtkUIManager *ui_mgr, RhythmDB *db)
+rb_shell_clipboard_new (GtkActionGroup *actiongroup,
+			GtkUIManager *ui_mgr,
+			RhythmDB *db)
 {
 	return g_object_new (RB_TYPE_SHELL_CLIPBOARD,
 			     "action-group", actiongroup,
@@ -451,6 +482,17 @@ rb_shell_clipboard_new (GtkActionGroup *actiongroup,GtkUIManager *ui_mgr, Rhythm
 }
 
 static gboolean
+rb_shell_clipboard_sync_idle (RBShellClipboard *clipboard)
+{
+	GDK_THREADS_ENTER ();
+	rb_shell_clipboard_sync (clipboard);
+	clipboard->priv->idle_sync_id = 0;
+	GDK_THREADS_LEAVE ();
+
+	return FALSE;
+}
+
+static void
 rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 {
 	RBEntryView *view;
@@ -464,11 +506,10 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 	gboolean can_move_to_trash = FALSE;
 	gboolean can_select_all = FALSE;
 	GtkAction *action;
+	RhythmDBEntryType entry_type;
 
 	if (!clipboard->priv->source)
-		return FALSE;
-
-	clipboard->priv->idle_sync_id = 0;
+		return;
 
 	view = rb_source_get_entry_view (clipboard->priv->source);
 	if (view) {
@@ -478,13 +519,13 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 
 	if (clipboard->priv->queue_source) {
 		RBEntryView *sidebar;
-		g_object_get (G_OBJECT (clipboard->priv->queue_source), "sidebar", &sidebar, NULL);
+		g_object_get (clipboard->priv->queue_source, "sidebar", &sidebar, NULL);
 		have_sidebar_selection = rb_entry_view_have_selection (sidebar);
-		g_object_unref (G_OBJECT (sidebar));
+		g_object_unref (sidebar);
 	}
 
 	rb_debug ("syncing clipboard");
-	
+
 	if (g_list_length (clipboard->priv->entries) > 0)
 		can_paste = rb_source_can_paste (clipboard->priv->source);
 
@@ -500,13 +541,13 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditCut");
 	g_object_set (G_OBJECT (action), "sensitive", can_cut, NULL);
-	
+
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditDelete");
 	g_object_set (G_OBJECT (action), "sensitive", can_delete, NULL);
-	
+
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditMovetoTrash");
 	g_object_set (G_OBJECT (action), "sensitive", can_move_to_trash, NULL);
-	
+
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditCopy");
 	g_object_set (G_OBJECT (action), "sensitive", can_copy, NULL);
 
@@ -515,26 +556,32 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditPlaylistAdd");
 	g_object_set (G_OBJECT (action), "sensitive", can_copy, NULL);
-	
+
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "AddToQueue");
 	g_object_set (G_OBJECT (action), "sensitive", can_add_to_queue, NULL);
-	
+
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "MusicProperties");
 	g_object_set (G_OBJECT (action), "sensitive", have_selection, NULL);
-	
+
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "QueueMusicProperties");
 	g_object_set (G_OBJECT (action), "sensitive", have_sidebar_selection, NULL);
-	
+
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "QueueDelete");
 	g_object_set (G_OBJECT (action), "sensitive", have_sidebar_selection, NULL);
-	
+
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditSelectAll");
 	g_object_set (G_OBJECT (action), "sensitive", can_select_all, NULL);
-	
+
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditSelectNone");
 	g_object_set (G_OBJECT (action), "sensitive", have_selection, NULL);
 
-	return FALSE;
+	/* disable the whole add-to-playlist menu if we can't add to a playlist
+	 * FIXME: change this when we support non-library playilst adding
+	 */
+	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditPlaylistAdd");
+	g_object_get (clipboard->priv->source, "entry-type", &entry_type, NULL);
+	gtk_action_set_sensitive (action, (entry_type == RHYTHMDB_ENTRY_TYPE_SONG));
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 }
 
 static void
@@ -613,7 +660,10 @@ static void
 rb_shell_clipboard_set (RBShellClipboard *clipboard,
 			GList *entries)
 {
-	g_list_free (clipboard->priv->entries);
+	if (clipboard->priv->entries != NULL) {
+		g_list_foreach (clipboard->priv->entries, (GFunc)rhythmdb_entry_unref, NULL);
+		g_list_free (clipboard->priv->entries);
+	}
 
 	clipboard->priv->entries = entries;
 }
@@ -637,14 +687,14 @@ rb_shell_clipboard_process_deletions (RBShellClipboard *clipboard)
 			rb_shell_clipboard_sync (clipboard);
 
 		for (tem = finished; tem; tem = tem->next)
-			rhythmdb_entry_unref (clipboard->priv->db, tem->data);
+			rhythmdb_entry_unref (tem->data);
 		g_list_free (finished);
 
 		return processed;
 	} else {
 		/* Fast path for when there's nothing in the clipboard */
 		while ((entry = g_async_queue_try_pop (clipboard->priv->deleted_queue)) != NULL)
-			rhythmdb_entry_unref (clipboard->priv->db, entry);
+			rhythmdb_entry_unref (entry);
 		return FALSE;
 	}
 }
@@ -669,7 +719,6 @@ rb_shell_clipboard_idle_poll_deletions (RBShellClipboard *clipboard)
 				       (GSourceFunc) rb_shell_clipboard_idle_poll_deletions,
 				       clipboard);
 
-
 	GDK_THREADS_LEAVE ();
 
 	return FALSE;
@@ -680,7 +729,7 @@ rb_shell_clipboard_entry_deleted_cb (RhythmDB *db,
 				     RhythmDBEntry *entry,
 				     RBShellClipboard *clipboard)
 {
-	rhythmdb_entry_ref (db, entry);
+	rhythmdb_entry_ref (entry);
 	g_async_queue_push (clipboard->priv->deleted_queue, entry);
 }
 
@@ -689,11 +738,11 @@ rb_shell_clipboard_entryview_changed_cb (RBEntryView *view,
 					 RBShellClipboard *clipboard)
 {
 	if (clipboard->priv->idle_sync_id == 0)
-		clipboard->priv->idle_sync_id = g_idle_add ((GSourceFunc) rb_shell_clipboard_sync,
+		clipboard->priv->idle_sync_id = g_idle_add ((GSourceFunc) rb_shell_clipboard_sync_idle,
 							    clipboard);
 	rb_debug ("entryview changed");
 }
-	
+
 static void
 rb_shell_clipboard_entries_changed_cb (RBEntryView *view,
 				       gpointer stuff,
@@ -701,7 +750,7 @@ rb_shell_clipboard_entries_changed_cb (RBEntryView *view,
 {
 	rb_debug ("entryview changed");
 	if (clipboard->priv->idle_sync_id == 0)
-		clipboard->priv->idle_sync_id = g_idle_add ((GSourceFunc) rb_shell_clipboard_sync,
+		clipboard->priv->idle_sync_id = g_idle_add ((GSourceFunc) rb_shell_clipboard_sync_idle,
 							    clipboard);
 }
 
@@ -713,11 +762,13 @@ rb_shell_clipboard_cmd_add_to_playlist_new (GtkAction *action,
 	RBSource *playlist_source;
 
 	rb_debug ("add to new playlist");
-	
+
 	entries = rb_source_copy (clipboard->priv->source);
 	playlist_source = rb_playlist_manager_new_playlist (clipboard->priv->playlist_manager,
 							    NULL, FALSE);
 	rb_source_paste (playlist_source, entries);
+
+	g_list_foreach (entries, (GFunc)rhythmdb_entry_unref, NULL);
 	g_list_free (entries);
 }
 
@@ -756,9 +807,11 @@ rb_shell_clipboard_playlist_add_cb (GtkAction *action,
 
 	rb_debug ("add to exisintg playlist");
 	playlist_source = g_object_get_data (G_OBJECT (action), "playlist-source");
-	
+
 	entries = rb_source_copy (clipboard->priv->source);
 	rb_source_paste (playlist_source, entries);
+
+	g_list_foreach (entries, (GFunc)rhythmdb_entry_unref, NULL);
 	g_list_free (entries);
 }
 
@@ -775,19 +828,16 @@ rb_shell_clipboard_playlist_deleted_cb (RBStaticPlaylistSource *source,
 {
 	char *action_name;
 	GtkAction *action;
-	
+
+	/* first rebuild the menu */
+	rebuild_playlist_menu (clipboard);
+
+	/* then remove the 'add to playlist' action for the deleted playlist */
 	action_name = generate_action_name (source, clipboard);
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, action_name);
 	g_assert (action);
 	gtk_action_group_remove_action (clipboard->priv->actiongroup, action);
 	g_free (action_name);
-	g_object_unref (G_OBJECT (action));
-
-	/* this will update the menu */
-	if (clipboard->priv->idle_playlist_id == 0) {
-		clipboard->priv->idle_playlist_id =
-			g_idle_add ((GSourceFunc)rebuild_playlist_menu, clipboard);
-	}
 }
 
 static void
@@ -798,16 +848,16 @@ rb_shell_clipboard_playlist_renamed_cb (RBStaticPlaylistSource *source,
 	char *name, *action_name;
 	GtkAction *action;
 
-	g_object_get (G_OBJECT (source), "name", &name, NULL);
+	g_object_get (source, "name", &name, NULL);
 
 	action_name = generate_action_name (source, clipboard);
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, action_name);
 	g_assert (action);
 	g_free (action_name);
-	
-	g_object_set (G_OBJECT (action), "label", name, NULL);
+
+	g_object_set (action, "label", name, NULL);
 	g_free (name);
-	g_object_unref (G_OBJECT (action));
+	g_object_unref (action);
 }
 
 static void
@@ -819,13 +869,13 @@ rb_shell_clipboard_playlist_visible_cb (RBStaticPlaylistSource *source,
 	char *action_name;
 	GtkAction *action;
 
-	g_object_get (G_OBJECT (source), "visibility", &visible, NULL);
+	g_object_get (source, "visibility", &visible, NULL);
 
 	action_name = generate_action_name (source, clipboard);
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, action_name);
 	g_assert (action);
 	g_free (action_name);
-	
+
 	gtk_action_set_visible (action, visible);
 	g_object_unref (G_OBJECT (action));
 }
@@ -836,7 +886,8 @@ add_playlist_to_menu (GtkTreeModel *model,
 		      GtkTreeIter *iter,
 		      RBShellClipboard *clipboard)
 {
-	RhythmDBEntryType entry_type, source_entry_type;
+	RhythmDBEntryType entry_type;
+	RhythmDBEntryType source_entry_type;
 	RBSource *source = NULL;
 	char *action_name;
 	GtkAction *action;
@@ -845,19 +896,32 @@ add_playlist_to_menu (GtkTreeModel *model,
 	gtk_tree_model_get (GTK_TREE_MODEL (model), iter,
 			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
 
-	if (!RB_IS_STATIC_PLAYLIST_SOURCE (source))
+	if (source == NULL) {
 		return FALSE;
+	}
+
+	if (!RB_IS_STATIC_PLAYLIST_SOURCE (source)) {
+		g_object_unref (source);
+		return FALSE;
+	}
+
+	/* FIXME: allow add-to-playlist for iPods and the like,
+	 * based on the currently selected source
+	 */
 	entry_type = RHYTHMDB_ENTRY_TYPE_SONG;
-	g_object_get (G_OBJECT (source), "entry-type", &source_entry_type, NULL);
-	if (source_entry_type != entry_type)
+	g_object_get (source, "entry-type", &source_entry_type, NULL);
+	if (source_entry_type != entry_type) {
+		g_object_unref (source);
+		g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, source_entry_type);
 		return FALSE;
+	}
 
 	action_name = generate_action_name (RB_STATIC_PLAYLIST_SOURCE (source), clipboard);
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, action_name);
 	if (action == NULL) {
 		char *name;
 
-		g_object_get (G_OBJECT (source), "name", &name, NULL);
+		g_object_get (source, "name", &name, NULL);
 		action = gtk_action_new (action_name, name, NULL, NULL);
 		gtk_action_group_add_action (clipboard->priv->actiongroup, action);
 		g_free (name);
@@ -867,13 +931,13 @@ add_playlist_to_menu (GtkTreeModel *model,
 					 "activate", G_CALLBACK (rb_shell_clipboard_playlist_add_cb),
 					 clipboard, 0);
 
-		g_signal_connect_object (G_OBJECT (source),
+		g_signal_connect_object (source,
 					 "deleted", G_CALLBACK (rb_shell_clipboard_playlist_deleted_cb),
 					 clipboard, 0);
-		g_signal_connect_object (G_OBJECT (source),
+		g_signal_connect_object (source,
 					 "notify::name", G_CALLBACK (rb_shell_clipboard_playlist_renamed_cb),
 					 clipboard, 0);
-		g_signal_connect_object (G_OBJECT (source),
+		g_signal_connect_object (source,
 					 "notify::visibility", G_CALLBACK (rb_shell_clipboard_playlist_visible_cb),
 					 clipboard, 0);
 	}
@@ -885,15 +949,21 @@ add_playlist_to_menu (GtkTreeModel *model,
 				       GTK_UI_MANAGER_AUTO, FALSE);
 	}
 
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, source_entry_type);
+	g_free (action_name);
+	g_object_unref (source);
+
 	return FALSE;
 }
 
-static gboolean
+static void
 rebuild_playlist_menu (RBShellClipboard *clipboard)
 {
 	GtkTreeModel *model = NULL;
 	GObject *sourcelist = NULL;
 
+	rb_debug ("rebuilding add-to-playlist menu");
+
 	if (clipboard->priv->playlist_menu_ui_id != 0) {
 		gtk_ui_manager_remove_ui (clipboard->priv->ui_mgr,
 					  clipboard->priv->playlist_menu_ui_id);
@@ -902,13 +972,28 @@ rebuild_playlist_menu (RBShellClipboard *clipboard)
 			gtk_ui_manager_new_merge_id (clipboard->priv->ui_mgr);
 	}
 
-	g_object_get (G_OBJECT (clipboard->priv->playlist_manager), "sourcelist", &sourcelist, NULL);
-	g_object_get (sourcelist, "model", &model, NULL);
-	g_object_unref (sourcelist);
-	gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc)add_playlist_to_menu, clipboard);
-	g_object_unref (G_OBJECT (model));
+	if (clipboard->priv->playlist_manager != NULL) {
+		g_object_get (clipboard->priv->playlist_manager, "sourcelist", &sourcelist, NULL);
+	}
 
+	if (sourcelist != NULL) {
+		g_object_get (sourcelist, "model", &model, NULL);
+		g_object_unref (sourcelist);
+	}
+
+	if (model != NULL) {
+		gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc)add_playlist_to_menu, clipboard);
+		g_object_unref (model);
+	}
+}
+
+static gboolean
+rebuild_playlist_menu_idle (RBShellClipboard *clipboard)
+{
+	GDK_THREADS_ENTER ();
+	rebuild_playlist_menu (clipboard);
 	clipboard->priv->idle_playlist_id = 0;
+	GDK_THREADS_LEAVE ();
 	return FALSE;
 }
 
@@ -922,7 +1007,6 @@ rb_shell_clipboard_playlist_added_cb (RBPlaylistManager *mgr,
 
 	if (clipboard->priv->idle_playlist_id == 0) {
 		clipboard->priv->idle_playlist_id =
-			g_idle_add ((GSourceFunc)rebuild_playlist_menu, clipboard);
+			g_idle_add ((GSourceFunc)rebuild_playlist_menu_idle, clipboard);
 	}
 }
-
