@@ -20,7 +20,6 @@
  *
  */
 
-
 /*
  * TODO
  *    * handle cases where MusicBrainz returns multiple albums
@@ -37,11 +36,11 @@
 #include <gst/gst.h>
 #include <totem-disc.h>
 
+#include "rb-plugin.h"
 #include "rhythmdb.h"
 #include "eel-gconf-extensions.h"
 #include "rb-audiocd-source.h"
 #include "rb-util.h"
-#include "rb-stock-icons.h"
 #include "rb-debug.h"
 #include "rb-dialog.h"
 
@@ -50,7 +49,6 @@
 #include "sj-structures.h"
 #endif
 
-
 static void rb_audiocd_source_dispose (GObject *object);
 static GObject *rb_audiocd_source_constructor (GType type, guint n_construct_properties,
 					        GObjectConstructParam *construct_properties);
@@ -71,17 +69,15 @@ typedef struct
 	GstElement *pipeline;
 	GstElement *cdda;
 	GstElement *fakesink;
-	
+
 #ifdef HAVE_MUSICBRAINZ
 	SjMetadata *metadata;
 #endif
 } RBAudioCdSourcePrivate;
 
-
-G_DEFINE_TYPE (RBAudioCdSource, rb_audiocd_source, RB_TYPE_REMOVABLE_MEDIA_SOURCE)
+RB_PLUGIN_DEFINE_TYPE (RBAudioCdSource, rb_audiocd_source, RB_TYPE_REMOVABLE_MEDIA_SOURCE)
 #define AUDIOCD_SOURCE_GET_PRIVATE(o)   (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_AUDIOCD_SOURCE, RBAudioCdSourcePrivate))
 
-
 static void
 rb_audiocd_source_class_init (RBAudioCdSourceClass *klass)
 {
@@ -111,7 +107,7 @@ rb_audiocd_source_init (RBAudioCdSource *self)
 
 }
 
-static void 
+static void
 rb_audiocd_source_dispose (GObject *object)
 {
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (object);
@@ -132,14 +128,12 @@ rb_audiocd_source_dispose (GObject *object)
 	G_OBJECT_CLASS (rb_audiocd_source_parent_class)->dispose (object);
 }
 
-
-
 static GObject *
 rb_audiocd_source_constructor (GType type,
 			       guint n_construct_properties,
 			       GObjectConstructParam *construct_properties)
 {
-	RBAudioCdSource *source; 
+	RBAudioCdSource *source;
 	RBEntryView *entry_view;
 
 	source = RB_AUDIOCD_SOURCE (G_OBJECT_CLASS (rb_audiocd_source_parent_class)->
@@ -164,10 +158,14 @@ rb_audiocd_source_new (RBShell *shell,
 	char *device_path;
 	GObject *source;
 	RhythmDBEntryType entry_type;
+	RhythmDB *db;
 
 	g_assert (rb_audiocd_is_volume_audiocd (volume));
 
-	entry_type =  rhythmdb_entry_register_type (NULL);
+	g_object_get (shell, "db", &db, NULL);
+	entry_type =  rhythmdb_entry_register_type (db, NULL);
+	g_object_unref (db);
+
 	entry_type->can_sync_metadata = (RhythmDBEntryCanSyncFunc)rb_true_function;
 	/* TODO same the metadata somewhere */
 	entry_type->sync_metadata = (RhythmDBEntrySyncFunc)rb_null_function;
@@ -189,7 +187,7 @@ rb_audiocd_source_new (RBShell *shell,
 	return RB_REMOVABLE_MEDIA_SOURCE (source);
 }
 
-static void 
+static void
 entry_set_string_prop (RhythmDB *db,
 		       RhythmDBEntry *entry,
 		       gboolean is_inserted,
@@ -211,7 +209,7 @@ entry_set_string_prop (RhythmDB *db,
 	if (is_inserted)
 		rhythmdb_entry_set (RHYTHMDB (db), entry, propid, &value);
 	else
-		rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry, propid, &value);
+		rhythmdb_entry_set (RHYTHMDB (db), entry, propid, &value);
 	g_value_unset (&value);
 }
 
@@ -232,6 +230,7 @@ rb_audiocd_create_track_entry (RBAudioCdSource *source,
 	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
 	rb_debug ("Audio CD - create entry for track %d from %s", track_number, audio_path);
 	entry = rhythmdb_entry_new (db, entry_type, audio_path);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 	if (entry == NULL) {
 		g_free (audio_path);
 		return NULL;
@@ -240,8 +239,8 @@ rb_audiocd_create_track_entry (RBAudioCdSource *source,
 	/* generate track # */
 	g_value_init (&value, G_TYPE_ULONG);
 	g_value_set_ulong (&value, track_number);
-	rhythmdb_entry_set_uninserted (db, entry,
-			    RHYTHMDB_PROP_TRACK_NUMBER, 
+	rhythmdb_entry_set (db, entry,
+			    RHYTHMDB_PROP_TRACK_NUMBER,
 			    &value);
 	g_value_unset (&value);
 
@@ -249,8 +248,8 @@ rb_audiocd_create_track_entry (RBAudioCdSource *source,
 	g_value_init (&value, G_TYPE_STRING);
 	str = g_strdup_printf (_("Track %u"), track_number);
 	g_value_take_string (&value, str);
-	rhythmdb_entry_set_uninserted (db, entry,
-			    RHYTHMDB_PROP_TITLE, 
+	rhythmdb_entry_set (db, entry,
+			    RHYTHMDB_PROP_TITLE,
 			    &value);
 	g_value_unset (&value);
 
@@ -263,7 +262,7 @@ rb_audiocd_create_track_entry (RBAudioCdSource *source,
 #ifdef HAVE_GSTREAMER_0_8
 		GstEvent *event;
 
-		event = gst_event_new_seek (track_format | GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH, 
+		event = gst_event_new_seek (track_format | GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH,
 					    (guint64) track_number - 1);
 		result = gst_element_send_event (priv->fakesink, event);
 
@@ -279,8 +278,8 @@ rb_audiocd_create_track_entry (RBAudioCdSource *source,
 		if (result) {
 			g_value_init (&value, G_TYPE_ULONG);
 			g_value_set_ulong (&value, (gulong)(duration / GST_SECOND));
-			rhythmdb_entry_set_uninserted (db, entry,
-					    RHYTHMDB_PROP_DURATION, 
+			rhythmdb_entry_set (db, entry,
+					    RHYTHMDB_PROP_DURATION,
 					    &value);
 			g_value_unset (&value);
 		} else {
@@ -379,6 +378,19 @@ rb_audiocd_scan_songs (RBAudioCdSource *source,
 	return ok;
 }
 
+static RhythmDB *
+get_db_for_source (RBAudioCdSource *source)
+{
+	RBShell *shell;
+	RhythmDB *db;
+
+  	g_object_get (source, "shell", &shell, NULL);
+  	g_object_get (shell, "db", &db, NULL);
+  	g_object_unref (shell);
+
+        return db;
+}
+
 #ifdef HAVE_MUSICBRAINZ
 static void
 metadata_cb (SjMetadata *metadata,
@@ -388,7 +400,6 @@ metadata_cb (SjMetadata *metadata,
 {
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 	GList *cd_track = priv->tracks;
-	RBShell *shell;
 	RhythmDB *db;
 	GValue true_value = {0,};
 
@@ -406,9 +417,7 @@ metadata_cb (SjMetadata *metadata,
 		return;
 	}
 
-	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-	g_object_get (G_OBJECT (shell), "db", &db, NULL);
-	g_object_unref (G_OBJECT (shell));
+	db = get_db_for_source (source);
 
 	g_value_init (&true_value, G_TYPE_BOOLEAN);
 	g_value_set_boolean (&true_value, TRUE);
@@ -432,15 +441,16 @@ metadata_cb (SjMetadata *metadata,
 			entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_ARTIST, track->artist);
 			entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_ALBUM, album->title);
 			entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_GENRE, album->genre);
+			entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_MUSICBRAINZ_TRACKID, track->track_id);
 
 			g_value_init (&value, G_TYPE_ULONG);
 			g_value_set_ulong (&value, track->duration);
 			rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_DURATION, &value);
 			g_value_unset (&value);
-			
+
 			/*album->release_date (could potentially have multiple values)*/
 			/* in current sj-structures.h, however, it does not */
-			
+
 			if (album->release_date) {
 				GType type = rhythmdb_get_property_type (db, RHYTHMDB_PROP_DATE);
 				g_value_init (&value, type);
@@ -471,7 +481,8 @@ metadata_cb (SjMetadata *metadata,
 
 	g_object_unref (metadata);
 	priv->metadata = NULL;
-	g_object_unref (G_OBJECT (db));
+
+	g_object_unref (db);
 }
 
 static void
@@ -520,17 +531,14 @@ static gpointer
 rb_audiocd_load_songs (RBAudioCdSource *source)
 {
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
-	RBShell *shell;
 	RhythmDB *db;
 	GnomeVFSVolume *volume;
-	
-	g_object_get (G_OBJECT (source), "volume", &volume, NULL);
+
+	g_object_get (source, "volume", &volume, NULL);
 	priv->device_path = gnome_vfs_volume_get_device_path (volume);
-	g_object_unref (G_OBJECT (volume));
-	
-	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-	g_object_get (G_OBJECT (shell), "db", &db, NULL);
-	g_object_unref (G_OBJECT (shell));
+	g_object_unref (volume);
+
+	db = get_db_for_source (source);
 
 	rb_debug ("loading Audio CD from %s", priv->device_path);
 	/* create a cdda gstreamer element, to get cd info from */
@@ -547,18 +555,18 @@ rb_audiocd_load_songs (RBAudioCdSource *source)
 #elif HAVE_GSTREAMER_0_10
 	g_object_set (G_OBJECT (priv->cdda), "device", priv->device_path, NULL);
 #endif
-	
+
 	priv->pipeline = gst_pipeline_new ("pipeline");
 	priv->fakesink = gst_element_factory_make ("fakesink", "fakesink");
 	gst_bin_add_many (GST_BIN (priv->pipeline), priv->cdda, priv->fakesink, NULL);
 	gst_element_link (priv->cdda, priv->fakesink);
-	
+
 	if (rb_audiocd_scan_songs (source, db))
 		rb_audiocd_load_metadata (source, db);
 
 error_out:
-	g_object_unref (G_OBJECT (db));
-	g_object_unref (G_OBJECT (source));
+	g_object_unref (db);
+	g_object_unref (source);
 
 	return NULL;
 }
@@ -567,24 +575,23 @@ static void
 impl_delete_thyself (RBSource *source)
 {
 	RhythmDB *db;
-	RBShell *shell;
 	RhythmDBEntryType entry_type;
+
 	rb_debug ("audio cd ejected\n");
 
 	/* cancel the loading of metadata */
 	rb_audiocd_load_metadata_cancel (RB_AUDIOCD_SOURCE (source));
 
-	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-	g_object_get (G_OBJECT (shell), "db", &db, NULL);
-	g_object_unref (G_OBJECT (shell));
+	db = get_db_for_source (RB_AUDIOCD_SOURCE (source));
 
-	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
+	g_object_get (source, "entry-type", &entry_type, NULL);
 	rhythmdb_entry_delete_by_type (db, entry_type);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+
 	rhythmdb_commit (db);
 	g_object_unref (db);
 }
 
-
 gboolean
 rb_audiocd_is_volume_audiocd (GnomeVFSVolume *volume)
 {
@@ -616,7 +623,6 @@ rb_audiocd_is_volume_audiocd (GnomeVFSVolume *volume)
 	return result;
 }
 
-
 static gboolean
 impl_show_popup (RBSource *source)
 {
@@ -636,4 +642,3 @@ impl_get_ui_actions (RBSource *source)
 
 	return actions;
 }
-
