@@ -21,7 +21,8 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
 
 #include <gtk/gtk.h>
@@ -35,7 +36,6 @@
 #include "rhythmdb-query-model.h"
 #include "rb-property-view.h"
 #include "rb-glade-helpers.h"
-#include "rb-stock-icons.h"
 #include "rb-entry-view.h"
 #include "rb-library-browser.h"
 #include "rb-util.h"
@@ -93,6 +93,7 @@ static void impl_song_properties (RBSource *source);
 static gboolean impl_show_popup (RBSource *source);
 static GList *impl_get_search_actions (RBSource *source);
 static void impl_browser_toggled (RBSource *asource, gboolean disclosed);
+static void default_show_entry_popup (RBBrowserSource *source);
 
 void rb_browser_source_browser_views_activated_cb (GtkWidget *widget,
 						 RBBrowserSource *source);
@@ -108,17 +109,13 @@ static void rb_browser_source_do_query (RBBrowserSource *source,
 struct RBBrowserSourcePrivate
 {
 	RhythmDB *db;
-	
+
 	RBLibraryBrowser *browser;
 	GtkWidget *vbox;
 
-	GdkPixbuf *pixbuf;
-
 	RBEntryView *songs;
 	GtkWidget *paned;
 
-	gboolean lock;
-
 	char *search_text;
 	RhythmDBQueryModel *cached_all_query;
 	RhythmDBPropType search_prop;
@@ -128,13 +125,13 @@ struct RBBrowserSourcePrivate
 
 	GtkActionGroup *action_group;
 	GtkActionGroup *search_action_group;
-	
-	RhythmDBEntryType entry_type;
 
 	char *sorting_key;
 	guint state_paned_notify_id;
 	guint state_browser_notify_id;
 	guint state_sorting_notify_id;
+
+	gboolean dispose_has_run;
 };
 
 #define RB_BROWSER_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_BROWSER_SOURCE, RBBrowserSourcePrivate))
@@ -165,11 +162,9 @@ static const GtkTargetEntry songs_view_drag_types[] = {{  "text/uri-list", 0, 0
 enum
 {
 	PROP_0,
-	PROP_ENTRY_TYPE,
 	PROP_SORTING_KEY
 };
 
-
 G_DEFINE_ABSTRACT_TYPE (RBBrowserSource, rb_browser_source, RB_TYPE_SOURCE)
 
 static void
@@ -199,20 +194,13 @@ rb_browser_source_class_init (RBBrowserSourceClass *klass)
 	source_class->impl_can_add_to_queue = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_delete = impl_delete;
-	source_class->impl_have_url = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_get_search_actions = impl_get_search_actions;
 	source_class->impl_browser_toggled = impl_browser_toggled;
 
 	klass->impl_get_paned_key = (RBBrowserSourceStringFunc)rb_null_function;
 	klass->impl_has_drop_support = (RBBrowserSourceFeatureFunc) rb_false_function;
-
-	g_object_class_install_property (object_class,
-					 PROP_ENTRY_TYPE,
-					 g_param_spec_pointer ("entry-type",
-							       "Entry type",
-							       "Type of the entries which should be displayed by this source",
-							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	klass->impl_show_entry_popup = default_show_entry_popup;
 
 	g_object_class_install_property (object_class,
 					 PROP_SORTING_KEY,
@@ -234,11 +222,6 @@ rb_browser_source_init (RBBrowserSource *source)
 
 	source->priv->vbox = gtk_vbox_new (FALSE, 5);
 
-	/* Default value, should be overridden at object construction by the 
-	 * "entry-type" property
-	 */
-	source->priv->entry_type = RHYTHMDB_ENTRY_TYPE_INVALID;
-
 	gtk_container_add (GTK_CONTAINER (source), source->priv->vbox);
 }
 
@@ -248,21 +231,28 @@ rb_browser_source_dispose (GObject *object)
 	RBBrowserSource *source;
 	source = RB_BROWSER_SOURCE (object);
 
-	if (source->priv->db) {
+	if (source->priv->dispose_has_run) {
+		/* If dispose did already run, return. */
+		return;
+	}
+	/* Make sure dispose does not run twice. */
+	source->priv->dispose_has_run = TRUE;
+
+	if (source->priv->db != NULL) {
 		g_object_unref (source->priv->db);
 		source->priv->db = NULL;
 	}
 
-	if (source->priv->search_text) {
+	if (source->priv->search_text != NULL) {
 		g_free (source->priv->search_text);
 		source->priv->search_text = NULL;
 	}
-	
-	if (source->priv->cached_all_query) {
-		g_object_unref (G_OBJECT (source->priv->cached_all_query));
+
+	if (source->priv->cached_all_query != NULL) {
+		g_object_unref (source->priv->cached_all_query);
 		source->priv->cached_all_query = NULL;
 	}
-	
+
 	G_OBJECT_CLASS (rb_browser_source_parent_class)->dispose (object);
 }
 
@@ -284,6 +274,10 @@ rb_browser_source_finalize (GObject *object)
 	eel_gconf_notification_remove (source->priv->state_paned_notify_id);
 	eel_gconf_notification_remove (source->priv->state_sorting_notify_id);
 
+	if (source->priv->action_group != NULL) {
+		g_object_unref (source->priv->action_group);
+	}
+
 	G_OBJECT_CLASS (rb_browser_source_parent_class)->finalize (object);
 }
 
@@ -292,8 +286,19 @@ rb_browser_source_songs_show_popup_cb (RBEntryView *view,
 				       gboolean over_entry,
 				       RBBrowserSource *source)
 {
-	if (over_entry)
-		_rb_source_show_popup (RB_SOURCE (source), "/BrowserSourceViewPopup");
+	if (over_entry) {
+		RBBrowserSourceClass *klass = RB_BROWSER_SOURCE_GET_CLASS (source);
+
+		klass->impl_show_entry_popup (source);
+	} else {
+		rb_source_show_popup (RB_SOURCE (source));
+	}
+}
+
+static void
+default_show_entry_popup (RBBrowserSource *source)
+{
+	_rb_source_show_popup (RB_SOURCE (source), "/BrowserSourceViewPopup");
 }
 
 static RhythmDBPropType
@@ -307,7 +312,7 @@ search_action_to_prop (GtkAction *action)
 	if (name == NULL) {
 		prop = RHYTHMDB_PROP_SEARCH_MATCH;
 	} else if (strcmp (name, "BrowserSourceSearchAll") == 0) {
-		prop = RHYTHMDB_PROP_SEARCH_MATCH;		
+		prop = RHYTHMDB_PROP_SEARCH_MATCH;
 	} else if (strcmp (name, "BrowserSourceSearchArtists") == 0) {
 		prop = RHYTHMDB_PROP_ARTIST_FOLDED;
 	} else if (strcmp (name, "BrowserSourceSearchAlbums") == 0) {
@@ -329,7 +334,7 @@ search_action_changed (GtkRadioAction  *action,
 	gboolean         active;
 	RBBrowserSource *source;
 
-	g_object_get (G_OBJECT (shell), "selected-source", &source, NULL);
+	g_object_get (shell, "selected-source", &source, NULL);
 
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (current));
 
@@ -339,6 +344,10 @@ search_action_changed (GtkRadioAction  *action,
 		rb_browser_source_do_query (source, FALSE);
 		rb_source_notify_filter_changed (RB_SOURCE (source));
 	}
+
+	if (source != NULL) {
+		g_object_unref (source);
+	}
 }
 
 static GObject *
@@ -351,16 +360,20 @@ rb_browser_source_constructor (GType type,
 	RBShell *shell;
 	GObject *shell_player;
 	char *browser_key;
+	RhythmDBEntryType entry_type;
 
 	klass = RB_BROWSER_SOURCE_CLASS (g_type_class_peek (RB_TYPE_BROWSER_SOURCE));
 
 	source = RB_BROWSER_SOURCE (G_OBJECT_CLASS (rb_browser_source_parent_class)
 			->constructor (type, n_construct_properties, construct_properties));
 
-	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-	g_object_get (G_OBJECT (shell), "db", &source->priv->db, NULL);
+	g_object_get (source,
+		      "shell", &shell,
+		      "entry-type", &entry_type,
+		      NULL);
+	g_object_get (shell, "db", &source->priv->db, NULL);
 	shell_player = rb_shell_get_player (shell);
-	
+
 	source->priv->action_group = _rb_source_register_action_group (RB_SOURCE (source),
 								       "BrowserSourceActions",
 								       rb_browser_source_actions,
@@ -373,11 +386,11 @@ rb_browser_source_constructor (GType type,
 					    0,
 					    (GCallback)search_action_changed,
 					    shell);
-	g_object_unref (G_OBJECT (shell));
+	g_object_unref (shell);
 
 	source->priv->paned = gtk_vpaned_new ();
 
-	source->priv->browser = rb_library_browser_new (source->priv->db, source->priv->entry_type);
+	source->priv->browser = rb_library_browser_new (source->priv->db, entry_type);
 	gtk_paned_pack1 (GTK_PANED (source->priv->paned), GTK_WIDGET (source->priv->browser), TRUE, FALSE);
 	gtk_container_child_set (GTK_CONTAINER (source->priv->paned),
 				 GTK_WIDGET (source->priv->browser),
@@ -449,7 +462,6 @@ rb_browser_source_constructor (GType type,
 					 source, 0);
 	}
 
-
 	/* this gets emitted when the paned thingie is moved */
 	g_signal_connect_object (G_OBJECT (source->priv->songs),
 				 "size_allocate",
@@ -465,13 +477,13 @@ rb_browser_source_constructor (GType type,
 	rb_library_browser_set_model (source->priv->browser, source->priv->cached_all_query, TRUE);
 	rhythmdb_do_full_query_async (source->priv->db,
 				      RHYTHMDB_QUERY_RESULTS (source->priv->cached_all_query),
-				      RHYTHMDB_QUERY_PROP_EQUALS, RHYTHMDB_PROP_TYPE, source->priv->entry_type,
+				      RHYTHMDB_QUERY_PROP_EQUALS, RHYTHMDB_PROP_TYPE, entry_type,
 				      RHYTHMDB_QUERY_END);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
 	return G_OBJECT (source);
 }
 
-
 static void
 rb_browser_source_set_property (GObject *object,
 				guint prop_id,
@@ -481,9 +493,6 @@ rb_browser_source_set_property (GObject *object,
 	RBBrowserSource *source = RB_BROWSER_SOURCE (object);
 
 	switch (prop_id) {
-	case PROP_ENTRY_TYPE:
-		source->priv->entry_type = g_value_get_pointer (value);
-		break;
 	case PROP_SORTING_KEY:
 		g_free (source->priv->sorting_key);
 		source->priv->sorting_key = g_strdup (g_value_get_string (value));
@@ -503,9 +512,6 @@ rb_browser_source_get_property (GObject *object,
 	RBBrowserSource *source = RB_BROWSER_SOURCE (object);
 
 	switch (prop_id) {
-	case PROP_ENTRY_TYPE:
-		g_value_set_pointer (value, source->priv->entry_type);
-		break;
 	case PROP_SORTING_KEY:
 		g_value_set_string (value, source->priv->sorting_key);
 		break;
@@ -530,7 +536,7 @@ rb_browser_source_cmd_choose_genre (GtkAction *action,
 	view = rb_library_browser_get_property_view (source->priv->browser, RHYTHMDB_PROP_GENRE);
 	if (view)
 		rb_property_view_set_selection (view, props);
-	
+
 	rb_list_deep_free (props);
 	g_object_unref (source);
 }
@@ -539,13 +545,13 @@ static void
 rb_browser_source_cmd_choose_artist (GtkAction *action,
 				     RBShell *shell)
 {
-	GList *props;	
+	GList *props;
 	RBBrowserSource *source;
 	RBPropertyView *view;
 
 	rb_debug ("choosing artist");
 
-	g_object_get (G_OBJECT (shell), "selected-source", &source, NULL);
+	g_object_get (shell, "selected-source", &source, NULL);
 	props = rb_source_gather_selected_properties (RB_SOURCE (source), RHYTHMDB_PROP_ARTIST);
 	view = rb_library_browser_get_property_view (source->priv->browser, RHYTHMDB_PROP_ARTIST);
 	if (view)
@@ -559,7 +565,7 @@ static void
 rb_browser_source_cmd_choose_album (GtkAction *action,
 				    RBShell *shell)
 {
-	GList *props;	
+	GList *props;
 	RBBrowserSource *source;
 	RBPropertyView *view;
 
@@ -611,7 +617,7 @@ impl_search (RBSource *asource, const char *search_text)
 			subset = (g_str_has_prefix (source->priv->search_text, old_search_text));
 	}
 	g_free (old_search_text);
-	
+
 	/* we can't do subset searches until the original query is complete, because they
 	 * reuse the query model.
 	 */
@@ -624,7 +630,6 @@ impl_search (RBSource *asource, const char *search_text)
 	}
 }
 
-
 static RBEntryView *
 impl_get_entry_view (RBSource *asource)
 {
@@ -674,6 +679,7 @@ impl_delete (RBSource *asource)
 		rhythmdb_entry_delete (source->priv->db, tem->data);
 		rhythmdb_commit (source->priv->db);
 	}
+	g_list_foreach (sel, (GFunc)rhythmdb_entry_unref, NULL);
 	g_list_free (sel);
 }
 
@@ -809,12 +815,12 @@ rb_browser_source_browser_changed_cb (RBLibraryBrowser *browser,
 				      RBBrowserSource *source)
 {
 	RhythmDBQueryModel *query_model;
-	
-	g_object_get (G_OBJECT (browser), "output-model", &query_model, NULL);
+
+	g_object_get (browser, "output-model", &query_model, NULL);
 	rb_entry_view_set_model (source->priv->songs, query_model);
-	g_object_set (RB_SOURCE (source), "query-model", query_model, NULL);
-	g_object_unref (G_OBJECT (query_model));
-	
+	g_object_set (source, "query-model", query_model, NULL);
+	g_object_unref (query_model);
+
 	rb_source_notify_filter_changed (RB_SOURCE (source));
 }
 
@@ -837,7 +843,7 @@ rb_browser_source_do_query (RBBrowserSource *source, gboolean subset)
 	RhythmDBQueryModel *query_model;
 	GPtrArray *query;
 	RhythmDBEntryType entry_type;
-	
+
 	/* use the cached 'all' query to optimise the no-search case */
 	if (!source->priv->search_text) {
 		rb_library_browser_set_model (source->priv->browser,
@@ -845,8 +851,8 @@ rb_browser_source_do_query (RBBrowserSource *source, gboolean subset)
 					      FALSE);
 		return;
 	}
-	
-	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
+
+	g_object_get (source, "entry-type", &entry_type, NULL);
 	query = rhythmdb_query_parse (source->priv->db,
 				      RHYTHMDB_QUERY_PROP_EQUALS,
 				      RHYTHMDB_PROP_TYPE,
@@ -855,29 +861,38 @@ rb_browser_source_do_query (RBBrowserSource *source, gboolean subset)
 				      source->priv->search_prop,
 				      source->priv->search_text,
 				      RHYTHMDB_QUERY_END);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
 	if (subset) {
 		/* if we're appending text to an existing search string, the results will be a subset
-		 * of the existing results, so we can just change the query on the existing query
-		 * model and reapply the query.
+		 * of the existing results, so rather than doing a whole new query, we can copy the
+		 * results to a new query model with a more restrictive query.
 		 */
-		g_object_get (G_OBJECT (source->priv->browser), "input-model", &query_model, NULL);
-		g_object_set (G_OBJECT (query_model), "query", query, NULL);
-		rhythmdb_query_model_reapply_query (query_model, FALSE);
-		g_object_unref (G_OBJECT (query_model));
-	} else {/* otherwise build a query based on the search text and feed it to the browser */
+		RhythmDBQueryModel *old;
+		g_object_get (source->priv->browser, "input-model", &old, NULL);
+
+		query_model = rhythmdb_query_model_new_empty (source->priv->db);
+		g_object_set (query_model, "query", query, NULL);
+		rhythmdb_query_model_copy_contents (query_model, old);
+		g_object_unref (old);
+
+		rb_library_browser_set_model (source->priv->browser, query_model, FALSE);
+		g_object_unref (query_model);
+
+	} else {
+		/* otherwise build a query based on the search text and feed it to the browser */
 		query_model = rhythmdb_query_model_new_empty (source->priv->db);
 		rb_library_browser_set_model (source->priv->browser, query_model, TRUE);
 		source->priv->query_active = TRUE;
 		source->priv->search_on_completion = FALSE;
-		g_signal_connect_object (G_OBJECT (query_model),
+		g_signal_connect_object (query_model,
 					 "complete", G_CALLBACK (rb_browser_source_query_complete_cb),
 					 source, 0);
 		rhythmdb_do_full_query_async_parsed (source->priv->db,
 						     RHYTHMDB_QUERY_RESULTS (query_model),
 						     query);
-		g_object_unref (G_OBJECT (query_model));
+		g_object_unref (query_model);
 	}
+
 	rhythmdb_query_free (query);
 }
-
