@@ -26,6 +26,11 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <libgnomeui/gnome-password-dialog.h>
+
+#ifdef WITH_GNOME_KEYRING
+#include <gnome-keyring.h>
+#endif
 
 #include "rhythmdb.h"
 #include "rb-shell.h"
@@ -37,6 +42,9 @@
 #include "rb-file-helpers.h"
 #include "rb-dialog.h"
 #include "rb-preferences.h"
+#ifdef HAVE_GSTREAMER
+#include "rb-daap-src.h"
+#endif
 
 #include "rb-daap-connection.h"
 #include "rb-daap-mdns-browser.h"
@@ -44,7 +52,6 @@
 
 #include "rb-static-playlist-source.h"
 
-
 static void rb_daap_source_dispose (GObject *object);
 static void rb_daap_source_set_property  (GObject *object,
 					  guint prop_id,
@@ -68,7 +75,6 @@ static void rb_daap_source_disconnect (RBDAAPSource *daap_source);
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/daap/paned_position"
 #define CONF_STATE_SHOW_BROWSER CONF_PREFIX "/state/daap/show_browser"
 
-
 static RBDaapMdnsBrowser *mdns_browser = NULL;
 
 static GHashTable *source_lookup = NULL;
@@ -95,6 +101,7 @@ struct RBDAAPSourcePrivate
 	const char *connection_status;
 	float connection_progress;
 
+	gboolean tried_password;
 	gboolean disconnecting;
 };
 
@@ -110,17 +117,33 @@ G_DEFINE_TYPE (RBDAAPSource, rb_daap_source, RB_TYPE_BROWSER_SOURCE)
 
 static GtkActionEntry rb_daap_source_actions [] =
 {
-	{ "DaapSourceDisconnect", GTK_STOCK_DISCONNECT, N_("_Disconnect"), NULL, 
-	  N_("Disconnect from DAAP share"), 
+	{ "DaapSourceDisconnect", GTK_STOCK_DISCONNECT, N_("_Disconnect"), NULL,
+	  N_("Disconnect from DAAP share"),
 	  G_CALLBACK (rb_daap_source_cmd_disconnect) },
 };
 GtkActionGroup *daap_action_group;
 guint daap_ui_merge_id;
 
-static RhythmDBEntryType
-rhythmdb_entry_daap_type_new (void)
+static void
+rb_daap_source_dispose (GObject *object)
 {
-	return rhythmdb_entry_register_type (NULL);
+	RBDAAPSource *source = RB_DAAP_SOURCE (object);
+
+	/* we should already have been disconnected */
+	g_assert (source->priv->connection == NULL);
+
+	G_OBJECT_CLASS (rb_daap_source_parent_class)->dispose (object);
+}
+
+static void
+rb_daap_source_finalize (GObject *object)
+{
+	RBDAAPSource *source = RB_DAAP_SOURCE (object);
+
+	g_free (source->priv->service_name);
+	g_free (source->priv->host);
+
+	G_OBJECT_CLASS (rb_daap_source_parent_class)->finalize (object);
 }
 
 static void
@@ -131,6 +154,7 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 	RBBrowserSourceClass *browser_source_class = RB_BROWSER_SOURCE_CLASS (klass);
 
 	object_class->dispose      = rb_daap_source_dispose;
+	object_class->finalize     = rb_daap_source_finalize;
 	object_class->get_property = rb_daap_source_get_property;
 	object_class->set_property = rb_daap_source_set_property;
 
@@ -150,7 +174,6 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 	browser_source_class->impl_get_paned_key = rb_daap_source_get_paned_key;
 	browser_source_class->impl_has_drop_support = (RBBrowserSourceFeatureFunc) rb_false_function;
 
-
 	g_object_class_install_property (object_class,
 					 PROP_SERVICE_NAME,
 					 g_param_spec_string ("service-name",
@@ -158,7 +181,7 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 							      "mDNS/DNS-SD service name of the share",
 							      NULL,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-	
+
 	g_object_class_install_property (object_class,
 					 PROP_HOST,
 					 g_param_spec_string ("host",
@@ -185,32 +208,21 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 							       G_PARAM_READWRITE));
 
 	g_type_class_add_private (klass, sizeof (RBDAAPSourcePrivate));
+
+#ifdef HAVE_GSTREAMER
+	/* small hack to ensure the linker doesn't drop the daap src code */
+	rb_daap_src_get_type ();
+#endif
 }
 
 static void
 rb_daap_source_init (RBDAAPSource *source)
 {
-	source->priv = G_TYPE_INSTANCE_GET_PRIVATE (source, 
-						    RB_TYPE_DAAP_SOURCE, 
+	source->priv = G_TYPE_INSTANCE_GET_PRIVATE (source,
+						    RB_TYPE_DAAP_SOURCE,
 						    RBDAAPSourcePrivate);
 }
 
-static void
-rb_daap_source_dispose (GObject *object)
-{
-	RBDAAPSource *source = RB_DAAP_SOURCE (object);
-
-	if (source->priv) {
-		/* we should already have been disconnected */
-		g_assert (source->priv->connection == NULL);
-	
-		g_free (source->priv->service_name);
-		g_free (source->priv->host);
-	}
-
-	G_OBJECT_CLASS (rb_daap_source_parent_class)->dispose (object);
-}
-
 static void
 rb_daap_source_set_property (GObject *object,
 			    guint prop_id,
@@ -220,7 +232,7 @@ rb_daap_source_set_property (GObject *object,
 	RBDAAPSource *source = RB_DAAP_SOURCE (object);
 
 	switch (prop_id) {
-		case PROP_SERVICE_NAME: 
+		case PROP_SERVICE_NAME:
 			source->priv->service_name = g_value_dup_string (value);
 			break;
 		case PROP_HOST:
@@ -275,15 +287,10 @@ rb_daap_get_icon (gboolean password_protected,
 		  gboolean connected)
 {
 	GdkPixbuf *icon;
-	GtkIconTheme *theme;
-	gint size;
 
 	g_return_val_if_fail (daap_share_pixbuf != NULL, NULL);
 	g_return_val_if_fail (daap_share_locked_pixbuf != NULL, NULL);
 
-	theme = gtk_icon_theme_get_default ();
-	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
-
 	if (! password_protected) {
 		icon = g_object_ref (daap_share_pixbuf);
 	} else if (connected) {
@@ -306,11 +313,13 @@ rb_daap_source_new (RBShell *shell,
 	RBSource *source;
 	RhythmDBEntryType type;
 	GdkPixbuf *icon;
+	RhythmDB *db;
 
-	type = rhythmdb_entry_daap_type_new ();
+	g_object_get (shell, "db", &db, NULL);
+	type = rhythmdb_entry_register_type (db, NULL);
+	g_object_unref (db);
 
 	icon = rb_daap_get_icon (password_protected, FALSE);
-
 	source = RB_SOURCE (g_object_new (RB_TYPE_DAAP_SOURCE,
 					  "service-name", service_name,
 					  "name", name,
@@ -335,7 +344,6 @@ rb_daap_source_new (RBShell *shell,
 	return source;
 }
 
-
 static RBSource *
 find_source_by_service_name (const char *service_name)
 {
@@ -437,8 +445,8 @@ start_browsing (RBShell *shell)
 	}
 
 	source_lookup = g_hash_table_new_full ((GHashFunc)g_str_hash,
-					       (GEqualFunc)g_str_equal, 
-					       (GDestroyNotify)g_free, 
+					       (GEqualFunc)g_str_equal,
+					       (GDestroyNotify)g_free,
 					       (GDestroyNotify)remove_source);
 
 }
@@ -556,6 +564,10 @@ create_pixbufs (void)
 	emblem = gtk_icon_theme_load_icon (theme, "stock_lock", size, 0, NULL);
 
 	daap_share_locked_pixbuf = composite_icons (daap_share_pixbuf, emblem);
+
+	if (emblem != NULL) {
+		g_object_unref (emblem);
+	}
 }
 
 static void
@@ -564,6 +576,7 @@ destroy_pixbufs (void)
 	if (daap_share_pixbuf != NULL) {
 		g_object_unref (daap_share_pixbuf);
 	}
+
 	daap_share_pixbuf = NULL;
 
 	if (daap_share_locked_pixbuf != NULL) {
@@ -601,8 +614,8 @@ rb_daap_sources_init (RBShell *shell)
 	create_pixbufs ();
 
 	/* add UI */
-	g_object_get (G_OBJECT (shell),
-		      "ui-manager", &uimanager, 
+	g_object_get (shell,
+		      "ui-manager", &uimanager,
 		      NULL);
 	daap_action_group = gtk_action_group_new ("DaapActions");
 	gtk_action_group_set_translation_domain (daap_action_group,
@@ -614,7 +627,7 @@ rb_daap_sources_init (RBShell *shell)
 	daap_ui_merge_id = gtk_ui_manager_add_ui_from_file (uimanager,
 							    rb_file ("daap-ui.xml"),
 							    NULL);
-	g_object_unref (G_OBJECT (uimanager));
+	g_object_unref (uimanager);
 
 	return NULL;
 }
@@ -624,13 +637,14 @@ rb_daap_sources_shutdown (RBShell *shell)
 {
 	GtkUIManager *uimanager = NULL;
 
-	rb_debug ("Shutting down DAAP sources");
+	if (daap_was_shutdown)
+		return;
 
-	g_assert (!daap_was_shutdown);
+	rb_debug ("Shutting down DAAP sources");
 	daap_was_shutdown = TRUE;
 
-	g_object_get (G_OBJECT (shell),
-		      "ui-manager", &uimanager, 
+	g_object_get (shell,
+		      "ui-manager", &uimanager,
 		      NULL);
 
 	if (mdns_browser) {
@@ -645,7 +659,7 @@ rb_daap_sources_shutdown (RBShell *shell)
 	gtk_ui_manager_remove_ui (uimanager, daap_ui_merge_id);
 	gtk_ui_manager_remove_action_group (uimanager, daap_action_group);
 
-	g_object_unref (G_OBJECT (uimanager));
+	g_object_unref (uimanager);
 	g_object_unref (shell);
 
 	destroy_pixbufs ();
@@ -654,13 +668,93 @@ rb_daap_sources_shutdown (RBShell *shell)
 static char *
 connection_auth_cb (RBDAAPConnection *connection,
 		    const char       *name,
-		    RBSource         *source)
+		    RBDAAPSource     *source)
 {
-	char      *password;
-	GtkWindow *parent;
+	gchar *password;
+#ifdef WITH_GNOME_KEYRING
+	GnomeKeyringResult keyringret;
+	gchar *keyring;
+	GList *list;
 
-	parent = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (source)));
-	password = rb_daap_password_dialog_new_run (parent, name);
+	if (!source->priv->tried_password) {
+		gnome_keyring_get_default_keyring_sync (&keyring);
+		keyringret = gnome_keyring_find_network_password_sync (
+				NULL,
+				"DAAP", name,
+				NULL, "daap", 
+				NULL, 0, &list);
+	} else {
+		keyringret = GNOME_KEYRING_RESULT_CANCELLED;
+	}
+
+	if (keyringret == GNOME_KEYRING_RESULT_OK) {
+		GnomeKeyringNetworkPasswordData *pwd_data;
+
+		pwd_data = (GnomeKeyringNetworkPasswordData*)list->data;
+		password = g_strdup (pwd_data->password);
+		source->priv->tried_password = TRUE;
+	} else {
+#endif
+		GtkWindow *parent;
+		GnomePasswordDialog *dialog;
+		char *message;
+
+		parent = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (source)));
+
+		message = g_strdup_printf (_("The music share '%s' requires a password to connect"), name);
+		dialog = GNOME_PASSWORD_DIALOG (gnome_password_dialog_new (
+					_("Password Required"), message,
+					NULL, NULL, TRUE));
+		gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+		g_free (message);
+
+		gnome_password_dialog_set_domain (dialog, "DAAP");
+		gnome_password_dialog_set_show_username (dialog, FALSE);
+		gnome_password_dialog_set_show_domain (dialog, FALSE);
+		gnome_password_dialog_set_show_userpass_buttons (dialog, FALSE);
+#ifdef WITH_GNOME_KEYRING
+		gnome_password_dialog_set_show_remember (dialog, gnome_keyring_is_available ());
+#endif
+
+		if (gnome_password_dialog_run_and_block (dialog)) {
+			password = gnome_password_dialog_get_password (dialog);
+
+#ifdef WITH_GNOME_KEYRING
+			switch (gnome_password_dialog_get_remember (dialog)) {
+			case GNOME_PASSWORD_DIALOG_REMEMBER_SESSION:
+				g_free (keyring);
+				keyring = g_strdup ("session");
+				/* fall through */
+			case GNOME_PASSWORD_DIALOG_REMEMBER_FOREVER:
+			{
+				guint32 item_id;
+				gnome_keyring_set_network_password_sync (keyring,
+					NULL,
+					"DAAP", name,
+					NULL, "daap", 
+					NULL, 0, 
+					password, 
+					&item_id);
+				break;
+			}
+			default:
+				break;
+			}
+#endif
+		} else {
+			password = NULL;
+		}
+
+		/* buggered if I know why we don't do this...
+		g_object_unref (G_OBJECT (dialog)); */
+#ifdef WITH_GNOME_KEYRING
+	}
+
+	/* or these...
+	if (list)
+		gnome_keyring_network_password_list_free (list); */
+	g_free (keyring);
+#endif
 
 	return password;
 }
@@ -736,6 +830,12 @@ release_connection (RBDAAPSource *daap_source)
 	/*daap_source->priv->connection = NULL;*/
 }
 
+static void
+_add_location_to_playlist (RBRefString *uri, RBStaticPlaylistSource *source)
+{
+	rb_static_playlist_source_add_location (source, rb_refstring_get (uri), -1);
+}
+
 static void
 rb_daap_source_connection_cb (RBDAAPConnection *connection,
 			      gboolean          result,
@@ -749,6 +849,7 @@ rb_daap_source_connection_cb (RBDAAPConnection *connection,
 	RhythmDBEntryType entry_type;
 
 	rb_debug ("Connection callback result: %s", result ? "success" : "failure");
+	daap_source->priv->tried_password = FALSE;
 
 	if (result == FALSE) {
 		if (reason != NULL) {
@@ -763,8 +864,8 @@ rb_daap_source_connection_cb (RBDAAPConnection *connection,
 		return;
 	}
 
-	g_object_get (G_OBJECT (daap_source), 
-		      "shell", &shell, 
+	g_object_get (daap_source,
+		      "shell", &shell,
 		      "entry-type", &entry_type,
 		      NULL);
 	playlists = rb_daap_connection_get_playlists (RB_DAAP_CONNECTION (daap_source->priv->connection));
@@ -773,12 +874,14 @@ rb_daap_source_connection_cb (RBDAAPConnection *connection,
 		RBSource *playlist_source;
 
 		playlist_source = rb_static_playlist_source_new (shell, playlist->name, FALSE, entry_type);
-		rb_static_playlist_source_add_locations (RB_STATIC_PLAYLIST_SOURCE (playlist_source), playlist->uris);
+		g_list_foreach (playlist->uris, (GFunc)_add_location_to_playlist, playlist_source);
 
 		rb_shell_append_source (shell, playlist_source, RB_SOURCE (daap_source));
 		daap_source->priv->playlist_sources = g_slist_prepend (daap_source->priv->playlist_sources, playlist_source);
 	}
-	g_object_unref (G_OBJECT (shell));
+
+	g_object_unref (shell);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 }
 
 static void
@@ -794,12 +897,12 @@ rb_daap_source_activate (RBSource *source)
 		return;
 	}
 
-	g_object_get (G_OBJECT (daap_source), 
-		      "shell", &shell, 
-		      "entry-type", &type, 
-		      "name", &name, 
+	g_object_get (daap_source,
+		      "shell", &shell,
+		      "entry-type", &type,
+		      "name", &name,
 		      NULL);
-	g_object_get (G_OBJECT (shell), "db", &db, NULL);
+	g_object_get (shell, "db", &db, NULL);
 
 	daap_source->priv->connection = rb_daap_connection_new (name,
 								daap_source->priv->host,
@@ -809,12 +912,7 @@ rb_daap_source_activate (RBSource *source)
 								type);
 	g_object_add_weak_pointer (G_OBJECT (daap_source->priv->connection), (gpointer *)&daap_source->priv->connection);
 
-	rb_daap_connection_connect (RB_DAAP_CONNECTION (daap_source->priv->connection),
-				    (RBDAAPConnectionCallback) rb_daap_source_connection_cb,
-				    source);
-
-	g_object_unref (G_OBJECT (db));
-	g_object_unref (G_OBJECT (shell));
+	g_free (name);
 
         g_signal_connect (daap_source->priv->connection,
 			  "authenticate",
@@ -828,6 +926,13 @@ rb_daap_source_activate (RBSource *source)
 			  "disconnected",
                           G_CALLBACK (connection_disconnected_cb),
 			  source);
+
+	rb_daap_connection_connect (RB_DAAP_CONNECTION (daap_source->priv->connection),
+				    (RBDAAPConnectionCallback) rb_daap_source_connection_cb,
+				    source);
+
+	g_object_unref (G_OBJECT (db));
+	g_object_unref (G_OBJECT (shell));
 }
 
 static void
@@ -851,7 +956,7 @@ rb_daap_source_cmd_disconnect (GtkAction *action,
 {
 	RBSource *source;
 
-	g_object_get (G_OBJECT (shell), 
+	g_object_get (shell,
 		      "selected-source", &source,
 		      NULL);
 
@@ -859,8 +964,12 @@ rb_daap_source_cmd_disconnect (GtkAction *action,
 		g_critical ("got non-Daap source for Daap action");
 		return;
 	}
-		
+
 	rb_daap_source_disconnect (RB_DAAP_SOURCE (source));
+
+	if (source != NULL) {
+		g_object_unref (source);
+	}
 }
 
 static void
@@ -876,13 +985,14 @@ rb_daap_source_disconnect (RBDAAPSource *daap_source)
 
 		daap_source->priv->disconnecting = TRUE;
 
-		g_object_get (G_OBJECT (daap_source), "shell", &shell, "entry-type", &type, NULL);
-		g_object_get (G_OBJECT (shell), "db", &db, NULL);
-		g_object_unref (G_OBJECT (shell));
+		g_object_get (daap_source, "shell", &shell, "entry-type", &type, NULL);
+		g_object_get (shell, "db", &db, NULL);
+		g_object_unref (shell);
 
 		rhythmdb_entry_delete_by_type (db, type);
 		rhythmdb_commit (db);
-		g_object_unref (G_OBJECT (db));
+
+		g_object_unref (db);
 
 		for (l = daap_source->priv->playlist_sources; l!= NULL; l = l->next) {
 			RBSource *playlist_source = RB_SOURCE (l->data);
@@ -957,10 +1067,11 @@ rb_daap_source_find_for_uri (const char *uri)
 	source = (RBDAAPSource *)g_hash_table_find (source_lookup, (GHRFunc)source_host_find, ip);
 
 	g_free (ip);
-	
+
 	return source;
 }
 
+#ifdef HAVE_GSTREAMER_0_8
 char *
 rb_daap_source_get_headers (RBDAAPSource *source,
 			    const char *uri,
@@ -983,36 +1094,49 @@ rb_daap_source_get_headers (RBDAAPSource *source,
 		RBShell *shell;
 		RhythmDBEntry *entry;
 		gulong bitrate;
-		
-		g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-		g_object_get (G_OBJECT (shell), "db", &db, NULL);
+
+		g_object_get (source, "shell", &shell, NULL);
+		g_object_get (shell, "db", &db, NULL);
 
 		entry = rhythmdb_entry_lookup_by_location (db, uri);
 
-		g_object_unref (G_OBJECT (shell));
-		g_object_unref (G_OBJECT (db));
-		
-		bitrate = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_BITRATE); 
+		g_object_unref (shell);
+		g_object_unref (db);
+
+		bitrate = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_BITRATE);
 		/* bitrate is kilobits per second */
 		/* a kilobit is 128 bytes */
-		bytes = (time * bitrate) * 128; 
+		bytes = (time * bitrate) * 128;
 	}
-	
+
 	if (bytes_out != NULL) {
 		*bytes_out = bytes;
 	}
 
 	return rb_daap_connection_get_headers (source->priv->connection, uri, bytes);
 }
+#else
+char *
+rb_daap_source_get_headers (RBDAAPSource *source,
+			    const char *uri,
+			    gint64 bytes)
+{
+	/* If there is no connection then bail */
+	if (source->priv->connection == NULL) {
+		return NULL;
+	}
 
+	return rb_daap_connection_get_headers (source->priv->connection, uri, bytes);
+}
+#endif
 
-static char * 
+static char *
 rb_daap_source_get_browser_key (RBSource *source)
 {
 	return g_strdup (CONF_STATE_SHOW_BROWSER);
 }
 
-static const char * 
+static const char *
 rb_daap_source_get_paned_key (RBBrowserSource *source)
 {
 	return CONF_STATE_PANED_POSITION;
@@ -1047,7 +1171,6 @@ rb_daap_source_get_status (RBSource *source,
 
 		return;
 	}
-	
+
 	RB_SOURCE_CLASS (rb_daap_source_parent_class)->impl_get_status (source, text, progress_text, progress);
 }
-
