@@ -21,7 +21,8 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
 
 #include <gtk/gtk.h>
@@ -53,6 +54,7 @@ static void rb_library_source_dispose (GObject *object);
 static void rb_library_source_finalize (GObject *object);
 
 /* RBSource implementations */
+static gboolean impl_show_popup (RBSource *source);
 static GtkWidget *impl_get_config_widget (RBSource *source, RBShellPreferences *prefs);
 static char *impl_get_browser_key (RBSource *source);
 static const char *impl_get_paned_key (RBBrowserSource *source);
@@ -61,6 +63,8 @@ static gboolean impl_can_paste (RBSource *asource);
 #ifdef ENABLE_TRACK_TRANSFER
 static void impl_paste (RBSource *source, GList *entries);
 #endif
+static guint impl_want_uri (RBSource *source, const char *uri);
+static gboolean impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre);
 
 static void rb_library_source_ui_prefs_sync (RBLibrarySource *source);
 static void rb_library_source_preferences_sync (RBLibrarySource *source);
@@ -90,9 +94,6 @@ static gboolean rb_library_source_library_location_cb (GtkEntry *entry,
 						       RBLibrarySource *source);
 static void rb_library_source_watch_toggled_cb (GtkToggleButton *button,
 						RBLibrarySource *source);
-static void rb_library_source_songs_show_popup_cb (RBEntryView *view,
-						   gboolean over_entry,
-						   RBLibrarySource *source);						
 static void rb_library_source_sync_child_sources (RBLibrarySource *source);
 #ifdef ENABLE_TRACK_TRANSFER
 static void rb_library_source_path_changed_cb (GtkComboBox *box,
@@ -177,6 +178,7 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 	object_class->finalize = rb_library_source_finalize;
 	object_class->constructor = rb_library_source_constructor;
 
+	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_get_config_widget = impl_get_config_widget;
 	source_class->impl_get_browser_key = impl_get_browser_key;
 	source_class->impl_receive_drag = impl_receive_drag;
@@ -185,6 +187,8 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 #ifdef ENABLE_TRACK_TRANSFER
 	source_class->impl_paste = impl_paste;
 #endif
+	source_class->impl_want_uri = impl_want_uri;
+	source_class->impl_add_uri = impl_add_uri;
 
 	browser_source_class->impl_get_paned_key = impl_get_paned_key;
 	browser_source_class->impl_has_drop_support = (RBBrowserSourceFeatureFunc) rb_true_function;
@@ -217,7 +221,7 @@ rb_library_source_dispose (GObject *object)
 		g_object_unref (source->priv->db);
 		source->priv->db = NULL;
 	}
-	
+
 	G_OBJECT_CLASS (rb_library_source_parent_class)->dispose (object);
 }
 
@@ -247,7 +251,9 @@ rb_library_source_finalize (GObject *object)
 static gboolean
 add_child_sources_idle (RBLibrarySource *source)
 {
+	GDK_THREADS_ENTER ();
 	rb_library_source_sync_child_sources (source);
+	GDK_THREADS_LEAVE ();
 
 	return FALSE;
 }
@@ -264,8 +270,8 @@ rb_library_source_constructor (GType type,
 	source = RB_LIBRARY_SOURCE (G_OBJECT_CLASS (rb_library_source_parent_class)
 			->constructor (type, n_construct_properties, construct_properties));
 
-	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-	g_object_get (G_OBJECT (shell), "db", &source->priv->db, NULL);
+	g_object_get (source, "shell", &shell, NULL);
+	g_object_get (shell, "db", &source->priv->db, NULL);
 
 	rb_library_source_ui_prefs_sync (source);
 
@@ -278,17 +284,14 @@ rb_library_source_constructor (GType type,
 				    (GConfClientNotifyFunc) rb_library_source_ui_pref_changed, source);
 
 	songs = rb_source_get_entry_view (RB_SOURCE (source));
-	
+
 	rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_RATING, FALSE);
 	rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_LAST_PLAYED, FALSE);
 	rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_FIRST_SEEN, FALSE);
 
-	g_signal_connect_object (G_OBJECT (rb_source_get_entry_view (RB_SOURCE (source))), "show_popup",
-				 G_CALLBACK (rb_library_source_songs_show_popup_cb), source, 0);
-
 	g_idle_add ((GSourceFunc)add_child_sources_idle, source);
 
-	g_object_unref (G_OBJECT (shell));
+	g_object_unref (shell);
 
 	return G_OBJECT (source);
 }
@@ -299,22 +302,27 @@ rb_library_source_new (RBShell *shell)
 	RBSource *source;
 	GdkPixbuf *icon;
 	gint size;
+	RhythmDBEntryType entry_type;
+
+	entry_type = RHYTHMDB_ENTRY_TYPE_SONG;
 
 	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
 	icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
 					 "stock_music-library",
 					 size,
 					 0, NULL);
-
 	source = RB_SOURCE (g_object_new (RB_TYPE_LIBRARY_SOURCE,
 					  "name", _("Library"),
-					  "entry-type", RHYTHMDB_ENTRY_TYPE_SONG,
+					  "entry-type", entry_type,
 					  "sorting-key", CONF_STATE_LIBRARY_SORTING,
 					  "shell", shell,
 					  "icon", icon,
 					  NULL));
-	rb_shell_register_entry_type_for_source (shell, source,
-						 RHYTHMDB_ENTRY_TYPE_SONG);
+	if (icon != NULL) {
+		g_object_unref (icon);
+	}
+
+	rb_shell_register_entry_type_for_source (shell, source, entry_type);
 
 	return source;
 }
@@ -327,6 +335,7 @@ rb_library_source_edit_profile_clicked_cb (GtkButton *button, RBLibrarySource *s
 
 	dialog = gm_audio_profiles_edit_new (eel_gconf_client_get_global (),
 					     GTK_WINDOW (source->priv->shell_prefs));
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_widget_show_all (dialog);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 }
@@ -336,7 +345,7 @@ static void
 rb_library_source_location_button_clicked_cb (GtkButton *button, RBLibrarySource *source)
 {
 	GtkWidget *dialog;
-	
+
 	dialog = rb_file_chooser_new (_("Choose Library Location"), GTK_WINDOW (source->priv->shell_prefs),
 				      GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, FALSE);
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
@@ -347,9 +356,9 @@ rb_library_source_location_button_clicked_cb (GtkButton *button, RBLibrarySource
 		if (uri == NULL) {
 			uri = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (dialog));
 		}
-		
+
 		path = gnome_vfs_format_uri_for_display (uri);
-		
+
 		gtk_entry_set_text (GTK_ENTRY (source->priv->library_location_entry), path);
 		rb_library_source_library_location_cb (GTK_ENTRY (source->priv->library_location_entry),
 						       NULL, source);
@@ -367,15 +376,16 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 	GtkWidget *tmp;
 	GladeXML *xml;
 #ifdef ENABLE_TRACK_TRANSFER
+	GtkWidget *label;
 	int i;
 #endif
 
 	if (source->priv->config_widget)
 		return source->priv->config_widget;
-	
+
 	g_object_ref (G_OBJECT (prefs));
 	source->priv->shell_prefs = prefs;
-	
+
 	xml = rb_glade_xml_new ("library-prefs.glade", "library_vbox", source);
 	source->priv->config_widget =
 		glade_xml_get_widget (xml, "library_vbox");
@@ -403,8 +413,10 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 	rb_glade_boldify_label (xml, "library_structure_label");
 
 	tmp = glade_xml_get_widget (xml, "layout_path_menu_box");
+	label = glade_xml_get_widget (xml, "layout_path_menu_label");
 	source->priv->layout_path_menu = gtk_combo_box_new_text ();
 	gtk_box_pack_start_defaults (GTK_BOX (tmp), source->priv->layout_path_menu);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), source->priv->layout_path_menu);
 	g_signal_connect (G_OBJECT (source->priv->layout_path_menu),
 			  "changed",
 			  G_CALLBACK (rb_library_source_path_changed_cb),
@@ -415,8 +427,10 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 	}
 
 	tmp = glade_xml_get_widget (xml, "layout_filename_menu_box");
+	label = glade_xml_get_widget (xml, "layout_filename_menu_label");
 	source->priv->layout_filename_menu = gtk_combo_box_new_text ();
 	gtk_box_pack_start_defaults (GTK_BOX (tmp), source->priv->layout_filename_menu);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), source->priv->layout_filename_menu);
 	g_signal_connect (G_OBJECT (source->priv->layout_filename_menu),
 			  "changed",
 			  G_CALLBACK (rb_library_source_filename_changed_cb),
@@ -433,8 +447,10 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 			  asource);
 
 	tmp = glade_xml_get_widget (xml, "preferred_format_menu_box");
+	label = glade_xml_get_widget (xml, "preferred_format_menu_label");
 	source->priv->preferred_format_menu = gm_audio_profile_choose_new ();
 	gtk_box_pack_start_defaults (GTK_BOX (tmp), source->priv->preferred_format_menu);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), source->priv->preferred_format_menu);
 	g_signal_connect (G_OBJECT (source->priv->preferred_format_menu),
 			  "changed",
 			  G_CALLBACK (rb_library_source_format_changed_cb),
@@ -448,7 +464,7 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 #endif
 
 	g_object_unref (G_OBJECT (xml));
-	
+
 	rb_library_source_preferences_sync (source);
 
 	return source->priv->config_widget;
@@ -576,7 +592,7 @@ rb_library_source_library_location_cb (GtkEntry *entry,
 	g_free (uri);
 	if (list)
 		g_slist_free (list);
-	
+
 	/* don't do the first-run druid if the user sets the library location */
 	if (list)
 		eel_gconf_set_boolean (CONF_FIRST_TIME, TRUE);
@@ -605,7 +621,6 @@ impl_get_paned_key (RBBrowserSource *status)
 	return CONF_STATE_PANED_POSITION;
 }
 
-
 static void
 rb_library_source_add_location_entry_changed_cb (GtkEntry *entry,
 						 GtkWidget *target)
@@ -715,13 +730,11 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 	return TRUE;
 }
 
-static void
-rb_library_source_songs_show_popup_cb (RBEntryView *view,
-				       gboolean over_entry,
-				       RBLibrarySource *source)
+static gboolean
+impl_show_popup (RBSource *source)
 {
-	if (!over_entry)
-		_rb_source_show_popup (RB_SOURCE (source), "/LibrarySourcePopup");
+	_rb_source_show_popup (source, "/LibrarySourcePopup");
+	return TRUE;
 }
 
 #ifdef ENABLE_TRACK_TRANSFER
@@ -772,7 +785,7 @@ sanitize_path (const char *str)
 	/* Skip leading periods, otherwise files disappear... */
 	while (*str == '.')
 		str++;
-  
+
 	s = g_strdup(str);
 	/* Replace path seperators with a hyphen */
 	g_strdelimit (s, "/", '-');
@@ -809,7 +822,8 @@ sanitize_path (const char *str)
  * %tS -- track artist sortname (lowercase)
  */
 static char *
-filepath_parse_pattern (const char *pattern, RhythmDBEntry *entry)
+filepath_parse_pattern (const char *pattern,
+			RhythmDBEntry *entry)
 {
 	/* p is the pattern iterator, i is a general purpose iterator */
 	const char *p;
@@ -868,7 +882,7 @@ filepath_parse_pattern (const char *pattern, RhythmDBEntry *entry)
 			default:
 				string = g_strdup_printf ("%%a%c", *p);
 			}
-			
+
 			break;
 
 		case 't':
@@ -909,7 +923,7 @@ filepath_parse_pattern (const char *pattern, RhythmDBEntry *entry)
  			}
 
 			break;
-			
+
 		default:
 			string = g_strdup_printf ("%%%c", *p);
 		}
@@ -936,6 +950,7 @@ layout_example_label_update (RBLibrarySource *source)
 	char *example;
 	char *format;
 	GMAudioProfile *profile;
+	RhythmDBEntryType entry_type;
 	RhythmDBEntry *sample_entry;
 
   	profile = gm_audio_profile_choose_get_active (source->priv->preferred_format_menu);
@@ -950,10 +965,13 @@ layout_example_label_update (RBLibrarySource *source)
 		path_pattern = g_strdup (library_layout_paths[0].path);
 	}
 
-	sample_entry = rhythmdb_entry_example_new (source->priv->db, RHYTHMDB_ENTRY_TYPE_SONG, NULL);
+	g_object_get (source, "entry-type", &entry_type, NULL);
+	sample_entry = rhythmdb_entry_example_new (source->priv->db, entry_type, NULL);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+
 	file_value = filepath_parse_pattern (file_pattern, sample_entry);
 	path_value = filepath_parse_pattern (path_pattern, sample_entry);
-	rhythmdb_entry_unref (source->priv->db, sample_entry);
+	rhythmdb_entry_unref (sample_entry);
 
 	example = g_build_filename (G_DIR_SEPARATOR_S, path_value, file_value, NULL);
 	g_free (file_value);
@@ -967,7 +985,7 @@ layout_example_label_update (RBLibrarySource *source)
 			      profile ? gm_audio_profile_get_extension (profile) : "ogg",
 			      "</i></small>", NULL);
 	g_free (example);
-  
+
 	gtk_label_set_markup (GTK_LABEL (source->priv->layout_example_label), format);
 	g_free (format);
 }
@@ -1011,11 +1029,11 @@ rb_library_source_layout_filename_changed (GConfClient *client,
 {
 	char *value;
 	int i = 0;
-  
+
 	g_return_if_fail (strcmp (entry->key, CONF_LIBRARY_LAYOUT_FILENAME) == 0);
 
 	rb_debug ("layout filename changed");
-  
+
 	if (entry->value == NULL) {
 		value = g_strdup (library_layout_filenames[0].path);
 	} else if (entry->value->type == GCONF_VALUE_STRING) {
@@ -1088,7 +1106,7 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 
 	if (extension == NULL) {
 		char *tmp;
-		
+
 		/* use the old extension. strip anything after a '?' for http/daap/etc */
 		extension = g_strdup (g_utf8_strrchr (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION), -1, '.') + 1);
 
@@ -1096,7 +1114,7 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 		if (tmp)
 			*tmp = '\0';
 	}
-	
+
 	realfile = filepath_parse_pattern (layout_filename, entry);
 	if (extension) {
 		filename = g_strdup_printf ("%s.%s", realfile, extension);
@@ -1130,7 +1148,6 @@ impl_can_paste (RBSource *asource)
 		     (eel_gconf_get_string (CONF_LIBRARY_LAYOUT_FILENAME) != NULL) &&
 		     (eel_gconf_get_string (CONF_LIBRARY_PREFERRED_FORMAT) != NULL));
 
-
 	g_slist_free (list);
 	return can_paste;
 #else
@@ -1153,6 +1170,7 @@ impl_paste (RBSource *asource, GList *entries)
 	GList *l;
 	GSList *sl;
 	RBShell *shell;
+	RhythmDBEntryType source_entry_type;
 
 	sl = eel_gconf_get_string_list (CONF_LIBRARY_LOCATION);
 	if ((sl == NULL) ||
@@ -1164,9 +1182,12 @@ impl_paste (RBSource *asource, GList *entries)
 		return;
 	}
 
-	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-	g_object_get (G_OBJECT (shell), "removable-media-manager", &rm_mgr, NULL);
-	g_object_unref (G_OBJECT (shell));
+	g_object_get (source,
+		      "shell", &shell,
+		      "entry-type", &source_entry_type,
+		      NULL);
+	g_object_get (shell, "removable-media-manager", &rm_mgr, NULL);
+	g_object_unref (shell);
 
 	for (l = entries; l != NULL; l = g_list_next (l)) {
 		RhythmDBEntry *entry = (RhythmDBEntry *)l->data;
@@ -1176,10 +1197,10 @@ impl_paste (RBSource *asource, GList *entries)
 		rb_debug ("pasting entry %s", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
 
 		entry_type = rhythmdb_entry_get_entry_type (entry);
-		if (entry_type == RHYTHMDB_ENTRY_TYPE_SONG)
+		if (entry_type == source_entry_type)
 			/* copying to ourselves would be silly */
 			continue;
-		
+
 		/* see if the responsible source lets us copy */
 		if (!rb_source_can_copy (rb_shell_get_source_by_entry_type (shell, entry_type)))
 			continue;
@@ -1194,11 +1215,34 @@ impl_paste (RBSource *asource, GList *entries)
 							  dest, NULL,
 							  (RBTranferCompleteCallback)completed_cb, source);
 	}
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, source_entry_type);
 
-	g_object_unref (G_OBJECT (rm_mgr));
+	g_object_unref (rm_mgr);
 }
 #endif
 
+static guint
+impl_want_uri (RBSource *source, const char *uri)
+{
+	/* assume anything local, on smb, or on sftp is a song */
+	if (rb_uri_is_local (uri) ||
+	    g_str_has_prefix (uri, "smb://") ||
+	    g_str_has_prefix (uri, "sftp://"))
+		return 50;
+
+	return 0;
+}
+
+static gboolean
+impl_add_uri (RBSource *asource, const char *uri, const char *title, const char *genre)
+{
+	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
+	/* FIXME should be synchronous */
+	rb_debug ("adding uri %s to library", uri);
+	rhythmdb_add_uri (source->priv->db, uri);
+	return TRUE;
+}
+
 static void
 rb_library_source_add_child_source (const char *path, RBLibrarySource *library_source)
 {
@@ -1208,15 +1252,19 @@ rb_library_source_add_child_source (const char *path, RBLibrarySource *library_s
 	GnomeVFSURI *uri;
 	char *name;
 	GdkPixbuf *icon;
+	RhythmDBEntryType entry_type;
 
-	g_object_get (G_OBJECT (library_source), "shell", &shell, NULL);
+	g_object_get (library_source,
+		      "shell", &shell,
+		      "entry-type", &entry_type,
+		      NULL);
 	uri = gnome_vfs_uri_new (path);
 	name = gnome_vfs_uri_extract_short_name (uri);
 	gnome_vfs_uri_unref (uri);
-	
+
 	source = rb_auto_playlist_source_new (shell, name, FALSE);
 	query = rhythmdb_query_parse (library_source->priv->db,
-				      RHYTHMDB_QUERY_PROP_EQUALS, RHYTHMDB_PROP_TYPE, RHYTHMDB_ENTRY_TYPE_SONG,
+				      RHYTHMDB_QUERY_PROP_EQUALS, RHYTHMDB_PROP_TYPE, entry_type,
 				      RHYTHMDB_QUERY_PROP_PREFIX, RHYTHMDB_PROP_LOCATION, path,
 				      RHYTHMDB_QUERY_END);
 	rb_auto_playlist_source_set_query (RB_AUTO_PLAYLIST_SOURCE (source), query,
@@ -1224,12 +1272,16 @@ rb_library_source_add_child_source (const char *path, RBLibrarySource *library_s
 					   NULL, 0);
 	rhythmdb_query_free (query);
 
-	g_object_get (G_OBJECT (library_source), "icon", &icon, NULL);
-	g_object_set (G_OBJECT (source), "icon", g_object_ref (icon), NULL);
+	g_object_get (library_source, "icon", &icon, NULL);
+	g_object_set (source, "icon", icon, NULL);
+	if (icon != NULL) {
+		g_object_unref (icon);
+	}
 
 	rb_shell_append_source (shell, source, RB_SOURCE (library_source));
 	library_source->priv->child_sources = g_list_prepend (library_source->priv->child_sources, source);
 
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 	g_object_unref (shell);
 	g_free (name);
 }
@@ -1251,4 +1303,3 @@ rb_library_source_sync_child_sources (RBLibrarySource *source)
 	g_slist_foreach (list, (GFunc) g_free, NULL);
 	g_slist_free (list);
 }
-
