@@ -22,7 +22,8 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
@@ -38,8 +39,8 @@
 #include <nautilus-burn-drive.h>
 #include <nautilus-burn-drive-selection.h>
 
-#ifndef NAUTILUS_BURN_CHECK_VERSION 	 
-#define NAUTILUS_BURN_CHECK_VERSION(a,b,c) FALSE 	 
+#ifndef NAUTILUS_BURN_CHECK_VERSION
+#define NAUTILUS_BURN_CHECK_VERSION(a,b,c) FALSE
 #endif
 
 #if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
@@ -166,7 +167,6 @@ rb_playlist_source_recorder_style_set (GtkWidget *widget,
         gtk_box_set_spacing (GTK_BOX (dialog->action_area), 6);
 }
 
-
 static void
 rb_playlist_source_recorder_class_init (RBPlaylistSourceRecorderClass *klass)
 {
@@ -187,7 +187,7 @@ rb_playlist_source_recorder_class_init (RBPlaylistSourceRecorderClass *klass)
                               G_TYPE_NONE,
                               1,
                               G_TYPE_STRING);
-        
+
         rb_playlist_source_recorder_signals [FILE_ADDED] =
                 g_signal_new ("file_added",
                               G_OBJECT_CLASS_TYPE (object_class),
@@ -224,7 +224,6 @@ rb_playlist_source_recorder_device_menu_create (void)
         return widget;
 }
 
-
 static const NautilusBurnDrive *
 lookup_current_recorder (RBPlaylistSourceRecorder *source)
 {
@@ -303,7 +302,6 @@ update_speed_combobox (RBPlaylistSourceRecorder *source)
                             1, 0,
                             -1);
 
-
         default_speed = eel_gconf_get_integer (CONF_STATE_BURN_SPEED);
         default_speed_index = -1;
 
@@ -316,7 +314,7 @@ update_speed_combobox (RBPlaylistSourceRecorder *source)
                 for (i = 0; write_speeds [i] > 0; i++) {
 
 #ifdef NAUTILUS_BURN_DRIVE_CD_SPEED
-                        name = g_strdup_printf ("%d \303\227", NAUTILUS_BURN_DRIVE_CD_SPEED (write_speeds [i]));
+                        name = g_strdup_printf ("%d \303\227", (int)NAUTILUS_BURN_DRIVE_CD_SPEED (write_speeds [i]));
 #else
                         name = g_strdup_printf ("%d \303\227", write_speeds [i]);
 #endif
@@ -400,9 +398,10 @@ set_message_text (RBPlaylistSourceRecorder *source,
 static gboolean
 response_idle_cb (RBPlaylistSourceRecorder *source)
 {
-
+	GDK_THREADS_ENTER ();
         gtk_dialog_response (GTK_DIALOG (source),
                              GTK_RESPONSE_CANCEL);
+	GDK_THREADS_LEAVE ();
 
         return FALSE;
 }
@@ -538,7 +537,7 @@ _nautilus_burn_drive_eject (NautilusBurnDrive *drive)
 
         if (drive->device == NULL)
                 return FALSE;
-        
+
         cmd = g_strdup_printf ("eject %s", drive->device);
         res = g_spawn_command_line_sync (cmd, NULL, NULL, NULL, NULL);
         g_free (cmd);
@@ -656,9 +655,9 @@ burn_cd (RBPlaylistSourceRecorder *source,
                 }
                 set_message_text (source, _("Finished creating audio CD.\nCreate another copy?"));
         } else if (res == RB_RECORDER_RESULT_ERROR) {
-                set_message_text (source, _("Writing failed.  Try again?"));  
+                set_message_text (source, _("Writing failed.  Try again?"));
         } else {
-                set_message_text (source, _("Writing cancelled.  Try again?"));  
+                set_message_text (source, _("Writing cancelled.  Try again?"));
         }
 
         progress_set_fraction (source->priv->progress, 0);
@@ -721,6 +720,8 @@ burn_cd_idle (RBPlaylistSourceRecorder *source)
         GError *error = NULL;
         int     res;
 
+	GDK_THREADS_ENTER ();
+
         res = burn_cd (source, &error);
         if (error) {
                 error_dialog (source,
@@ -729,6 +730,7 @@ burn_cd_idle (RBPlaylistSourceRecorder *source)
                 g_error_free (error);
         }
 
+	GDK_THREADS_LEAVE ();
         return FALSE;
 }
 
@@ -973,6 +975,7 @@ insert_media_request_cb (RBRecorder *recorder,
                 title = N_("Reload a blank CD");
         }
 
+        GDK_THREADS_ENTER ();
         dialog = gtk_message_dialog_new (GTK_WINDOW (source),
                                          GTK_DIALOG_DESTROY_WITH_PARENT,
                                          GTK_MESSAGE_ERROR,
@@ -985,11 +988,10 @@ insert_media_request_cb (RBRecorder *recorder,
 
         gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
 
-        GDK_THREADS_ENTER ();
         res = gtk_dialog_run (GTK_DIALOG (dialog));
-        GDK_THREADS_LEAVE ();
 
         gtk_widget_destroy (dialog);
+        GDK_THREADS_LEAVE ();
 
         if (res == GTK_RESPONSE_CANCEL)
                 return FALSE;
@@ -1067,7 +1069,6 @@ ask_rewrite_disc (RBPlaylistSourceRecorder *source,
         msg = g_strdup_printf (_("This %s appears to have information already recorded on it."),
                                nautilus_burn_drive_media_type_get_string (type));
 
-
         dialog = gtk_message_dialog_new (GTK_WINDOW (source),
                                          GTK_DIALOG_DESTROY_WITH_PARENT,
                                          GTK_MESSAGE_WARNING,
@@ -1094,9 +1095,7 @@ ask_rewrite_disc (RBPlaylistSourceRecorder *source,
         gtk_dialog_set_default_response (GTK_DIALOG (dialog),
                                          RB_RECORDER_RESPONSE_CANCEL);
 
-        GDK_THREADS_ENTER ();
         res = gtk_dialog_run (GTK_DIALOG (dialog));
-        GDK_THREADS_LEAVE ();
 
         gtk_widget_destroy (dialog);
 
@@ -1117,7 +1116,9 @@ warn_data_loss_cb (RBRecorder               *recorder,
         int   res;
 
         device = rb_recorder_get_device (recorder, NULL);
+	GDK_THREADS_ENTER();
         res = ask_rewrite_disc (source, device);
+	GDK_THREADS_LEAVE();
 
         g_free (device);
 
@@ -1202,7 +1203,6 @@ rb_playlist_source_recorder_init (RBPlaylistSourceRecorder *source)
 
         gtk_dialog_set_default_response (GTK_DIALOG (source), GTK_RESPONSE_ACCEPT);
 
-
         xml = rb_glade_xml_new ("recorder.glade",
                                 "recorder_vbox",
                                 source);
@@ -1373,11 +1373,11 @@ rb_playlist_source_recorder_new (GtkWidget  *parent,
 {
         GtkWidget *result;
         RBPlaylistSourceRecorder *source;
-        
+
         result = g_object_new (RB_TYPE_PLAYLIST_SOURCE_RECORDER,
                                "title", _("Create Audio CD"),
                                NULL);
-        
+
         source = RB_PLAYLIST_SOURCE_RECORDER (result);
         if (parent) {
                 source->priv->parent = gtk_widget_get_toplevel (parent);
@@ -1392,7 +1392,7 @@ rb_playlist_source_recorder_new (GtkWidget  *parent,
         if (name) {
                 source->priv->name = g_strdup (name);
 
-                set_message_text (source, _("Create audio CD from '%s' playlist?"), name);
+                set_message_text (source, _("Create audio CD from '%s'?"), name);
         }
 
         return result;
