@@ -39,6 +39,7 @@
 
 static void rb_source_class_init (RBSourceClass *klass);
 static void rb_source_init (RBSource *source);
+static void rb_source_dispose (GObject *object);
 static void rb_source_finalize (GObject *object);
 static void rb_source_set_property (GObject *object,
 					guint prop_id,
@@ -60,7 +61,7 @@ static gboolean default_try_playlist (RBSource *source);
 static RBSourceEOFType default_handle_eos (RBSource *source);
 static gboolean default_show_popup  (RBSource *source);
 static void default_delete_thyself (RBSource *source);
-static RBEntryView* default_get_entry_view (RBSource *source);
+static RBEntryView *default_get_entry_view (RBSource *source);
 static void default_activate (RBSource *source);
 static void default_deactivate (RBSource *source);
 static void default_add_to_queue (RBSource *source, RBSource *queue);
@@ -69,9 +70,9 @@ static void default_move_to_trash (RBSource *source);
 static GList * default_get_ui_actions (RBSource *source);
 static GList * default_get_search_actions (RBSource *source);
 
-static void rb_source_row_deleted_cb (GtkTreeModel *model,
-				      GtkTreePath *path,
-				      RBSource *source);
+static void rb_source_post_entry_deleted_cb (GtkTreeModel *model,
+					     RhythmDBEntry *entry,
+					     RBSource *source);
 static void rb_source_row_inserted_cb (GtkTreeModel *model,
 				       GtkTreePath *path,
 				       GtkTreeIter *iter,
@@ -82,13 +83,16 @@ G_DEFINE_ABSTRACT_TYPE (RBSource, rb_source, GTK_TYPE_HBOX)
 struct _RBSourcePrivate
 {
 	char *name;
-	
+
 	RBShell *shell;
 	gboolean visible;
 	RhythmDBQueryModel *query_model;
-	guint idle_status_changed_id;
 	GdkPixbuf *pixbuf;
 	RBSourceListGroup sourcelist_group;
+	guint hidden_when_empty : 1;
+	guint update_visibility_id;
+	guint update_status_id;
+	RhythmDBEntryType entry_type;
 };
 
 enum
@@ -100,7 +104,9 @@ enum
 	PROP_UI_MANAGER,
 	PROP_VISIBLE,
 	PROP_QUERY_MODEL,
-	PROP_SOURCELIST_GROUP
+	PROP_HIDDEN_WHEN_EMPTY,
+	PROP_SOURCELIST_GROUP,
+	PROP_ENTRY_TYPE
 };
 
 enum
@@ -118,6 +124,7 @@ rb_source_class_init (RBSourceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	object_class->dispose = rb_source_dispose;
 	object_class->finalize = rb_source_finalize;
 
 	object_class->set_property = rb_source_set_property;
@@ -136,7 +143,6 @@ rb_source_class_init (RBSourceClass *klass)
 	klass->impl_can_add_to_queue = (RBSourceFeatureFunc) rb_false_function;
 	klass->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_false_function;
 	klass->impl_get_entry_view = default_get_entry_view;
-	klass->impl_have_url = (RBSourceFeatureFunc) rb_false_function;
 	klass->impl_copy = default_copy;
 	klass->impl_reset_filters = default_reset_filters;
 	klass->impl_song_properties = default_song_properties;
@@ -185,8 +191,7 @@ rb_source_class_init (RBSourceClass *klass)
 							      GTK_TYPE_UI_MANAGER,
 							      G_PARAM_READABLE));
 
-
-	g_object_class_install_property (object_class, 
+	g_object_class_install_property (object_class,
 					 PROP_VISIBLE,
 					 /* FIXME: This property could probably
 					  * be named better, there's already
@@ -194,12 +199,20 @@ rb_source_class_init (RBSourceClass *klass)
 					  * since RBSource derives from
 					  * GtkWidget, this can be confusing
 					  */
-					 g_param_spec_boolean ("visibility", 
+					 g_param_spec_boolean ("visibility",
 							       "visibility",
 							       "Whether the source should be displayed in the source list",
 							       TRUE,
 							       G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_HIDDEN_WHEN_EMPTY,
+					 g_param_spec_boolean ("hidden-when-empty",
+							       "hidden-when-empty",
+							       "Whether the source should be displayed in the source list when it is empty",
+							       FALSE,
+							       G_PARAM_READWRITE));
+
 	g_object_class_install_property (object_class,
 					 PROP_QUERY_MODEL,
 					 g_param_spec_object ("query-model",
@@ -216,6 +229,14 @@ rb_source_class_init (RBSourceClass *klass)
 							    RB_SOURCELIST_GROUP_FIXED,
 							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	g_object_class_install_property (object_class,
+					 PROP_ENTRY_TYPE,
+					 g_param_spec_boxed ("entry-type",
+							     "Entry type",
+							     "Type of the entries which should be displayed by this source",
+							     RHYTHMDB_TYPE_ENTRY_TYPE,
+							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 	rb_source_signals[DELETED] =
 		g_signal_new ("deleted",
 			      RB_TYPE_SOURCE,
@@ -250,7 +271,7 @@ rb_source_class_init (RBSourceClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
-	
+
 	g_type_class_add_private (object_class, sizeof (RBSourcePrivate));
 }
 
@@ -261,11 +282,11 @@ rb_source_init (RBSource *source)
 }
 
 static void
-rb_source_finalize (GObject *object)
+rb_source_dispose (GObject *object)
 {
 	RBSource *source;
 	RBSourcePrivate *priv;
-	
+
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (RB_IS_SOURCE (object));
 
@@ -273,12 +294,39 @@ rb_source_finalize (GObject *object)
 	priv = RB_SOURCE_GET_PRIVATE (source);
 	g_return_if_fail (priv != NULL);
 
-	rb_debug ("Finalizing view %p", source);
+	rb_debug ("Disposing source %p: '%s'", source, priv->name);
 
-	if (priv->idle_status_changed_id)
-		g_source_remove (priv->idle_status_changed_id);
-	
-	g_object_unref (G_OBJECT (priv->query_model));
+	if (priv->update_visibility_id != 0) {
+		g_source_remove (priv->update_visibility_id);
+		priv->update_visibility_id = 0;
+	}
+	if (priv->update_status_id != 0) {
+		g_source_remove (priv->update_status_id);
+		priv->update_status_id = 0;
+	}
+
+	G_OBJECT_CLASS (rb_source_parent_class)->dispose (object);
+}
+
+static void
+rb_source_finalize (GObject *object)
+{
+	RBSource *source;
+	RBSourcePrivate *priv;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (RB_IS_SOURCE (object));
+
+	source = RB_SOURCE (object);
+	priv = RB_SOURCE_GET_PRIVATE (source);
+	g_return_if_fail (priv != NULL);
+
+	rb_debug ("Finalizing source %p: '%s'", source, priv->name);
+
+	if (priv->query_model != NULL) {
+		rb_debug ("Unreffing model %p count: %d", priv->query_model, G_OBJECT (priv->query_model)->ref_count);
+		g_object_unref (priv->query_model);
+	}
 
 	if (priv->pixbuf != NULL) {
 		g_object_unref (priv->pixbuf);
@@ -308,6 +356,87 @@ rb_source_set_pixbuf (RBSource  *source,
 	}
 }
 
+static gboolean
+update_visibility_idle (RBSource *source)
+{
+	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (source);
+	gboolean visibility;
+
+	GDK_THREADS_ENTER ();
+
+	gint count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (priv->query_model), NULL);
+
+	visibility = (count > 0);
+
+	if (visibility != priv->visible) {
+		g_object_set (G_OBJECT (source), "visibility", visibility, NULL);
+	}
+
+	priv->update_visibility_id = 0;
+	GDK_THREADS_LEAVE ();
+	return FALSE;
+}
+
+static void
+queue_update_visibility (RBSource *source)
+{
+	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (source);
+
+	if (priv->update_visibility_id != 0) {
+		g_source_remove (priv->update_visibility_id);
+	}
+	priv->update_visibility_id = g_idle_add ((GSourceFunc) update_visibility_idle, source);
+}
+
+void
+rb_source_set_hidden_when_empty (RBSource *source,
+				 gboolean  hidden)
+{
+	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (source);
+
+	g_return_if_fail (RB_IS_SOURCE (source));
+
+	if (priv->hidden_when_empty != hidden) {
+		priv->hidden_when_empty = hidden;
+		queue_update_visibility (source);
+	}
+}
+
+static void
+rb_source_set_query_model_internal (RBSource *source,
+				    RhythmDBQueryModel *model)
+{
+	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (source);
+
+	if (priv->query_model == model) {
+		return;
+	}
+
+	if (priv->query_model != NULL) {
+		g_signal_handlers_disconnect_by_func (priv->query_model,
+						      G_CALLBACK (rb_source_post_entry_deleted_cb),
+						      source);
+		g_signal_handlers_disconnect_by_func (priv->query_model,
+						      G_CALLBACK (rb_source_row_inserted_cb),
+						      source);
+		g_object_unref (priv->query_model);
+	}
+
+	priv->query_model = model;
+	if (priv->query_model != NULL) {
+		g_object_ref (priv->query_model);
+		g_signal_connect_object (model, "post-entry-delete",
+					 G_CALLBACK (rb_source_post_entry_deleted_cb),
+					 source, 0);
+		g_signal_connect_object (model, "row_inserted",
+					 G_CALLBACK (rb_source_row_inserted_cb),
+					 source, 0);
+	}
+
+	/* g_object_notify (G_OBJECT (source), "query-model"); */
+	rb_source_notify_status_changed (source);
+}
+
 static void
 rb_source_set_property (GObject *object,
 			guint prop_id,
@@ -315,7 +444,6 @@ rb_source_set_property (GObject *object,
 			GParamSpec *pspec)
 {
 	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (object);
-	RhythmDBQueryModel *model;
 	RBSource *source = RB_SOURCE (object);
 
 	switch (prop_id) {
@@ -331,49 +459,29 @@ rb_source_set_property (GObject *object,
 		break;
 	case PROP_VISIBLE:
 		priv->visible = g_value_get_boolean (value);
-		rb_debug ("Setting %s visibility to %u", 
-			  priv->name, 
+		rb_debug ("Setting %s visibility to %u",
+			  priv->name,
 			  priv->visible);
 		break;
+	case PROP_HIDDEN_WHEN_EMPTY:
+		rb_source_set_hidden_when_empty (source, g_value_get_boolean (value));
+		break;
 	case PROP_QUERY_MODEL:
-		model = g_value_get_object (value); 
-		if (priv->query_model == model)
-			return;
-
-		if (priv->query_model) {
-			g_signal_handlers_disconnect_by_func (G_OBJECT (model),
-							      G_CALLBACK (rb_source_row_deleted_cb),
-							      source);
-			g_signal_handlers_disconnect_by_func (G_OBJECT (model),
-							      G_CALLBACK (rb_source_row_inserted_cb),
-							      source);
-			g_object_unref (G_OBJECT (priv->query_model));
-		}
-		
-		priv->query_model = model;
-		if (priv->query_model) {
-			g_object_ref (G_OBJECT (model));
-			g_signal_connect_object (G_OBJECT (model), "row_deleted",
-						 G_CALLBACK (rb_source_row_deleted_cb),
-						 source, 0);
-			g_signal_connect_object (G_OBJECT (model), "row_inserted",
-						 G_CALLBACK (rb_source_row_inserted_cb),
-						 source, 0);
-		}
-
-		/* g_object_notify (G_OBJECT (source), "query-model"); */
-		rb_source_notify_status_changed (source);
+		rb_source_set_query_model_internal (source, g_value_get_object (value));
 		break;
 	case PROP_SOURCELIST_GROUP:
 		priv->sourcelist_group = g_value_get_enum (value);
 		break;
+	case PROP_ENTRY_TYPE:
+		priv->entry_type = g_value_get_boxed (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
-static void 
+static void
 rb_source_get_property (GObject *object,
 			guint prop_id,
 			GValue *value,
@@ -397,11 +505,11 @@ rb_source_get_property (GObject *object,
 	case PROP_UI_MANAGER:
 	{
 		GtkUIManager *manager;
-		g_object_get (G_OBJECT (priv->shell), 
+		g_object_get (priv->shell,
 			      "ui-manager", &manager,
 			      NULL);
 		g_value_set_object (value, manager);
-		g_object_unref (G_OBJECT(manager));
+		g_object_unref (manager);
 		break;
 	}
 	case PROP_QUERY_MODEL:
@@ -410,6 +518,9 @@ rb_source_get_property (GObject *object,
 	case PROP_SOURCELIST_GROUP:
 		g_value_set_enum (value, priv->sourcelist_group);
 		break;
+	case PROP_ENTRY_TYPE:
+		g_value_set_boxed (value, priv->entry_type);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -417,16 +528,21 @@ rb_source_get_property (GObject *object,
 }
 
 static void
-default_get_status (RBSource *source, char **text, char **progress_text, float *progress)
+default_get_status (RBSource *source,
+		    char **text,
+		    char **progress_text,
+		    float *progress)
 {
 	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (source);
+
 	/* hack to get these strings marked for translation */
 	if (0) {
 		ngettext ("%d song", "%d songs", 0);
 	}
+
 	if (priv->query_model) {
-		*text = rhythmdb_query_model_compute_status_normal (priv->query_model, 
-								    "%d song", 
+		*text = rhythmdb_query_model_compute_status_normal (priv->query_model,
+								    "%d song",
 								    "%d songs");
 		if (rhythmdb_query_model_has_pending_changes (priv->query_model))
 			*progress = -1.0f;
@@ -448,7 +564,10 @@ default_get_status (RBSource *source, char **text, char **progress_text, float *
  * the progress bar will be hidden.
  **/
 void
-rb_source_get_status (RBSource *source, char **text, char **progress_text, float *progress)
+rb_source_get_status (RBSource *source,
+		      char **text,
+		      char **progress_text,
+		      float *progress)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
@@ -478,7 +597,8 @@ rb_source_can_browse (RBSource *source)
 }
 
 void
-rb_source_browser_toggled (RBSource *source, gboolean enabled)
+rb_source_browser_toggled (RBSource *source,
+			   gboolean enabled)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
@@ -499,7 +619,9 @@ rb_source_notify_filter_changed (RBSource *source)
 }
 
 void
-rb_source_update_play_statistics (RBSource *source, RhythmDB *db, RhythmDBEntry *entry)
+rb_source_update_play_statistics (RBSource *source,
+				  RhythmDB *db,
+				  RhythmDBEntry *entry)
 {
 	time_t now;
 	gulong current_count;
@@ -514,7 +636,7 @@ rb_source_update_play_statistics (RBSource *source, RhythmDB *db, RhythmDBEntry
 	/* Increment current play count */
 	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_PLAY_COUNT, &value);
 	g_value_unset (&value);
-	
+
 	/* Reset the last played time */
 	time (&now);
 
@@ -582,7 +704,8 @@ rb_source_can_search (RBSource *source)
 }
 
 void
-rb_source_search (RBSource *source, const char *text)
+rb_source_search (RBSource *source,
+		  const char *text)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
@@ -592,7 +715,8 @@ rb_source_search (RBSource *source, const char *text)
 }
 
 GtkWidget *
-rb_source_get_config_widget (RBSource *source, RBShellPreferences *prefs)
+rb_source_get_config_widget (RBSource *source,
+			     RBShellPreferences *prefs)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
@@ -666,7 +790,7 @@ default_copy (RBSource *source)
 {
 	return rb_entry_view_get_selected_entries (rb_source_get_entry_view (source));
 }
-	
+
 GList *
 rb_source_copy (RBSource *source)
 {
@@ -691,25 +815,28 @@ rb_source_can_add_to_queue (RBSource *source)
 }
 
 static void
-default_add_to_queue (RBSource *source, RBSource *queue)
+default_add_to_queue (RBSource *source,
+		      RBSource *queue)
 {
 	RBEntryView *songs = rb_source_get_entry_view (source);
 	GList *selection = rb_entry_view_get_selected_entries (songs);
 	GList *iter;
 
-	if (selection == NULL) 
+	if (selection == NULL)
 		return;
 
 	for (iter = selection; iter; iter = iter->next) {
-		rb_static_playlist_source_add_entry (RB_STATIC_PLAYLIST_SOURCE (queue), 
+		rb_static_playlist_source_add_entry (RB_STATIC_PLAYLIST_SOURCE (queue),
 						     (RhythmDBEntry *)iter->data, -1);
 	}
 
+	g_list_foreach (selection, (GFunc)rhythmdb_entry_unref, NULL);
 	g_list_free (selection);
 }
 
 void
-rb_source_add_to_queue (RBSource *source, RBSource *queue)
+rb_source_add_to_queue (RBSource *source,
+			RBSource *queue)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 	klass->impl_add_to_queue (source, queue);
@@ -731,18 +858,20 @@ default_move_to_trash (RBSource *source)
 	RhythmDB *db;
 	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (source);
 
-	g_object_get (G_OBJECT (priv->shell), "db", &db, NULL);
+	g_object_get (priv->shell, "db", &db, NULL);
 
 	entry_view = rb_source_get_entry_view (source);
 	sel = rb_entry_view_get_selected_entries (entry_view);
+
 	for (tem = sel; tem != NULL; tem = tem->next) {
 		rhythmdb_entry_move_to_trash (db, (RhythmDBEntry *)tem->data);
 		rhythmdb_commit (db);
 	}
-	g_list_free (sel);
-	g_object_unref (G_OBJECT (db));
-}
 
+	g_list_foreach (sel, (GFunc)rhythmdb_entry_unref, NULL);
+	g_list_free (sel);
+	g_object_unref (db);
+}
 
 void
 rb_source_move_to_trash (RBSource *source)
@@ -788,6 +917,24 @@ rb_source_try_playlist (RBSource *source)
 	return klass->impl_try_playlist (source);
 }
 
+guint
+rb_source_want_uri (RBSource *source, const char *uri)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+	if (klass->impl_want_uri)
+		return klass->impl_want_uri (source, uri);
+	return 0;
+}
+
+gboolean
+rb_source_add_uri (RBSource *source, const char *uri, const char *title, const char *genre)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+	if (klass->impl_add_uri)
+		return klass->impl_add_uri (source, uri, title, genre);
+	return FALSE;
+}
+
 gboolean
 rb_source_can_pause (RBSource *source)
 {
@@ -817,15 +964,8 @@ rb_source_handle_eos (RBSource *source)
 }
 
 gboolean
-rb_source_have_url (RBSource *source)
-{
-	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
-
-	return klass->impl_have_url (source);
-}
-
-gboolean
-rb_source_receive_drag (RBSource *source, GtkSelectionData *data)
+rb_source_receive_drag (RBSource *source,
+			GtkSelectionData *data)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
@@ -840,10 +980,10 @@ _rb_source_show_popup (RBSource *source, const char *ui_path)
 {
 	GtkUIManager *uimanager;
 
-	g_object_get (G_OBJECT (RB_SOURCE_GET_PRIVATE (source)->shell), 
+	g_object_get (RB_SOURCE_GET_PRIVATE (source)->shell,
 		      "ui-manager", &uimanager, NULL);
 	rb_gtk_action_popup_menu (uimanager, ui_path);
-	g_object_unref (G_OBJECT (uimanager));
+	g_object_unref (uimanager);
 
 }
 
@@ -861,7 +1001,6 @@ rb_source_show_popup (RBSource *source)
 	return klass->impl_show_popup (source);
 }
 
-
 static void
 default_delete_thyself (RBSource *source)
 {
@@ -938,41 +1077,47 @@ rb_source_get_search_actions (RBSource *source)
 	return klass->impl_get_search_actions (source);
 }
 
+static gboolean
+_update_status_idle (RBSource *source)
+{
+	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (source);
+
+	rb_source_notify_status_changed (source);
+
+	if (priv->hidden_when_empty)
+		update_visibility_idle (source);
+
+	priv->update_status_id = 0;
+	return FALSE;
+}
+
 static void
 rb_source_row_inserted_cb (GtkTreeModel *model,
 			   GtkTreePath *path,
 			   GtkTreeIter *iter,
 			   RBSource *source)
-{
-	rb_source_notify_status_changed (source);
-}
-
-static gboolean
-idle_emit_status_changed (RBSource *source)
 {
 	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (source);
-	rb_source_notify_status_changed (source);
-	priv->idle_status_changed_id = 0;
-	return FALSE;
+
+	if (priv->update_status_id == 0)
+		priv->update_status_id = g_idle_add ((GSourceFunc)_update_status_idle, source);
 }
 
 static void
-rb_source_row_deleted_cb (GtkTreeModel *model,
-			  GtkTreePath *path,
-			  RBSource *source)
+rb_source_post_entry_deleted_cb (GtkTreeModel *model,
+				 RhythmDBEntry *entry,
+				 RBSource *source)
 {
-	/* Emit the signal after the deletion has actually been processed,
-	 * since if we update the status now it'll show the old state.
-	 */
 	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (source);
-	if (priv->idle_status_changed_id == 0) {
-		priv->idle_status_changed_id = 
-			g_idle_add ((GSourceFunc) idle_emit_status_changed, source);
-	}
+
+	if (priv->update_status_id == 0)
+		priv->update_status_id = g_idle_add ((GSourceFunc)_update_status_idle, source);
 }
 
 static void
-rb_source_gather_hash_keys (char *key, gpointer unused, GList **data)
+rb_source_gather_hash_keys (char *key,
+			    gpointer unused,
+			    GList **data)
 {
 	*data = g_list_prepend (*data, key);
 }
@@ -986,14 +1131,16 @@ rb_source_gather_selected_properties (RBSource *source,
 
 	selected_set = g_hash_table_new (g_str_hash, g_str_equal);
 	selected = rb_entry_view_get_selected_entries (rb_source_get_entry_view (RB_SOURCE (source)));
+
 	for (tem = selected; tem; tem = tem->next) {
 		RhythmDBEntry *entry = tem->data;
 		char *val = g_strdup (rhythmdb_entry_get_string (entry, prop));
 		g_hash_table_insert (selected_set, val, NULL);
 	}
 
+	g_list_foreach (selected, (GFunc)rhythmdb_entry_unref, NULL);
 	g_list_free (selected);
-	
+
 	tem = NULL;
 	g_hash_table_foreach (selected_set, (GHFunc) rb_source_gather_hash_keys,
 			      &tem);
@@ -1013,17 +1160,21 @@ _rb_source_register_action_group (RBSource *source,
 	GList *i;
 	GtkActionGroup *group;
 
-	g_object_get (G_OBJECT (source), "ui-manager", &uimanager, NULL);
+	g_return_val_if_fail (group_name != NULL, NULL);
+
+	g_object_get (source, "ui-manager", &uimanager, NULL);
 	actiongroups = gtk_ui_manager_get_action_groups (uimanager);
 
 	/* Don't create the action group if it's already registered */
 	for (i = actiongroups; i != NULL; i = i->next) {
-		const gchar *name;
+		const char *name;
 
 		name = gtk_action_group_get_name (GTK_ACTION_GROUP (i->data));
-		if (strcmp (name, group_name) == 0) {
-			g_object_unref (G_OBJECT (uimanager));
-			return GTK_ACTION_GROUP (i->data);
+		if (name != NULL && strcmp (name, group_name) == 0) {
+			group = GTK_ACTION_GROUP (i->data);
+			/* Add a reference */
+			g_object_ref (group);
+			goto out;
 		}
 	}
 
@@ -1034,49 +1185,13 @@ _rb_source_register_action_group (RBSource *source,
 				      actions, num_actions,
 				      user_data);
 	gtk_ui_manager_insert_action_group (uimanager, group, 0);
-	g_object_unref (G_OBJECT (uimanager));
+
+ out:
+	g_object_unref (uimanager);
 
 	return group;
 }
 
-static void
-_autohide_update_visibility (RBSource *source, GtkTreeModel *model, int adjust)
-{
-	gint count = gtk_tree_model_iter_n_children (model, NULL) + adjust;
-	/* only update the property if it will have actually changed */
-	if (count < 2)
-		g_object_set (G_OBJECT (source), "visibility", (count > 0), NULL);
-}
-
-static void
-_rb_autohide_source_row_deleted_cb (GtkTreeModel *model,
-				    GtkTreePath *path,
-				    RBSource *source)
-{
-	_autohide_update_visibility (source, model, -1);
-}
-
-static void
-_rb_autohide_source_row_inserted_cb (GtkTreeModel *model,
-				     GtkTreePath *path,
-				     GtkTreeIter *iter,
-				     RBSource *source)
-{
-	_autohide_update_visibility (source, model, 0);
-}
-
-void
-_rb_source_hide_when_empty (RBSource *source,
-			    RhythmDBQueryModel *model)
-{
-	g_signal_connect_object (G_OBJECT (model), "row-deleted",
-				 G_CALLBACK (_rb_autohide_source_row_deleted_cb),
-				 source, 0);
-	g_signal_connect_object (G_OBJECT (model), "row-inserted",
-				 G_CALLBACK (_rb_autohide_source_row_inserted_cb),
-				 source, 0);
-}
-
 /* This should really be standard. */
 #define ENUM_ENTRY(NAME, DESC) { NAME, "" #NAME "", DESC }
 
