@@ -50,13 +50,14 @@ static void impl_browser_toggled (RBSource *source, gboolean enabled);
 static void impl_reset_filters (RBSource *asource);
 static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
 static GList *impl_get_search_actions (RBSource *source);
+static guint impl_want_uri (RBSource *source, const char *uri);
+
 static GPtrArray *construct_query_from_selection (RBStaticPlaylistSource *source);
 
 /* playlist methods */
 static void impl_save_contents_to_xml (RBPlaylistSource *source,
 				       xmlNodePtr node);
 
-
 /* browser stuff */
 static GList *impl_get_property_views (RBSource *source);
 void rb_static_playlist_source_browser_views_activated_cb (GtkWidget *widget,
@@ -95,7 +96,6 @@ G_DEFINE_TYPE (RBStaticPlaylistSource, rb_static_playlist_source, RB_TYPE_PLAYLI
 								RB_TYPE_STATIC_PLAYLIST_SOURCE, \
 								RBStaticPlaylistSourcePrivate))
 
-
 typedef struct
 {
 	RhythmDBQueryModel *base_model;
@@ -109,18 +109,18 @@ typedef struct
 
 	GtkActionGroup *action_group;
 	RhythmDBPropType search_prop;
+	gboolean dispose_has_run;
 } RBStaticPlaylistSourcePrivate;
 
+static gpointer playlist_pixbuf = NULL;
 
-	
 static void
 rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 {
-	gint size;
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 	RBPlaylistSourceClass *playlist_class = RB_PLAYLIST_SOURCE_CLASS (klass);
-	
+
 	object_class->constructor = rb_static_playlist_source_constructor;
 	object_class->dispose = rb_static_playlist_source_dispose;
 	object_class->finalize = rb_static_playlist_source_finalize;
@@ -139,38 +139,61 @@ rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 	source_class->impl_browser_toggled = impl_browser_toggled;
 	source_class->impl_get_property_views = impl_get_property_views;
 	source_class->impl_get_search_actions = impl_get_search_actions;
+	source_class->impl_want_uri = impl_want_uri;
 
 	playlist_class->impl_save_contents_to_xml = impl_save_contents_to_xml;
-	
-	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
-	klass->pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-						  GNOME_MEDIA_PLAYLIST,
-						  size,
-						  0, NULL);
-	
+
 	g_type_class_add_private (klass, sizeof (RBStaticPlaylistSourcePrivate));
 }
 
 static void
 rb_static_playlist_source_init (RBStaticPlaylistSource *source)
 {
-	RBStaticPlaylistSourceClass *klass = RB_STATIC_PLAYLIST_SOURCE_GET_CLASS (source);
+	if (playlist_pixbuf == NULL) {
+		gint size;
+		gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+		playlist_pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+							    GNOME_MEDIA_PLAYLIST,
+							    size,
+							    0, NULL);
+		if (playlist_pixbuf) {
+			g_object_add_weak_pointer (playlist_pixbuf,
+					   (gpointer *) &playlist_pixbuf);
 
-	rb_source_set_pixbuf (RB_SOURCE (source), klass->pixbuf);
+			rb_source_set_pixbuf (RB_SOURCE (source), playlist_pixbuf);
+
+			/* drop the initial reference to the icon */
+			g_object_unref (playlist_pixbuf);
+		}
+	} else {
+		rb_source_set_pixbuf (RB_SOURCE (source), playlist_pixbuf);
+	}
 }
 
 static void
 rb_static_playlist_source_dispose (GObject *object)
 {
 	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (object);
-	if (!priv)
-		return;
 
-	if (priv->base_model) {
-		g_object_unref (G_OBJECT (priv->base_model));
+	if (priv->dispose_has_run) {
+		/* If dispose did already run, return. */
+		rb_debug ("Dispose has already run for static playlist source %p", object);
+		return;
+	}
+	/* Make sure dispose does not run twice. */
+	priv->dispose_has_run = TRUE;
+
+	rb_debug ("Disposing static playlist source %p", object);
+
+	if (priv->base_model != NULL) {
+		g_object_unref (priv->base_model);
 		priv->base_model = NULL;
 	}
 
+	if (priv->filter_model != NULL) {
+		g_object_unref (priv->filter_model);
+	}
+
 	G_OBJECT_CLASS (rb_static_playlist_source_parent_class)->dispose (object);
 }
 
@@ -179,13 +202,20 @@ rb_static_playlist_source_finalize (GObject *object)
 {
 	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (object);
 
+	rb_debug ("Finalizing static playlist source %p", object);
+
 	g_free (priv->search_text);
 
+	if (priv->action_group != NULL) {
+		g_object_unref (priv->action_group);
+	}
+
 	G_OBJECT_CLASS (rb_static_playlist_source_parent_class)->finalize (object);
 }
 
 static GObject *
-rb_static_playlist_source_constructor (GType type, guint n_construct_properties,
+rb_static_playlist_source_constructor (GType type,
+				       guint n_construct_properties,
 				       GObjectConstructParam *construct_properties)
 {
 	GObjectClass *parent_class = G_OBJECT_CLASS (rb_static_playlist_source_parent_class);
@@ -198,12 +228,12 @@ rb_static_playlist_source_constructor (GType type, guint n_construct_properties,
 	RhythmDBEntryType entry_type;
 
 	priv->base_model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (psource));
-	g_object_set (G_OBJECT (priv->base_model), "show-hidden", TRUE, NULL);
-	g_object_ref (G_OBJECT (priv->base_model));
+	g_object_set (priv->base_model, "show-hidden", TRUE, NULL);
+	g_object_ref (priv->base_model);
 
 	priv->paned = gtk_vpaned_new ();
 
-	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
+	g_object_get (source, "shell", &shell, NULL);
 	priv->action_group = _rb_source_register_action_group (RB_SOURCE (source),
 							       "StaticPlaylistActions",
 							       NULL, 0,
@@ -215,33 +245,36 @@ rb_static_playlist_source_constructor (GType type, guint n_construct_properties,
 					    (GCallback)search_action_changed,
 					    shell);
 	priv->search_prop = RHYTHMDB_PROP_SEARCH_MATCH;
-	g_object_unref (G_OBJECT (shell));
 
-	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
-	priv->browser = rb_library_browser_new (rb_playlist_source_get_db (RB_PLAYLIST_SOURCE (source)), 
+	g_object_unref (shell);
+
+	g_object_get (source, "entry-type", &entry_type, NULL);
+	priv->browser = rb_library_browser_new (rb_playlist_source_get_db (RB_PLAYLIST_SOURCE (source)),
 						entry_type);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 	gtk_paned_pack1 (GTK_PANED (priv->paned), GTK_WIDGET (priv->browser), TRUE, FALSE);
-	g_signal_connect_object (G_OBJECT (priv->browser), "notify::output-model",
+	g_signal_connect_object (priv->browser, "notify::output-model",
 				 G_CALLBACK (rb_static_playlist_source_browser_changed_cb),
 				 source, 0);
 
 	rb_library_browser_set_model (priv->browser, priv->base_model, FALSE);
 	rb_static_playlist_source_do_query (source);
-			 
+
 	/* reparent the entry view */
 	songs = rb_source_get_entry_view (RB_SOURCE (source));
-	g_object_ref (G_OBJECT (songs));
+	g_object_ref (songs);
 	gtk_container_remove (GTK_CONTAINER (source), GTK_WIDGET (songs));
 	gtk_paned_pack2 (GTK_PANED (priv->paned), GTK_WIDGET (songs), TRUE, FALSE);
 	gtk_container_add (GTK_CONTAINER (source), priv->paned);
+	g_object_unref (songs);
 
 	/* watch these to find out when things are dropped into the entry view */
-	g_signal_connect_object (G_OBJECT (priv->base_model), "row-inserted",
-			 G_CALLBACK (rb_static_playlist_source_row_inserted),
-			 source, 0);
-	g_signal_connect_object (G_OBJECT (priv->base_model), "non-entry-dropped",
-			 G_CALLBACK (rb_static_playlist_source_non_entry_dropped),
-			 source, 0);
+	g_signal_connect_object (priv->base_model, "row-inserted",
+				 G_CALLBACK (rb_static_playlist_source_row_inserted),
+				 source, 0);
+	g_signal_connect_object (priv->base_model, "non-entry-dropped",
+				 G_CALLBACK (rb_static_playlist_source_non_entry_dropped),
+				 source, 0);
 
 	gtk_widget_show_all (GTK_WIDGET (source));
 
@@ -273,18 +306,18 @@ rb_static_playlist_source_load_from_xml (RBStaticPlaylistSource *source, xmlNode
 
 		if (xmlNodeIsText (child))
 			continue;
-	
+
 		if (xmlStrcmp (child->name, RB_PLAYLIST_LOCATION))
 			continue;
-	
+
 		location = xmlNodeGetContent (child);
 		rb_static_playlist_source_add_location (source,
 						        (char *) location, -1);
+		xmlFree (location);
 	}
 }
 
-
-RBSource *	
+RBSource *
 rb_static_playlist_source_new_from_xml (RBShell *shell, xmlNodePtr node)
 {
 	RBSource *psource = rb_static_playlist_source_new (shell,
@@ -332,6 +365,7 @@ impl_delete (RBSource *asource)
 	for (tem = sel; tem != NULL; tem = tem->next) {
 		rb_static_playlist_source_remove_entry (source, (RhythmDBEntry *) tem->data);
 	}
+	g_list_foreach (sel, (GFunc)rhythmdb_entry_unref, NULL);
 	g_list_free (sel);
 }
 
@@ -340,7 +374,7 @@ impl_reset_filters (RBSource *source)
 {
 	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
 	gboolean changed = FALSE;
-	
+
 	if (rb_library_browser_reset (priv->browser))
 		changed = TRUE;
 
@@ -378,7 +412,7 @@ impl_search (RBSource *source, const char *search_text)
 		priv->search_text = g_strdup (search_text);
 	}
 	g_free (old_search_text);
-	
+
 	rb_debug ("doing search for \"%s\"", priv->search_text ? priv->search_text : "(NULL)");
 
 	rb_static_playlist_source_do_query (RB_STATIC_PLAYLIST_SOURCE (source));
@@ -394,7 +428,6 @@ impl_get_property_views (RBSource *source)
 	return ret;
 }
 
-
 static void
 impl_browser_toggled (RBSource *source, gboolean enabled)
 {
@@ -435,14 +468,15 @@ rb_static_playlist_source_do_query (RBStaticPlaylistSource *source)
 	RBPlaylistSource *psource = RB_PLAYLIST_SOURCE (source);
 	RhythmDB *db = rb_playlist_source_get_db (psource);
 	GPtrArray *query;
-	
-	if (priv->filter_model)
+
+	if (priv->filter_model != NULL) {
 		g_object_unref (priv->filter_model);
+	}
 	priv->filter_model = rhythmdb_query_model_new_empty (db);
-	g_object_set (G_OBJECT (priv->filter_model), "base-model", priv->base_model, NULL);
+	g_object_set (priv->filter_model, "base-model", priv->base_model, NULL);
 
 	query = construct_query_from_selection (source);
-	g_object_set (G_OBJECT (priv->filter_model), "query", query, NULL);
+	g_object_set (priv->filter_model, "query", query, NULL);
 	rhythmdb_query_free (query);
 
 	rhythmdb_query_model_reapply_query (priv->filter_model, TRUE);
@@ -450,18 +484,18 @@ rb_static_playlist_source_do_query (RBStaticPlaylistSource *source)
 }
 
 static void
-rb_static_playlist_source_browser_changed_cb (RBLibraryBrowser *browser, 
+rb_static_playlist_source_browser_changed_cb (RBLibraryBrowser *browser,
 					      GParamSpec *pspec,
 					      RBStaticPlaylistSource *source)
 {
 	RBEntryView *songs = rb_source_get_entry_view (RB_SOURCE (source));
 	RhythmDBQueryModel *query_model;
-	
-	g_object_get (G_OBJECT (browser), "output-model", &query_model, NULL);
+
+	g_object_get (browser, "output-model", &query_model, NULL);
 	rb_entry_view_set_model (songs, query_model);
 	rb_playlist_source_set_query_model (RB_PLAYLIST_SOURCE (source), query_model);
-	g_object_unref (G_OBJECT (query_model));
-	
+	g_object_unref (query_model);
+
 	rb_source_notify_filter_changed (RB_SOURCE (source));
 }
 
@@ -475,14 +509,14 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 		list = rb_uri_list_parse ((char *)data->data);
 		if (list != NULL) {
 			rb_static_playlist_source_add_list_uri (source, list);
-		} else 
+		} else
 			return FALSE;
 	}
 
         return TRUE;
 }
 
-static void 
+static void
 impl_save_contents_to_xml (RBPlaylistSource *source,
 			   xmlNodePtr node)
 {
@@ -490,11 +524,11 @@ impl_save_contents_to_xml (RBPlaylistSource *source,
 	GtkTreeIter iter;
 
 	xmlSetProp (node, RB_PLAYLIST_TYPE, RB_PLAYLIST_STATIC);
-	
+
 	if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->base_model), &iter))
 		return;
 
-	do { 
+	do {
 		xmlNodePtr child_node = xmlNewChild (node, NULL, RB_PLAYLIST_LOCATION, NULL);
 		RhythmDBEntry *entry;
 		xmlChar *encoded;
@@ -506,7 +540,9 @@ impl_save_contents_to_xml (RBPlaylistSource *source,
 		encoded = xmlEncodeEntitiesReentrant (NULL, BAD_CAST location);
 
 		xmlNodeSetContent (child_node, encoded);
+
 		g_free (encoded);
+		rhythmdb_entry_unref (entry);
 	} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->base_model), &iter));
 }
 
@@ -563,16 +599,19 @@ rb_static_playlist_source_add_location_internal (RBStaticPlaylistSource *source,
 		RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
 		RhythmDBEntryType entry_type;
 
-		g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
+		g_object_get (source, "entry-type", &entry_type, NULL);
 		if (entry_type != RHYTHMDB_ENTRY_TYPE_INVALID &&
 		    rhythmdb_entry_get_entry_type (entry) != entry_type) {
+			g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 			rb_debug ("attempting to add an entry of the wrong type to playlist");
 			return;
 		}
 
-		rhythmdb_entry_ref (db, entry);
+		rhythmdb_entry_ref (entry);
 		rhythmdb_query_model_add_entry (priv->base_model, entry, index);
-		rhythmdb_entry_unref (db, entry);
+		rhythmdb_entry_unref (entry);
+
+		g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 	}
 
 	rb_playlist_source_add_to_map (psource, location);
@@ -581,13 +620,12 @@ rb_static_playlist_source_add_location_internal (RBStaticPlaylistSource *source,
 }
 
 static void
-rb_static_playlist_source_add_location_swapped (const char *uri, 
+rb_static_playlist_source_add_location_swapped (const char *uri,
 						RBStaticPlaylistSource *source)
 {
 	rb_static_playlist_source_add_location_internal (source, uri, -1);
 }
 
-
 void
 rb_static_playlist_source_add_location (RBStaticPlaylistSource *source,
 					const char *location,
@@ -677,7 +715,6 @@ rb_static_playlist_source_move_entry (RBStaticPlaylistSource *source,
 	rb_playlist_source_mark_dirty (psource);
 }
 
-
 static void
 rb_static_playlist_source_non_entry_dropped (GtkTreeModel *model,
 					     const char *uri,
@@ -701,6 +738,8 @@ rb_static_playlist_source_row_inserted (GtkTreeModel *model,
 	gtk_tree_model_get (model, iter, 0, &entry, -1);
 
 	rb_static_playlist_source_add_entry (source, entry, -1);
+
+	rhythmdb_entry_unref (entry);
 }
 
 static GList *
@@ -727,7 +766,7 @@ search_action_to_prop (GtkAction *action)
 	if (name == NULL) {
 		prop = RHYTHMDB_PROP_SEARCH_MATCH;
 	} else if (strcmp (name, "StaticPlaylistSearchAll") == 0) {
-		prop = RHYTHMDB_PROP_SEARCH_MATCH;		
+		prop = RHYTHMDB_PROP_SEARCH_MATCH;
 	} else if (strcmp (name, "StaticPlaylistSearchArtists") == 0) {
 		prop = RHYTHMDB_PROP_ARTIST_FOLDED;
 	} else if (strcmp (name, "StaticPlaylistSearchAlbums") == 0) {
@@ -750,7 +789,7 @@ search_action_changed (GtkRadioAction  *action,
 	RBStaticPlaylistSourcePrivate *priv;
 	gboolean active;
 
-	g_object_get (G_OBJECT (shell), "selected-source", &source, NULL);
+	g_object_get (shell, "selected-source", &source, NULL);
 	priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
 
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (current));
@@ -761,4 +800,20 @@ search_action_changed (GtkRadioAction  *action,
 		rb_static_playlist_source_do_query (source);
 		rb_source_notify_filter_changed (RB_SOURCE (source));
 	}
+
+	if (source != NULL) {
+		g_object_unref (source);
+	}
+}
+
+static guint
+impl_want_uri (RBSource *source, const char *uri)
+{
+	/* take anything local, on smb, or sftp */
+	if (rb_uri_is_local (uri) ||
+	    g_str_has_prefix (uri, "smb://") ||
+	    g_str_has_prefix (uri, "sftp://"))
+		return 25;	/* less than what the library returns */
+
+	return 0;
 }
