@@ -20,7 +20,7 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -132,7 +132,6 @@ static const GtkTargetEntry targets_artist [] = {
 
 G_DEFINE_TYPE (RBPropertyView, rb_property_view, GTK_TYPE_SCROLLED_WINDOW)
 
-
 static void
 rb_property_view_class_init (RBPropertyViewClass *klass)
 {
@@ -235,7 +234,6 @@ rb_property_view_class_init (RBPropertyViewClass *klass)
 			      G_TYPE_NONE,
 			      0);
 
-
 	g_type_class_add_private (klass, sizeof (RBPropertyViewPrivate));
 }
 
@@ -245,6 +243,56 @@ rb_property_view_init (RBPropertyView *view)
 	view->priv = RB_PROPERTY_VIEW_GET_PRIVATE (view);
 }
 
+static void
+rb_property_view_set_model_internal (RBPropertyView *view,
+				     RhythmDBPropertyModel *model)
+{
+	if (view->priv->prop_model != NULL) {
+		g_signal_handlers_disconnect_by_func (view->priv->prop_model,
+						      G_CALLBACK (rb_property_view_pre_row_deleted_cb),
+						      view);
+		g_signal_handlers_disconnect_by_func (view->priv->prop_model,
+						      G_CALLBACK (rb_property_view_post_row_deleted_cb),
+						      view);
+		g_object_unref (view->priv->prop_model);
+	}
+
+	view->priv->prop_model = model;
+
+	if (view->priv->prop_model != NULL) {
+		GtkTreeIter iter;
+
+		g_object_ref (view->priv->prop_model);
+
+		gtk_tree_view_set_model (GTK_TREE_VIEW (view->priv->treeview),
+					 GTK_TREE_MODEL (view->priv->prop_model));
+
+		g_signal_connect_object (view->priv->prop_model,
+					 "pre-row-deletion",
+					 G_CALLBACK (rb_property_view_pre_row_deleted_cb),
+					 view,
+					 0);
+		g_signal_connect_object (view->priv->prop_model,
+					 "row_deleted",
+					 G_CALLBACK (rb_property_view_post_row_deleted_cb),
+					 view,
+					 G_CONNECT_AFTER);
+
+		g_signal_handlers_block_by_func (view->priv->selection,
+						 G_CALLBACK (rb_property_view_selection_changed_cb),
+						 view);
+
+		gtk_tree_selection_unselect_all (view->priv->selection);
+
+		if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (view->priv->prop_model), &iter))
+			gtk_tree_selection_select_iter (view->priv->selection, &iter);
+
+		g_signal_handlers_unblock_by_func (view->priv->selection,
+						   G_CALLBACK (rb_property_view_selection_changed_cb),
+						   view);
+	}
+}
+
 static void
 rb_property_view_finalize (GObject *object)
 {
@@ -257,12 +305,13 @@ rb_property_view_finalize (GObject *object)
 
 	g_return_if_fail (view->priv != NULL);
 
+	rb_property_view_set_model_internal (view, NULL);
+
 	g_free (view->priv->title);
 
 	G_OBJECT_CLASS (rb_property_view_parent_class)->finalize (object);
 }
 
-
 static void
 rb_property_view_set_property (GObject *object,
 			       guint prop_id,
@@ -271,8 +320,7 @@ rb_property_view_set_property (GObject *object,
 {
 	RBPropertyView *view = RB_PROPERTY_VIEW (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_DB:
 		view->priv->db = g_value_get_object (value);
 		break;
@@ -283,51 +331,7 @@ rb_property_view_set_property (GObject *object,
 		view->priv->title = g_value_dup_string (value);
 		break;
 	case PROP_MODEL:
-	{
-		GtkTreeIter iter;
-
-		if (view->priv->prop_model) {
-			g_signal_handlers_disconnect_by_func (G_OBJECT (view->priv->prop_model),
-							      G_CALLBACK (rb_property_view_pre_row_deleted_cb),
-							      view);
-			g_signal_handlers_disconnect_by_func (G_OBJECT (view->priv->prop_model),
-							      G_CALLBACK (rb_property_view_post_row_deleted_cb),
-							      view);
-			g_object_unref (G_OBJECT (view->priv->prop_model));
-		}
-
-		view->priv->prop_model = g_value_get_object (value);
-
-		if (!view->priv->prop_model)
-			break;
-
-		gtk_tree_view_set_model (GTK_TREE_VIEW (view->priv->treeview),
-					 GTK_TREE_MODEL (view->priv->prop_model));
-
-		g_object_ref (G_OBJECT (view->priv->prop_model));
-		g_signal_connect_object (G_OBJECT (view->priv->prop_model),
-					 "pre-row-deletion",
-					 G_CALLBACK (rb_property_view_pre_row_deleted_cb),
-					 view,
-					 0);
-		g_signal_connect_object (G_OBJECT (view->priv->prop_model),
-					 "row_deleted",
-					 G_CALLBACK (rb_property_view_post_row_deleted_cb),
-					 view,
-					 G_CONNECT_AFTER);
-
-		g_signal_handlers_block_by_func (G_OBJECT (view->priv->selection),
-						 G_CALLBACK (rb_property_view_selection_changed_cb),
-						 view);
-		
-		gtk_tree_selection_unselect_all (view->priv->selection);
-		if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (view->priv->prop_model), &iter))
-			gtk_tree_selection_select_iter (view->priv->selection, &iter);
-		g_signal_handlers_unblock_by_func (G_OBJECT (view->priv->selection),
-						   G_CALLBACK (rb_property_view_selection_changed_cb),
-						   view);
-
-	}
+		rb_property_view_set_model_internal (view, g_value_get_object (value));
 		break;
 	case PROP_DRAGGABLE:
 		view->priv->draggable = g_value_get_boolean (value);
@@ -338,7 +342,7 @@ rb_property_view_set_property (GObject *object,
 	}
 }
 
-static void 
+static void
 rb_property_view_get_property (GObject *object,
 			       guint prop_id,
 			       GValue *value,
@@ -346,8 +350,7 @@ rb_property_view_get_property (GObject *object,
 {
 	RBPropertyView *view = RB_PROPERTY_VIEW (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_DB:
 		g_value_set_object (value, view->priv->db);
 		break;
@@ -397,7 +400,9 @@ void
 rb_property_view_set_selection_mode (RBPropertyView *view,
 				     GtkSelectionMode mode)
 {
+	g_return_if_fail (RB_IS_PROPERTY_VIEW (view));
 	g_return_if_fail (mode == GTK_SELECTION_SINGLE || mode == GTK_SELECTION_MULTIPLE);
+
 	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (view->priv->treeview)),
 				     mode);
 }
@@ -406,26 +411,34 @@ void
 rb_property_view_reset (RBPropertyView *view)
 {
 	RhythmDBPropertyModel *model;
-	
+
+	g_return_if_fail (RB_IS_PROPERTY_VIEW (view));
+
 	model = rhythmdb_property_model_new (view->priv->db, view->priv->propid);
 
-	g_object_set (G_OBJECT (view), "property-model", model, NULL);
-	g_object_unref (G_OBJECT (model));
+	rb_property_view_set_model_internal (view, model);
+	g_object_unref (model);
 }
 
 RhythmDBPropertyModel *
 rb_property_view_get_model (RBPropertyView *view)
 {
 	RhythmDBPropertyModel *model;
-	
-	g_object_get (G_OBJECT (view), "property-model", &model, NULL);
+
+	g_return_val_if_fail (RB_IS_PROPERTY_VIEW (view), NULL);
+
+	model = view->priv->prop_model;
+
 	return model;
 }
 
 void
-rb_property_view_set_model (RBPropertyView *view, RhythmDBPropertyModel *model)
+rb_property_view_set_model (RBPropertyView *view,
+			    RhythmDBPropertyModel *model)
 {
-	g_object_set (G_OBJECT (view), "property-model", model, NULL);
+	g_return_if_fail (RB_IS_PROPERTY_VIEW (view));
+
+	rb_property_view_set_model_internal (view, model);
 }
 
 static void
@@ -462,6 +475,8 @@ rb_property_view_post_row_deleted_cb (GtkTreeModel *model,
 guint
 rb_property_view_get_num_properties (RBPropertyView *view)
 {
+	g_return_val_if_fail (RB_IS_PROPERTY_VIEW (view), 0);
+
 	return gtk_tree_model_iter_n_children (GTK_TREE_MODEL (view->priv->prop_model),
 					       NULL)-1;
 }
@@ -546,7 +561,7 @@ rb_property_view_constructor (GType type,
 			         G_CALLBACK (rb_property_view_row_activated_cb),
 			         view,
 				 0);
-	
+
 	view->priv->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view->priv->treeview));
 	g_signal_connect_object (G_OBJECT (view->priv->selection),
 			         "changed",
@@ -558,19 +573,18 @@ rb_property_view_constructor (GType type,
 				 G_CALLBACK (rb_property_view_popup_menu_cb),
 				 view,
 				 0);
-	
+
 	g_signal_connect_object (G_OBJECT (view->priv->treeview),
 			         "button_press_event",
 			         G_CALLBACK (rb_property_view_button_press_cb),
 			         view,
 				 0);
 
-
 	gtk_container_add (GTK_CONTAINER (view), view->priv->treeview);
 
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (view->priv->treeview), TRUE);
 	gtk_tree_selection_set_mode (view->priv->selection, GTK_SELECTION_SINGLE);
-	
+
 	column = gtk_tree_view_column_new ();
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
@@ -592,7 +606,7 @@ rb_property_view_row_activated_cb (GtkTreeView *treeview,
 				   RBPropertyView *view)
 {
 	GtkTreeIter iter;
-	const char *val;
+	char *val;
 	gboolean is_all;
 
 	rb_debug ("row activated");
@@ -606,18 +620,23 @@ rb_property_view_row_activated_cb (GtkTreeView *treeview,
 	rb_debug ("emitting property activated");
 	g_signal_emit (G_OBJECT (view), rb_property_view_signals[PROPERTY_ACTIVATED], 0,
 		       is_all ? NULL : val);
+
+	g_free (val);
 }
 
 void
-rb_property_view_set_selection (RBPropertyView *view, const GList *vals)
+rb_property_view_set_selection (RBPropertyView *view,
+				const GList *vals)
 {
+	g_return_if_fail (RB_IS_PROPERTY_VIEW (view));
+
 	view->priv->handling_row_deletion = TRUE;
-	
+
 	gtk_tree_selection_unselect_all (view->priv->selection);
-	
+
 	for (; vals ; vals = vals->next) {
 		GtkTreeIter iter;
-		
+
 		if (rhythmdb_property_model_iter_from_string (view->priv->prop_model, vals->data, &iter)) {
 			GtkTreePath *path;
 
@@ -632,7 +651,7 @@ rb_property_view_set_selection (RBPropertyView *view, const GList *vals)
 
 		}
 	}
-		
+
 	view->priv->handling_row_deletion = FALSE;
 	rb_property_view_selection_changed_cb (view->priv->selection, view);
 }
@@ -653,7 +672,7 @@ rb_property_view_selection_changed_cb (GtkTreeSelection *selection,
 	if (gtk_tree_selection_get_mode (selection) == GTK_SELECTION_MULTIPLE) {
 		GList *selected_rows, *tem;
 		GList *selected_properties = NULL;
-		
+
 		selected_rows = gtk_tree_selection_get_selected_rows (view->priv->selection, &model);
 		for (tem = selected_rows; tem; tem = tem->next) {
 			g_assert (gtk_tree_model_get_iter (model, &iter, tem->data));
@@ -699,7 +718,7 @@ rb_property_view_selection_changed_cb (GtkTreeSelection *selection,
 	g_free (selected_prop);
 }
 
-static gboolean 
+static gboolean
 rb_property_view_popup_menu_cb (GtkTreeView *treeview,
 				RBPropertyView *view)
 {
@@ -707,10 +726,12 @@ rb_property_view_popup_menu_cb (GtkTreeView *treeview,
 	return TRUE;
 }
 
-void		
+void
 rb_property_view_append_column_custom (RBPropertyView *view,
 				       GtkTreeViewColumn *column)
 {
+	g_return_if_fail (RB_IS_PROPERTY_VIEW (view));
+
 	gtk_tree_view_append_column (GTK_TREE_VIEW (view->priv->treeview), column);
 }
 
@@ -732,14 +753,16 @@ rb_property_view_button_press_cb (GtkTreeView *tree,
 		} else {
 			GtkTreeModel *model;
 			GtkTreeIter iter;
-			const char *val;
+			char *val;
 			GList *lst = NULL;
-			
+
 			model = gtk_tree_view_get_model (GTK_TREE_VIEW (view->priv->treeview));
 			if (gtk_tree_model_get_iter (model, &iter, path)) {
-				gtk_tree_model_get (model, &iter, 0, &val, -1);
+				gtk_tree_model_get (model, &iter,
+						    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &val, -1);
 				lst = g_list_prepend (lst, (gpointer) val);
 				rb_property_view_set_selection (view, lst);
+				g_free (val);
 			}
 		}
 		g_signal_emit (G_OBJECT (view), rb_property_view_signals[SHOW_POPUP], 0);
@@ -755,9 +778,9 @@ rb_property_view_set_search_func (RBPropertyView *view,
 				  gpointer func_data,
 				  GtkDestroyNotify notify)
 {
+	g_return_if_fail (RB_IS_PROPERTY_VIEW (view));
+
 	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (view->priv->treeview),
 					     func, func_data,
 					     notify);
 }
-
-
