@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -50,6 +50,8 @@ struct RBSearchEntryPrivate
 	gboolean clearing;
 
 	guint timeout;
+
+	gboolean is_a11y_theme;
 };
 
 G_DEFINE_TYPE (RBSearchEntry, rb_search_entry, GTK_TYPE_HBOX)
@@ -64,7 +66,6 @@ enum
 
 static guint rb_search_entry_signals[LAST_SIGNAL] = { 0 };
 
-
 static void
 rb_search_entry_class_init (RBSearchEntryClass *klass)
 {
@@ -99,9 +100,17 @@ static void
 rb_search_entry_init (RBSearchEntry *entry)
 {
 	GtkWidget *label;
+	GtkSettings *settings;
+	char *theme;
 
 	entry->priv = RB_SEARCH_ENTRY_GET_PRIVATE (entry);
 
+	settings = gtk_settings_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (entry)));
+	g_object_get (settings, "gtk-theme-name", &theme, NULL);
+	entry->priv->is_a11y_theme = strncmp (theme, "HighContrast", strlen ("HighContrast")) == 0 ||
+					strncmp (theme, "LowContrast", strlen ("LowContrast")) == 0;
+	g_free (theme);
+
 	/* this string can only be so long, or there wont be a search entry :) */
 	label = gtk_label_new_with_mnemonic (_("_Search:"));
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
@@ -165,9 +174,9 @@ rb_search_entry_clear (RBSearchEntry *entry)
 		g_source_remove (entry->priv->timeout);
 		entry->priv->timeout = 0;
 	}
-		
+
 	entry->priv->clearing = TRUE;
-	
+
 	gtk_entry_set_text (GTK_ENTRY (entry->priv->entry), "");
 
 	entry->priv->clearing = FALSE;
@@ -180,6 +189,32 @@ rb_search_entry_set_text (RBSearchEntry *entry, const char *text)
 			    text ? text : "");
 }
 
+static void
+rb_search_entry_check_style (RBSearchEntry *entry)
+{
+	/*GdkColor fg_colour;*/
+	GdkColor bg_colour;
+	static const GdkColor fallback_bg_colour = { 0, 0xf7f7, 0xf7f7, 0xbebe }; /* yellow-ish */
+	const gchar* text;
+
+	if (entry->priv->is_a11y_theme)
+		return;
+
+	/*fg_colour = GTK_WIDGET(entry)->style->text[GTK_STATE_NORMAL];*/
+	bg_colour = fallback_bg_colour;
+
+	text = gtk_entry_get_text (GTK_ENTRY (entry->priv->entry));
+	if (text && *text) {
+		/*gtk_widget_modify_text (entry->priv->entry, GTK_STATE_NORMAL, &fg_colour);*/
+		gtk_widget_modify_base (entry->priv->entry, GTK_STATE_NORMAL, &bg_colour);
+	} else {
+		/*gtk_widget_modify_text (entry->priv->entry, GTK_STATE_NORMAL, NULL);*/
+		gtk_widget_modify_base (entry->priv->entry, GTK_STATE_NORMAL, NULL);
+	}
+
+	gtk_widget_queue_draw (GTK_WIDGET (entry));
+}
+
 static void
 rb_search_entry_changed_cb (GtkEditable *editable,
 			    RBSearchEntry *entry)
@@ -192,6 +227,8 @@ rb_search_entry_changed_cb (GtkEditable *editable,
 		entry->priv->timeout = 0;
 	}
 
+	rb_search_entry_check_style (entry);
+
 	/* emit it now if we're clearing the entry */
 	if (gtk_entry_get_text (GTK_ENTRY (entry->priv->entry)))
 		entry->priv->timeout = g_timeout_add (300, (GSourceFunc) rb_search_entry_timeout_cb, entry);
@@ -203,7 +240,7 @@ static gboolean
 rb_search_entry_timeout_cb (RBSearchEntry *entry)
 {
 	gdk_threads_enter ();
-	
+
 	g_signal_emit (G_OBJECT (entry), rb_search_entry_signals[SEARCH], 0,
 		       gtk_entry_get_text (GTK_ENTRY (entry->priv->entry)));
 	entry->priv->timeout = 0;
